package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.Arrays;
import java.util.List;


/**
 * <p>
 * Base of the Activity_2 sensor profile.
 * </p>
 * 
 * @author Monnit Corp
 * @version 0.8.1.1, 18 February 2016
 */
public class Activity_2Base
  extends _ApplicationBase
{
  public static long MonnitApplicationID = 19L;
  public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
  public int Count;
  public List<Datum> Data;
  
  public List<Datum> getData()
  {
    return Arrays.asList(new Datum[] { new Datum("Count", Integer.valueOf(this.Count)) });
  }
  
  public static Activity_2Base Create(int state, byte[] rawData)
    throws Exception
  {
    Activity_2Base obj = new Activity_2Base();
    obj.Count = BitConverter.ToUInt16(rawData, 0);
    return obj;
  }
  
  public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
  public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
  public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
  public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
  public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
  public static long DefaultHysteresis = 2147483647L;
  public static long DefaultMinimumThreshold = _ApplicationBase.DefaultMinimumThreshold;
  public static long DefaultMaximumThreshold = _ApplicationBase.DefaultMaximumThreshold;
  public static long DefaultCalibration1 = _ApplicationBase.DefaultCalibration1;
  public static long DefaultCalibration2 = _ApplicationBase.DefaultCalibration2;
  public static long DefaultCalibration3 = _ApplicationBase.DefaultCalibration3;
  public static long DefaultCalibration4 = _ApplicationBase.DefaultCalibration4;
  public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
  public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
  public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
  public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
  public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
  
  
  /**
   * <p>
   * sets the ReportInterval as minutes on the sensor object<br/>
   * the next time this sensor checks in it will update
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorHeartBeat(Sensor sens,double heartbeat,Boolean sendSensorUpdate)
  {	
	  sens.ReportInterval = heartbeat;
	 
	  if(sens.ReportInterval < sens.getActiveStateInterval())
		  sens.setActiveStateInterval(sens.ReportInterval);
	  
	  sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  
  /**
   * <p>
   * sets all data for the GeneralConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
   * @param activeStateInterval a double value this is the aware state heart beat represented in minutes so aware state heart beat equals 1 that is one minute.
   * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateInterval,int transmitIntervalLink,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval((activeStateInterval));
	  	
	  	if(sens.ReportInterval < sens.getActiveStateInterval())
			  sens.setActiveStateInterval(sens.ReportInterval);
	  	
	  	if(transmitIntervalLink < 0)
	  		transmitIntervalLink = 0;
	  	else if (transmitIntervalLink > 250)
	  		transmitIntervalLink=250;
	  				
	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
  { 	
	  
	  if(recovery < 0)
		  recovery = 0;
		
		if (recovery > 10)
			recovery = 10;
	  	sens.Recovery = recovery;
	    sens.GeneralConfig2Dirty = sendSensorUpdate;
  }
}
