package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class CTPowerBase extends _ApplicationBase
{
    public static long MonnitApplicationID = 81;
    public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
    public double Power;
    public double AvgPower;
    public double Volts;
    public double Current;
    public double Freq;
    public double PowerFactor;
    
 public List<Datum> Data;
    
    public List<Datum> getData()
    {
    	  return Arrays.asList(new Datum[] { new Datum("Power Usage", Power), new Datum("AvgPower", AvgPower), new Datum("Volts", Volts), new Datum("Current", Current), new Datum("Freq", Freq), new Datum("Power Factor", PowerFactor)});
    	  
    }
    
   
    public static CTPowerBase Create(int state, byte[] rawData) throws Exception
    {
        CTPowerBase obj = new CTPowerBase();
        obj.Power = (BitConverter.ToUInt32(rawData, 1) / 1000d);
        obj.AvgPower = (BitConverter.toInt32(rawData, 5) / 1000d);
        obj.Volts = (BitConverter.ToUInt16(rawData, 9) / 10d);
        obj.Current = (BitConverter.ToUInt16(rawData, 11) / 100d);
        obj.Freq = rawData[13];
        obj.PowerFactor = (rawData[14] / 100d);
        return obj;
    }
    
    public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
    public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
    public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
    public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
    public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
    public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
    public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
    public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
    public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
    public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
    public static long DefaultHysteresis = 0x00007D01;
    public static long DefaultMinimumThreshold = 0xFFFFFFFF;
    public static long DefaultMaximumThreshold = 0xFFFFFFFF;
    public static long DefaultCalibration1 = 219650;
    public static long DefaultCalibration2 = 1619;
    public static long DefaultCalibration3 = 0x073c0096;
    public static long DefaultCalibration4 = 0x03FF03FFL;
    public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
    public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
    public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
    public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
    public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
    
    public static List<byte[]> CalibrateFrame(long sensorID, int acc, int channel, double voltage,double current)
    {
    	
    	 List<byte[]> commandList = new ArrayList();
        int length = 0;
        //Calibrate Temp
        switch (acc)
        {
            case 4: // Zero Voltage

                length = 6;

                byte[] api = new byte[length];
                api[0] = 0x74;// MSAPI.CMD_APPCMD_REQUEST;
                System.arraycopy(BitConverter.getBytes(sensorID), 0, api, 1, 4);
                api[5] = (byte)(acc);
                commandList.add(api);
                return commandList;
            case 5: // Calibrate AC RMS Voltage
                length = 8;

                byte[] api1 = new byte[length];
                api1[0] = 0x74;
                System.arraycopy(BitConverter.getBytes(sensorID), 0, api1, 1, 4);
                api1[5] = (byte)(acc);
                System.arraycopy(BitConverter.getBytes((short)(voltage * 10)), 0, api1, 6, 2);

                commandList.add(api1);
                return commandList;
            case 6: // Calibrate AC RMS Current
                length = 8;

                byte[] api2 = new byte[length];
                api2[0] = 0x74;
                System.arraycopy(BitConverter.getBytes(sensorID), 0, api2, 1, 4);
                api2[5] = (byte)(acc);
                System.arraycopy(BitConverter.getBytes((short)(current * 100)), 0, api2, 6, 2);

                commandList.add(api2);
                return commandList;
            case 10: // Reset Accumlator

                length = 7;

                byte[] api3 = new byte[length];
                api3[0] = 0x74;// MSAPI.CMD_APPCMD_REQUEST;
                System.arraycopy(BitConverter.getBytes(sensorID), 0, api3, 1, 4);
                api3[5] = (byte)(acc);
                api3[6] = (byte)(channel); //any value of 3 or above will reset all
                commandList.add(api3);
                return commandList;              
        }

        return commandList;
    }
    
       /**
	    * <p>
	    * sets the ReportInterval as minutes on the sensor object<br/>
	    * the next time this sensor checks in it will update
	    * </p>
	    * 
	    * @param sens the Sensor object to be updated 
	    * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
	    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
	    */
	   public static void UpdateSensorHeartBeat(Sensor sens,double heartBeat,Boolean sendSensorUpdate)
	   { 	  
	 	  	sens.ReportInterval = (heartBeat);
	 	  
	 	  	 if(sens.ReportInterval < sens.getActiveStateInterval() )
	 	  	 {  
	 	  		sens.setActiveStateInterval(sens.ReportInterval);
	 	  	 }
	 	  	
	 	    sens.GeneralConfig1Dirty = sendSensorUpdate;
	   }
	   
	   /**
	    * <p>
	    * sets all data for the GeneralConfig1 configuration available on the sensor
	    * </p>
	    * 
	    * @param sens the Sensor object to be updated 
	    * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
	    * @param activeStateHeartBeat a double value this is the aware interval and is represented in minutes so if heart beat equals 1 that is one minute
	    * @param transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
	    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
	    */
	   public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateHeartBeat,int transmitIntervalLink,Boolean sendSensorUpdate)
	   {
	 	  sens.ReportInterval = (heartBeat);
	 	  sens.setActiveStateInterval(activeStateHeartBeat);
	 	  	
	 	  	if(sens.ReportInterval < sens.getActiveStateInterval() )
	 	  		sens.setActiveStateInterval(sens.ReportInterval); 
	 	  	 
	 	  	if(transmitIntervalLink < 0)
	 	  		transmitIntervalLink = 0;
	 	  	else if (transmitIntervalLink > 250)
	 	  		transmitIntervalLink=250;
	 	  				
	 	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	 	    sens.GeneralConfig1Dirty = sendSensorUpdate;
	   }
	  
	   /**
	    * <p>
	    * sets all data for the GeneralConfig2 configuration available on the sensor
	    * </p>
	    * 
	    * @param sens the Sensor object to be updated 
	    * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
	    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
	    */
	   public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
	   { 	
	 	  if(recovery < 0)
	 		  recovery = 0;
	 		
	 		if (recovery > 10)
	 			recovery = 10;
	 	  
	 	  	sens.Recovery = recovery;
	 	    sens.GeneralConfig2Dirty = sendSensorUpdate;
	   }
	   
	   /**
	    * <p>
	    * sets all data for the ProfileConfig1 configuration available on the sensor
	    * </p>
	    * 
	    * @param sens the Sensor object to be updated 
	    * @param accumulate This will cause the kWh reading to accumulate.
	    * @param kWhLED kWh LED will flash every time this many Wh are accumulated. Saving this as 0 will turn off the LED
	    * @param awareMaxPowerThreshold If the sensor is aware and power accumulation is less than this value the sensor will become not aware.
	    * @param maxPowerThreshold if the sensor is not aware and power accumulation exceeds this value in a single standard heartbeat the sensor will become aware and report immediately
	    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
	    */
	   public static void UpdateSensorProfileConfig1(Sensor sens,Boolean Accumulate,double kWhLED,double awareMaxPowerThreshold,double maxPowerThreshold,Boolean sendSensorUpdate )
	   { 
		   sens.setMinimumThreshold((long)(awareMaxPowerThreshold * 1000));

           if (sens.getMinimumThreshold() < 1000)
        	   sens.setMinimumThreshold(1000);

           if (sens.getMinimumThreshold() > 4294967295L)
        	   sens.setMinimumThreshold(4294967295L);
	 		
           sens.setMaximumThreshold((long)(maxPowerThreshold * 1000));

           if (sens.getMaximumThreshold() < 1000)
        	   sens.setMaximumThreshold(1000);

           if (sens.getMaximumThreshold() > 4294967295L)
        	   sens.setMaximumThreshold(4294967295L);
           
           int kwhtemp = (int)(kWhLED * 1000);

           HysteresisKWHLED(sens, kwhtemp);
          
	       sens.setProfileConfig1Dirty(sendSensorUpdate);
	   }
	   
	   /**
	    * <p>
	    * sets all data for the ProfileConfig2 configuration available on the sensor
	    * </p>
	    * 
	    * @param sens the Sensor object to be updated 
	    * @param activeChannel1 Boolean for active power channels. Channel 0 is always active because the sensor is also power through this channel.
	    * @param activeChannel2 Boolean for active power channels. Channel 0 is always active because the sensor is also power through this channel.
	    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
	    */
	   public static void UpdateSensorProfileConfig2(Sensor sens,Boolean activeChannel1,Boolean activeChannel2,Boolean sendSensorUpdate )
	   { 
	         int tempVal1 = 1;
	           
	           if (activeChannel1)
	               tempVal1 = tempVal1 + 2;
	           else if (activeChannel2)
	               tempVal1 = tempVal1 + 4;
	        
	          SetCalVal3FourthByte(sens, (byte)tempVal1);
	             
	 	      sens.setProfileConfig2Dirty(sendSensorUpdate);
	   }
	   
	   public static void SetHystFourthByte(Sensor sensor, byte value)
       {
           long Period = (int)((int)(sensor.getHysteresis()) & 0x00FFFFFF);
           sensor.setHysteresis( Period | (((long)value & 0x000000FF) << 24));
       }

       public static byte GetHystFourthByte(Sensor sensor)
       {
           return (byte)(int)((long)(sensor.getHysteresis() & 0xFF000000) >> 24);
       }

       public static int HysteresisRollOver(Sensor sensor)
       {
           return sensor.getHysteresis() == 4294967295L ? 0 : (int)(((sensor.getHysteresis() & 0x000000FF)));
       }

       public static void HysteresisRollOver(Sensor sensor, int value)
       {
           if (value < 0) value = 0;
           if (value > 1) value = 1;

           long HRO = (long)((int)(sensor.getHysteresis()) & 0xFFFFFF00);
           sensor.setHysteresis((HRO | (((long)value))));
       }

       public static int HysteresisKWHLED(Sensor sensor)
       {
           return (int)(((sensor.getHysteresis() & 0x00FFFF00) >> 8));
       }

       public static void HysteresisKWHLED(Sensor sensor, int value)
       {
           if (value < 125) value = 125;
           if (value > 65535) value = 65535;

           int twa = (int)((sensor.getHysteresis()) & 0xFF0000FF);
           sensor.setHysteresis((twa | (((int)value) << 8)));
       }

       public static void SetHystZeroByte(Sensor sensor, byte value)
       {
           long Period = (long)((sensor.getHysteresis()) & 0xFFFFFF00);
           sensor.setHysteresis(Period | (((long)value & 0x000000FF)));
       }

       public static byte GetHystZeroByte(Sensor sensor)
       {
           return (byte)(int)((long)(sensor.getHysteresis() & 0x000000FF));
       }

       public static void SetHystMiddle(Sensor sensor, int value)
       {
           long TemperatureTransForm = (long)((int)(sensor.getHysteresis()) & 0xFF0000FF);
           sensor.setHysteresis(TemperatureTransForm | (((long)value & 0x0000FFFF) << 8));
       }

       public static int GetHystMiddle(Sensor sensor)
       {
           return (int)((sensor.getHysteresis() & 0x00FFFF00) >> 8);
       }
       
       public static void SetCalVal1Upper(Sensor sensor, int value)
       {
           long Period = (long)((sensor.getCalibration1()) & 0x0000FFFF);
           sensor.setCalibration1(Period | (((long)value) << 16));
       }

       public static int GetCalVal1Upper(Sensor sensor)
       {
           int temp = (int)(sensor.getCalibration1() >> 16);
           return temp;
       }

       public static void SetCalVal1Lower(Sensor sensor, int value)
       {
           long TemperatureTransForm = (long)((sensor.getCalibration1()) & 0xFFFF0000);
           sensor.setCalibration1(TemperatureTransForm | ((long)value));
       }

       public static int GetCalVal1Lower(Sensor sensor)
       {
           int temp = (int)(sensor.getCalibration1() & 0x0000FFFF);
           return temp;
       }
       
       public static void SetCalVal3FourthByte(Sensor sensor, short value)
       {
           long Period = ((long)(sensor.getCalibration3()) & 0x00FFFFFF);
           sensor.setCalibration1(Period | (((long)value & 0x000000FF) << 24));
       }

       public static short GetCalVal3FourthByte(Sensor sensor)
       {
           return (short)(((sensor.getCalibration3()) & 0xFF000000) >> 24);
       }

       public static void SetCalVal3ThirdByte(Sensor sensor, short value)
       {
           long Period = ((long)(sensor.getCalibration3()) & 0xFF00FFFF);
           sensor.setCalibration1(Period | (((long)value & 0x000000FF) << 16));
       }

       public static short GetCalVal3ThirdByte(Sensor sensor)
       {
           return (short)(((sensor.getCalibration3()) & 0x00FF0000) >> 16);
       }

       public static void SetCalVal3Lower(Sensor sensor, int value)
       {
           long TemperatureTransForm = (long)((sensor.getCalibration3()) & 0xFFFF0000);
           sensor.setCalibration3(TemperatureTransForm | ((long)value));
       }

       public static int GetCalVal3Lower(Sensor sensor)
       {
           return (int)(sensor.getCalibration3() & 0x0000FFFF);
       }
}

