package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Command;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class Control_1Base
  extends _ApplicationBase
{
  public static long MonnitApplicationID = 12L;
  public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
  public boolean RelayState1;
  public boolean RelayState2;
  
  public List<Datum> getData()
  {
    return new ArrayList(Arrays.asList(new Datum[] {
      new Datum("RelayState1", Boolean.valueOf(this.RelayState1)), 
      new Datum("RelayState2", Boolean.valueOf(this.RelayState2)) }));
  }
  
  public static Control_1Base Create(int state, byte[] rawData)
  {
    Control_1Base obj = new Control_1Base();
    obj.RelayState1 = ((rawData[0] & 0x1) != 0);
    obj.RelayState2 = ((rawData[0] & 0x2) != 0);
    return obj;
  }
  
  public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
  public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
  public static int DefaultTransmitIntervalLink = 101;
  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
  public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
  public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
  public static long DefaultHysteresis = 60L;
  public static long DefaultMinimumThreshold = -_ApplicationBase.DefaultMinimumThreshold;
  public static long DefaultMaximumThreshold = _ApplicationBase.DefaultMaximumThreshold;
  public static long DefaultCalibration1 = 0L;
  public static long DefaultCalibration2 = 0L;
  public static long DefaultCalibration3 = 0L;
  public static long DefaultCalibration4 = 0L;
  public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
  public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
  public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
  public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
  public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
  
  public static List<byte[]> ControlFrame(int sensorID, int state1, int state2, int time1, int time2, int queueID)
    throws Exception
  {
    try
    {
    	 List<byte[]> commandList = new ArrayList();
    	
      int length = 13;
      byte[] api = new byte[length];
      api[0] = 116;
      System.arraycopy(BitConverter.getBytes(sensorID), 0, api, 1, 4);
      
      api[5] = 3;
      api[6] = ((byte)state1);
      api[7] = ((byte)state2);
      if (time1 < 0) {
        time1 = 0;
      }
      if (time2 < 0) {
        time2 = 0;
      }
      System.arraycopy(BitConverter.getTwoBytes(time1), 0, api, 8, 2);
      
      System.arraycopy(BitConverter.getTwoBytes(time2), 0, api, 10, 2);
      
      api[12] = ((byte)queueID);
      
      
      
      commandList.add(api);
      
      return commandList;
    }
    catch (Exception e)
    {
      e.printStackTrace();
      throw e;
    }
  }
  
  public static void ClearPendingActionControlCommand(Sensor sensor, byte[] actionControlEventArgs_Data)
  {
    try
    {
      int index = 0;
      
      int queID = actionControlEventArgs_Data.length > 0 ? actionControlEventArgs_Data[0] : Integer.MIN_VALUE;
      for (byte[] Message : sensor.getActionControlCommands())
      {
        if (queID == Message[(Message.length - 1)])
        {
          sensor.removeActionControlCommands(index);
          break;
        }
        index++;
      }
      if (sensor.getActionControlCommands().isEmpty()) {
        sensor.setPendingActionControlCommand(Boolean.valueOf(false));
      }
    }
    catch (Exception localException) {}
  }
  
  /**
   * <p>
   * sets the ReportInterval as minutes on the sensor object<br/>
   * the next time this sensor checks in it will update
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorHeartBeat(Sensor sens,double heartBeat,Boolean sendSensorUpdate)
  {
	  
	  
	  	sens.ReportInterval = (heartBeat);
	  
	  	 if(sens.ReportInterval < sens.getHysteresis() / 60)
	  	 {  
	  		 sens.setHysteresis((long)(sens.ReportInterval)); 	
	  		 sens.setProfileConfig1Dirty(sendSensorUpdate);
	  	 }
	  	sens.setActiveStateInterval(sens.ReportInterval);
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  /**
   * <p>
   * sets all data for the GeneralConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
   * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,int transmitIntervalLink,Boolean sendSensorUpdate)
  {
	  sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval(sens.ReportInterval);
	  	
	  	 if(sens.ReportInterval  < sens.getHysteresis() / 60)
	  	 {  sens.setHysteresis((long)(sens.ReportInterval)); 
		  
		
	      sens.setProfileConfig1Dirty(sendSensorUpdate);
	  	 }
	  	
	  	if(transmitIntervalLink < 0)
	  		transmitIntervalLink = 0;
	  	else if (transmitIntervalLink > 250)
	  		transmitIntervalLink=250;
	  				
	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
 
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
  { 	
	  if(recovery < 0)
		  recovery = 0;
		
		if (recovery > 10)
			recovery = 10;
	  
	  	sens.Recovery = recovery;
	    sens.GeneralConfig2Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the ProfileConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param pollrate The frequency that this unit polls the gateway to ask if there are control messages waiting to be acted on
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorProfileConfig1(Sensor sens,double pollrate,Boolean sendSensorUpdate )
  { 
		
		  sens.setHysteresis((long)(pollrate * 60));
		  
		  if(sens.ReportInterval / 60 < sens.getHysteresis() / 60)
			  sens.setHysteresis((long)(sens.ReportInterval)); 
		  
		
	      sens.setProfileConfig1Dirty(sendSensorUpdate);
  }
  
  /**
   * <p>
   * sets all data for the ProfileConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param defaultState1 The default state is the state that relay 1 is in on power up and also the state that the relay is in while the paired sensor is in the standard operating state. if a sensor is paired and the sensor enters the aware mode,the relay will toggle to the non default state
   * @param defaultState2 The default state is the state that relay 2 is in on power up and also the state that the relay is in while the paired sensor is in the standard operating state. if a sensor is paired and the sensor enters the aware mode,the relay will toggle to the non default state
   * @param pairedSensorID1 Sensor ID of sensor that will toggle relay 1 to non default state when sensor is in aware state.
   * @param pairedSensorID2 Sensor ID of sensor that will toggle relay 2 to non default state when sensor is in aware state.
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorProfileConfig2(Sensor sens,Boolean defaultState1,Boolean defaultState2,long pairedSensorID1,long pairedSensorID2,Boolean sendSensorUpdate )
  { 
		 long calibration1 = Long.MIN_VALUE;
		 long calibration2 = Long.MIN_VALUE;
		
		 if(defaultState1)
			 calibration1 = 1;
		 else
			 calibration1 =0;
				 
		 if(defaultState2)
			 calibration2 = 1;
		 else
			 calibration2 =0;
		 
		 if(pairedSensorID1 < 1 || pairedSensorID1 == Long.MAX_VALUE)
			 pairedSensorID1 = DefaultCalibration3;
			 
		 if(pairedSensorID2 < 1 || pairedSensorID2 == Long.MAX_VALUE)
			 pairedSensorID2 = DefaultCalibration4; 
		 
		  sens.setCalibration1(calibration1);
		  sens.setCalibration2(calibration2);
		  sens.setCalibration3(pairedSensorID1);
		  sens.setCalibration4(pairedSensorID2);
		
	      sens.setProfileConfig2Dirty(sendSensorUpdate);
  }
  

  
}

