package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class CurrentBase extends _ApplicationBase{

	long MonnitApplicationID = 89L;
	  eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
	  public double AmpHours;
	  public double AvgCurrent;
	  public double MaxCurrent;
	  public double MinCurrent;
	  
	  public List<Datum> Data;
	  
	  public List<Datum> getData()
	  {
	    return Arrays.asList(new Datum[] { new Datum("Amp Hours", Double.valueOf(AmpHours)), new Datum("Average Current", Double.valueOf(AvgCurrent)), new Datum("Maximum Current", Double.valueOf(MaxCurrent)), new Datum("Minimum Current", Double.valueOf(MinCurrent)) });
	  }
	  
	  public void setData(List<Datum> data)
	  {
	    this.Data = data;
	  }
	  
	  public static CurrentBase Create(int state, byte[] rawData)
	    throws Exception
	  {
		  CurrentBase obj = new CurrentBase();
		  obj.AmpHours = (BitConverter.toInt64(rawData, 0) / 100.0);
      obj.AvgCurrent = BitConverter.ToUInt16(rawData, 0 + 8) / 100.0;
      obj.MaxCurrent = BitConverter.ToUInt16(rawData, 0 + 10) / 100.0;
      obj.MinCurrent = BitConverter.ToUInt16(rawData, 0 + 12) / 100.0;
	    return obj;
	  }
	  
	  public static double DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
	  public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
	  public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
	  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
	  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
	  public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
	  public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
	  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
	  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
	  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
	  public static long DefaultHysteresis = 0;
	  public static long DefaultMinimumThreshold = 0;
	  public static long DefaultMaximumThreshold = 20000;
	  public static long DefaultCalibration1 = 0x073D000C;
	  public static long DefaultCalibration2 = 0x07D00028;
	  public static long DefaultCalibration3 = 0x00001388;
	  public static long DefaultCalibration4 = 0;
	  public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
	  public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
	  public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
	  public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
	  public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
	  
	  public static List<byte[]> CalibrateFrame(long sensorID,  int acc, int oldVal,int newVal)
  {  
		  List<byte[]> commandList = new ArrayList();
      //Calibrate Temp
      if (acc == 3)
      {
          int length = 10;

          byte[] api = new byte[length];
          api[0] = 0x74;// MSAPI.CMD_APPCMD_REQUEST;
          System.arraycopy(BitConverter.getBytes(sensorID), 0, api, 1, 4);
          api[5] = (byte)(acc);;//Calibration
          System.arraycopy(BitConverter.getBytes(oldVal), 0, api, 6, 2);
          System.arraycopy(BitConverter.getBytes(newVal), 0, api, 8, 2);
          commandList.add(api);        
      }
      else
      {
          int length = 6;

          byte[] api = new byte[length];
          api[0] = 0x74;// MSAPI.CMD_APPCMD_REQUEST;
          System.arraycopy(BitConverter.getBytes(sensorID), 0, api, 1, 4);
          api[5] = (byte)(acc);//Calibration
          
          commandList.add(api);      
      }
      byte[] api1 = new byte[6];
      api1[0] = 0x70;// Read Sector
      System.arraycopy(BitConverter.getBytes(sensorID), 0, api1, 1, 4);
      api1[5] = (byte)29;
      

      commandList.add(api1);
      
      return commandList;
    
  }
	  
	  /**
	    * <p>
	    * sets the ReportInterval as minutes on the sensor object<br/>
	    * the next time this sensor checks in it will update
	    * </p>
	    * 
	    * @param sens the Sensor object to be updated 
	    * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
	    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
	    */
	   public static void UpdateSensorHeartBeat(Sensor sens,double heartBeat,Boolean sendSensorUpdate)
	   {
	 	  
	 	  
	 	  	sens.ReportInterval = (heartBeat);
	 	  
	 	  	 if(sens.ReportInterval < sens.getActiveStateInterval() )
	 	  	 {  
	 	  		sens.setActiveStateInterval(sens.ReportInterval);
	 	  	 }
	 	  	
	 	    sens.GeneralConfig1Dirty = sendSensorUpdate;
	   }
	   /**
	    * <p>
	    * sets all data for the GeneralConfig1 configuration available on the sensor
	    * </p>
	    * 
	    * @param sens the Sensor object to be updated 
	    * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
	    * @param activeStateHeartBeat a double value this is the aware interval and is represented in minutes so if heart beat equals 1 that is one minute
	    * @param transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
	    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
	    */
	   public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateHeartBeat,int transmitIntervalLink,Boolean sendSensorUpdate)
	   {
	 	  sens.ReportInterval = (heartBeat);
	 	  sens.setActiveStateInterval(activeStateHeartBeat);
	 	  	
	 	  	if(sens.ReportInterval < sens.getActiveStateInterval() )
	 	  		sens.setActiveStateInterval(sens.ReportInterval); 
	 	  	 
	 	  	
	 	  	if(transmitIntervalLink < 0)
	 	  		transmitIntervalLink = 0;
	 	  	else if (transmitIntervalLink > 250)
	 	  		transmitIntervalLink=250;
	 	  				
	 	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	 	    sens.GeneralConfig1Dirty = sendSensorUpdate;
	   }
	  
	   /**
	    * <p>
	    * sets all data for the GeneralConfig2 configuration available on the sensor
	    * </p>
	    * 
	    * @param sens the Sensor object to be updated 
	    * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
	    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
	    */
	   public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
	   { 	
	 	  if(recovery < 0)
	 		  recovery = 0;
	 		
	 		if (recovery > 10)
	 			recovery = 10;
	 	  
	 	  	sens.Recovery = recovery;
	 	    sens.GeneralConfig2Dirty = sendSensorUpdate;
	   }
	   
	   /**
	    * <p>
	    * sets all data for the ProfileConfig1 configuration available on the sensor
	    * </p>
	    * 
	    * @param sens the Sensor object to be updated 
	    * @param measurementsPerTransmission the amount of assessments the sensor where take inbetween heartbeats if during these assesments the value is above or below respective triggering thresholds it will send an aware state message. 
	    * @param accumulate With accumulate false, only the Amp Hours/Watt Hourse consumed during a single heartbeat will be reported. With accumulate true, the energy consumed during the current heartbeat will be added to the energy consumed during the previous heartbeats.
	    * @param hysteresis A buffer to prevent the sensor from bouncing between standard operation and aware state when the asseessments are very close to a threashold. for example, if a maximum threashold is set to 90 degrees and a hysteresis of 1 degree, then once the sensor takes an assessment of 90.0 degrees and enters the aware state it will remain in the aware state until the temperature readings drop to 89.0 degrees
	    * @param minimumThreshold the minimum temperature the sensor can report before it goes aware
	    * @param maximumThreshold the maximum temperature the sensor can report before it goes aware
	    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
	    */
	   public static void UpdateSensorProfileConfig1(Sensor sens,int measurementsPerTransmission,boolean accumulate,int hysteresis,double minimumThreshold,double maximumThreshold,Boolean sendSensorUpdate )
	   { 
	 		  sens.setMeasurementsPerTransmission(measurementsPerTransmission); 
	 		 if (hysteresis < 0)
             SetHystLower(sens, 0);
         else if (hysteresis > 5)
             SetHystLower(sens, 5 * 100);
         else
             SetHystLower(sens, hysteresis * 100);
	 		 
	 		  if (minimumThreshold < 0)
	 			 sens.setMinimumThreshold(0);
          else if (minimumThreshold > 120)
        	  sens.setMinimumThreshold(120 * 100);
          else
        	  sens.setMinimumThreshold((long)minimumThreshold * 100);
	 		 
	 		  if (minimumThreshold < 0)
		 			 sens.setMaximumThreshold(0);
	              else if (minimumThreshold > 225)
	            	  sens.setMaximumThreshold( 225* 100);
	              else
	            	  sens.setMaximumThreshold((long)maximumThreshold * 100);
	 	
	 		  if(sens.getMaximumThreshold() < sens.getMinimumThreshold())
	 			  sens.setMaximumThreshold(sens.getMinimumThreshold());
	 		 //offset hidden shouldn't be needed
	 		  
	 		  //accumulate
	 		  
	 		  
	 	      sens.setProfileConfig1Dirty(sendSensorUpdate);
	   }
	   
	   
	   /**
	    * <p>
	    * sets all data for the ProfileConfig2 configuration available on the sensor
	    * </p>
	    * 
	    * @param sens the Sensor object to be updated 
	    * @param accumulate The default is off standard operation reports the number of pulses detected between heartbeats. Accumulate on will accumulate the readings heartbeat after the heartbeat, accumulation rolls over at 65535 (2^16 or 0xFFFF)
	    * @param edge determines if the pulses are recorded at the positive or negative edge of a pulse, positive edge = 0, negative edge = 1, and both edges = 2
	    * @param filter determines if there is a filter and what Hz its set to, 40 Hz filter = 0, 4 Hz filter = 1, and no filter = 2
	    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
	    */
	   public static void UpdateSensorProfileConfig2(Sensor sens,int ctSize,double avgInterval,double currentShift,Boolean sendSensorUpdate )
	   { 
		   int temp2 = ctSize;
		   SetCalVal3Upper(sens,temp2);
		   SetCalVal3Lower(sens,(int)(avgInterval * 1000));
		   sens.setCalibration4((long)(currentShift * 100));
	 	      sens.setProfileConfig2Dirty(sendSensorUpdate);
	   }
	   
	   /**
	     * <p>
	     * Calibrates the sensor to correct values set by the user
	     * </p>
	     * 
	     * @param sens the Sensor object to be updated 
	     * @param isZeroCal chooses between the type of calibration being used either Zero Calibration or a Span Calibration
	     * @param actualValue what the actual Value of the Zero Calibration or Span Calibration
	     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
	     */
	    public static void UpdateSensorCalibration(Sensor sens,Boolean isZeroCal,double actualValue,Boolean sendSensorUpdate) throws Exception
	    { 
	       
	    	if (isZeroCal)
            {
                SetCalVal2Lower(sens, (int)(actualValue * 100));
                SetHystUpper(sens, 0x03);
            }
            else
            {
                double temp = actualValue * 100;
                SetCalVal2Upper(sens, (int)(temp));
                SetHystUpper(sens, 0x04);
            }
	    	
	    	sens.setProfileConfig1Dirty(false);
	    	sens.setProfileConfig2Dirty(false);
	    	sens.setPendingActionControlCommand(sendSensorUpdate);
	       
	    }
	  
	   
	    public static void SetHystUpper(Sensor sensor, int value)
        {
            long Period = (long)((int)(sensor.getHysteresis()) & 0x0000FFFF);
            sensor.setHysteresis(Period | (((long)value) << 16));
        }

        public static int GetHystUpper(Sensor sensor)
        {

            return (int)(sensor.getHysteresis() >> 16);
        }

        public static void SetHystLower(Sensor sensor, int value)
        {
            long TemperatureTransForm = (long)((int)(sensor.getCalibration2()) & 0xFFFF0000);
            sensor.setHysteresis(TemperatureTransForm | ((long)value));
        }

        public static int GetHystLower(Sensor sensor)
        {

            return (int)(sensor.getHysteresis() & 0x0000FFFF);
        }



        public static void SetCalVal2Upper(Sensor sensor, int value)
        {
            long Period = (long)((int)(sensor.getCalibration2()) & 0x0000FFFF);
            sensor.setCalibration2(Period | (((long)value) << 16));
        }

        public static int GetCalVal2Upper(Sensor sensor)
        {

            return (int)(sensor.getCalibration2() >> 16);
        }

        public static void SetCalVal2Lower(Sensor sensor, int value)
        {
            long TemperatureTransForm = (long)((int)(sensor.getCalibration2()) & 0xFFFF0000);
            sensor.setCalibration2(TemperatureTransForm | ((long)value));
        }

        public static int GetCalVal2Lower(Sensor sensor)
        {
            return (int)(sensor.getCalibration2() & 0x0000FFFF);
        }



        public static void SetCalVal3Upper(Sensor sensor, int value)
        {
            long Period = (long)((int)(sensor.getCalibration3()) & 0x0000FFFF);
            sensor.setCalibration3(Period | (((long)value) << 16));
        }

        public static int GetCalVal3Upper(Sensor sensor)
        {
            return (int)(sensor.getCalibration3() >> 16);
        }

        public static void SetCalVal3Lower(Sensor sensor, int value)
        {
            long TemperatureTransForm = (long)((int)(sensor.getCalibration3()) & 0xFFFF0000);
            sensor.setCalibration3(TemperatureTransForm | ((long)value));
        }

        public static int GetCalVal3Lower(Sensor sensor)
        {
            return (int)(sensor.getCalibration3() & 0x0000FFFF);
        }
}
