package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class FilteredPulseCounterBase
  extends _ApplicationBase
{
  public static long MonnitApplicationID = 73L;
  public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
  public int Count;
  public List<Datum> Data;
  
  public List<Datum> getData()
  {
    return Arrays.asList(new Datum[] { new Datum("Count", Integer.valueOf(this.Count)) });
  }
  
  public static List<byte[]> CalibrateFrame(long sensorID)
  {
	  List<byte[]> commandList = new ArrayList();
    int length = 6;
    byte[] api = new byte[length];
    api[0] = 116;
    System.arraycopy(BitConverter.getBytes(sensorID), 0, api, 1, 4);
    api[5] = 3;
    commandList.add(api);
    
    byte[] api1 = new byte[6];
    api1[0] = 0x70;// Read Sector
    System.arraycopy(BitConverter.getBytes(sensorID), 0, api1, 1, 4);
    api1[5] = (byte)29;
    
    commandList.add(api1);
    
    return commandList;
  }
  
  public static FilteredPulseCounterBase Create(int state, byte[] rawData)
    throws Exception
  {
    FilteredPulseCounterBase obj = new FilteredPulseCounterBase();
    obj.Count = BitConverter.ToUInt32(rawData, 0);
    return obj;
  }
  
  public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
  public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
  public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
  public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
  public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
  public static long DefaultHysteresis = _ApplicationBase.DefaultHysteresis;
  public static long DefaultMinimumThreshold = -1L;
  public static long DefaultMaximumThreshold = -1L;
  public static long DefaultCalibration1 = 0L;
  public static long DefaultCalibration2 = 630L;
  public static long DefaultCalibration3 = 0L;
  public static long DefaultCalibration4 = 1L;
  public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
  public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
  public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
  public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
  public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
  
  
  /**
   * <p>
   * sets the ReportInterval as minutes on the sensor object<br/>
   * the next time this sensor checks in it will update
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorHeartBeat(Sensor sens,double heartbeat,Boolean sendSensorUpdate)
  {	
	  sens.ReportInterval = heartbeat;
	 
	  if(sens.ReportInterval < sens.getActiveStateInterval())
		  sens.setActiveStateInterval(sens.ReportInterval);
	  
	  sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
   * @param activeStateInterval a double value this is the aware state heart beat represented in minutes so aware state heart beat equals 1 that is one minute
   * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateInterval,int transmitIntervalLink,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval((activeStateInterval));
	  	
	  	if(sens.ReportInterval < sens.getActiveStateInterval())
			  sens.setActiveStateInterval(sens.ReportInterval);
	  	
	  	if(transmitIntervalLink < 0)
	  		transmitIntervalLink = 0;
	  	else if (transmitIntervalLink > 250)
	  		transmitIntervalLink=250;
	  				
	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
  { 	
	  if(recovery < 0)
		  recovery = 0;
		
		if (recovery > 10)
			recovery = 10;
	  
	  	sens.Recovery = recovery;
	    sens.GeneralConfig2Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the ProfileConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param awareOverflowCount while in the aware state counts below this value set the sensor back to normal
   * @param overflowCount while in normal state counts above this value send the sensor into aware state
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorProfileConfig1(Sensor sens,long awareOverflowCount,long overflowCount,Boolean sendSensorUpdate )
  { 
	  if (awareOverflowCount < 0)
          awareOverflowCount = 0;
		  sens.setMinimumThreshold(awareOverflowCount);
		  if (overflowCount < 0)
              overflowCount = 0;
		  sens.setMaximumThreshold(overflowCount);//
	      sens.setProfileConfig1Dirty(sendSensorUpdate);
  }
  
  /**
   * <p>
   * sets all data for the ProfileConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param accumulate The default is off standard operation reports the number of pulses detected between heartbeats. Accumulate on will accumulate the readings heartbeat after the heartbeat, accumulation rolls over at 65535 (2^16 or 0xFFFF)
   * @param edge determines if the pulses are recorded at the positive or negative edge of a pulse, positive edge = 0, negative edge = 1, and both edges = 2
   * @param filter determines if there is a filter and what Hz its set to, 40 Hz filter = 0, 4 Hz filter = 1, and no filter = 2
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorProfileConfig2(Sensor sens,Boolean accumulate,int edge,int filter,Boolean sendSensorUpdate )
  { 
	     if(edge > 2)
	    	 edge = 2;
	     else if (edge < 0)
	    	 edge = 0;
		  sens.setCalibration1(edge);
		
		  sens.setCalibration3(accumulate ? 1 : 0);
		  
		  if(filter > 2)
			  filter = 2;
		     else if (filter < 0)
		    	 filter = 0;
		  sens.setCalibration4(filter);
	      sens.setProfileConfig2Dirty(sendSensorUpdate);
  }
  
}