package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.Arrays;
import java.util.List;

public class HumiditySHT25Base
  extends _ApplicationBase
{
  public static long MonnitApplicationID = 43L;
  public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
  public double Hum;
  public double Temperature;
  public List<Datum> Data;
  
  public double getHum()
  {
    return this.Hum;
  }
  
  public void setHum(double hum)
  {
    this.Hum = hum;
  }
  
  public double getTemperature()
  {
    return this.Temperature;
  }
  
  public void setTemperature(double temperature)
  {
    this.Temperature = temperature;
  }
  
  public List<Datum> getData()
  {
    return Arrays.asList(new Datum[] { new Datum("Humidity", Double.valueOf(this.Hum)), new Datum("Temperature", Double.valueOf(this.Temperature)), new Datum("DewPoint", Double.valueOf(Dewpoint(this.Hum,this.Temperature))) });
  }
  
  public void setData(List<Datum> data)
  {
    this.Data = data;
  }
  
  public static HumiditySHT25Base Create(int state, byte[] rawData)
    throws Exception
  {
    HumiditySHT25Base obj = new HumiditySHT25Base();
    if ((state & 0xF0) == 0)
    {
      obj.Temperature = (BitConverter.toInt16Backwards(rawData, 0) / 100.0D);
      obj.Hum = (BitConverter.toInt16Backwards(rawData, 2) / 100.0D);
    }
    else
    {
      obj.Temperature = 0.0D;
      obj.Hum = -100.0D;
    }
    return obj;
  }
  
  public static double Dewpoint(double Humidity, double Temperature)
  {
  // calculates dew point
  // input:   HumiditySHT25 [%RH], temperature [C]
  // output:  dew point [C]
      double logEx, dew_point;
      logEx = 0.66077 + 7.5 * Temperature / (237.3 + Temperature) + (Math.log10(Humidity) - 2.0);
      dew_point = (logEx - 0.66077) * 237.3 / (0.66077 + 7.5 - logEx);
      return dew_point;
  }
  
  public static double DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
  public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
  public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
  public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
  public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
  public static long DefaultHysteresis = _ApplicationBase.DefaultHysteresis;
  public static long DefaultMinimumThreshold = 60536L;
  public static long DefaultMaximumThreshold = 655375000L;
  public static long DefaultCalibration1 = _ApplicationBase.DefaultCalibration1;
  public static long DefaultCalibration2 = _ApplicationBase.DefaultCalibration2;
  public static long DefaultCalibration3 = _ApplicationBase.DefaultCalibration3;
  public static long DefaultCalibration4 = _ApplicationBase.DefaultCalibration4;
  public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
  public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
  public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
  public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
  public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
  
  /**
   * <p>
   * sets the ReportInterval as minutes on the sensor object<br/>
   * the next time this sensor checks in it will update
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorHeartBeat(Sensor sens,double heartbeat,Boolean sendSensorUpdate)
  {	
	  sens.ReportInterval = heartbeat;
	 
	  if(sens.ReportInterval < sens.getActiveStateInterval())
		  sens.setActiveStateInterval(sens.ReportInterval);
	  
	  sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
   * @param activeStateInterval a double value this is the aware state heart beat represented in minutes so aware state heart beat equals 1 that is one minute
   * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateInterval,int transmitIntervalLink,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval((activeStateInterval));
	  	
	  	if(sens.ReportInterval < sens.getActiveStateInterval())
			  sens.setActiveStateInterval(sens.ReportInterval);
	  	
	  	if(transmitIntervalLink < 0)
	  		transmitIntervalLink = 0;
	  	else if (transmitIntervalLink > 250)
	  		transmitIntervalLink=250;
	  				
	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
  { 	
	  if(recovery < 0)
		  recovery = 0;
		
		if (recovery > 10)
			recovery = 10;
	  
	  	sens.Recovery = recovery;
	    sens.GeneralConfig2Dirty = true;
  }
  
  /**
   * <p>
   * sets all data for the ProfileConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param measurementsPerTransmission the amount of assessments the sensor where take inbetween heartbeats if during these assesments the value is above or below respective triggering thresholds it will send an aware state message. 
   * @param hysteresis A buffer to prevent the sensor from bouncing between standard operation and aware state when the asseessments are very close to a threashold. for example, if a maximum threashold is set to 90 degrees and a hysteresis of 1 degree, then once the sensor takes an assessment of 90.0 degrees and enters the aware state it will remain in the aware state until the temperature readings drop to 89.0 degrees
   * @param minimumThreshold the minimum temperature the sensor can report before it goes aware
   * @param maximumThreshold the maximum temperature the sensor can report before it goes aware
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorProfileConfig1(Sensor sens,int measurementsPerTransmission,double hysteresis,double minimumThreshold,double maximumThreshold,Boolean sendSensorUpdate )
  { 
		  sens.setMeasurementsPerTransmission(measurementsPerTransmission); 
		  sens.setHysteresis(((((int)(hysteresis * 100)) << 16))); // leave last 16 all 0's for temp
		  
		  sens.setMinimumThreshold(((((int)(minimumThreshold * 100)) << 16)));
		  sens.setMinimumThreshold(sens.getMinimumThreshold() + 60536); // this is -5000 (-50*100) which is 0xEC78 Than converted to retain sign in larger integer base
		  
		  if(sens.getMinimumThreshold() < DefaultMinimumThreshold)
			  sens.setMinimumThreshold(DefaultMinimumThreshold);
		  
		  if (sens.getMinimumThreshold() > DefaultMaximumThreshold)
			  sens.setMinimumThreshold(DefaultMaximumThreshold);
		  
		  sens.setMaximumThreshold((((int)(maximumThreshold * 100) << 16)));//
		  sens.setMaximumThreshold(sens.getMaximumThreshold() + 15000); // max temp * 100
		  
		  if(sens.getMaximumThreshold() < DefaultMinimumThreshold)
			  sens.setMaximumThreshold(DefaultMinimumThreshold);
		  
		  if (sens.getMaximumThreshold() > DefaultMaximumThreshold)
			  sens.setMaximumThreshold(DefaultMaximumThreshold);
		  
		  if (sens.getMinimumThreshold() > sens.getMaximumThreshold())
			  sens.setMaximumThreshold(sens.getMinimumThreshold());
		  
		  sens.setProfileConfig1Dirty(sendSensorUpdate);
  }
  
}

