package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.Arrays;
import java.util.List;

public class LoggedActivityBase
  extends _ApplicationBase
{
  public static long MonnitApplicationID = 42L;
  public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
  public int Count;
  public int PastCount;
  public List<Datum> Data;
  
  public int getCount()
  {
    return this.Count;
  }
  
  public void setCount(int count)
  {
    this.Count = count;
  }
  
  public int getPastCount()
  {
    return this.PastCount;
  }
  
  public void setPastCount(int pastCount)
  {
    this.PastCount = pastCount;
  }
  
  public List<Datum> getData()
  {
    return Arrays.asList(new Datum[] { new Datum("Count", Integer.valueOf(this.Count)), new Datum("PastCount", Integer.valueOf(this.PastCount)) });
  }
  
  public void setData(List<Datum> data)
  {
    this.Data = data;
  }
  
  public static LoggedActivityBase Create(int state, byte[] rawData)
    throws Exception
  {
    LoggedActivityBase obj = new LoggedActivityBase();
    obj.Count = BitConverter.ToUInt16(rawData, 0);
    obj.PastCount = BitConverter.ToUInt16(rawData, 2);
    return obj;
  }
  
  public static double DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
  public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
  public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
  public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
  public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
  public static long DefaultHysteresis = _ApplicationBase.DefaultHysteresis;
  public static long DefaultMinimumThreshold = _ApplicationBase.DefaultMinimumThreshold;
  public static long DefaultMaximumThreshold = _ApplicationBase.DefaultMaximumThreshold;
  public static long DefaultCalibration1 = 2000L;
  public static long DefaultCalibration2 = 1L;
  public static long DefaultCalibration3 = 2L;
  public static long DefaultCalibration4 = 6L;
  public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
  public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
  public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
  public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
  public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
  
  /**
   * <p>
   * sets the ReportInterval as minutes on the sensor object<br/>
   * the next time this sensor checks in it will update
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorHeartBeat(Sensor sens,double heartBeat,Boolean sendSensorUpdate)
  {
	  sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval((sens.ReportInterval));
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
	
  /**
   * <p>
   * sets all data for the GeneralConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
   * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,int transmitIntervalLink,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval((heartBeat));
	  	
	  	if(transmitIntervalLink < 0)
	  		transmitIntervalLink = 0;
	  	else if (transmitIntervalLink > 250)
	  		transmitIntervalLink=250;
	  				
	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
  { 	
	  if(recovery < 0)
		  recovery = 0;
		
		if (recovery > 10)
			recovery = 10;
	  
	  	sens.Recovery = recovery;
	    sens.GeneralConfig2Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the ProfileConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param Sensitivity Default value is 3 1 is the most sensitive to motion, 4 is the least sensitive
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorProfileConfig2(Sensor sens,int Sensitivity,Boolean sendSensorUpdate )
  { 
	  if(Sensitivity < 1)
		  Sensitivity = 1;
	  if(Sensitivity > 4)
		  Sensitivity = 4;
		  
	  
	  switch (Sensitivity)
      {
          case 1://Most Sensitive
        	  sens.setCalibration1(500);
        	  sens.setCalibration3(2);
        	  sens.setCalibration4(9);
              break;
          case 2:
        	  sens.setCalibration1(1000);
        	  sens.setCalibration3(2);
        	  sens.setCalibration4(8);
              break;
          case 3:
          default:
        	  sens.setCalibration1(2000);
        	  sens.setCalibration3(2);
        	  sens.setCalibration4(6);
              break;
          case 4://Least Sensitive
        	  sens.setCalibration1(4000);
        	  sens.setCalibration3(2);
        	  sens.setCalibration4(2);
              break;
      }
  
	  
		   sens.setProfileConfig2Dirty(sendSensorUpdate);
  }
}

