package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Command;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;
import com.monnit.mine.MonnitMineAPI.enums.eConfigSection;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class LuxBase
  extends _ApplicationBase
{
  public static long MonnitApplicationID = 21L;
  public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
  public int LuxReading;
  public List<Datum> Data = Arrays.asList(new Datum[] {new Datum("LuxReading", Integer.valueOf(this.LuxReading)) });
  
  public List<Datum> getData()
  {
    return Arrays.asList(new Datum[] { new Datum("LuxReading", Integer.valueOf(this.LuxReading)) });
  }
  
  public static LuxBase Create(int state, byte[] rawData)
    throws Exception
  {
    LuxBase obj = new LuxBase();
    obj.LuxReading = BitConverter.toInt16Backwards(rawData, 0);
    return obj;
  }
  
  public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
  public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
  public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
  public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
  public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
  public static long DefaultHysteresis = _ApplicationBase.DefaultHysteresis;
  public static long DefaultMinimumThreshold = 0L;
  public static long DefaultMaximumThreshold = 1000L;
  public static long DefaultCalibration1 = 727941L;
  public static long DefaultCalibration2 = 1L;
  public static long DefaultCalibration3 = 0L;
  public static long DefaultCalibration4 = _ApplicationBase.DefaultCalibration4;
  public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
  public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
  public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
  public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
  public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
  
  public static List<byte[]> CalibrateFrame(int sensorID, int actual, int filter, double temperatureCelcius)
  {
    List<byte[]> commandList = new ArrayList();
    
    
    int length = 12;
    byte[] api = new byte[length];
    api[0] = 116;
    System.arraycopy(BitConverter.getBytes(sensorID), 0, api, 1, 4);
    api[5] = 3;
    byte[] buf = new byte[6];
    System.arraycopy(BitConverter.getBytes(actual), 0, buf, 0, 2);
    System.arraycopy(BitConverter.getBytes(filter), 0, buf, 2, 2);
    System.arraycopy(BitConverter.getBytes(temperatureCelcius * 10.0D), 0, 
      buf, 4, 2);
    System.arraycopy(buf, 0, api, 6, 6);
    
    commandList.add(api);
    
    byte[] api1 = new byte[6];
    api1[0] = 0x70;// Read Sector
    System.arraycopy(BitConverter.getBytes(sensorID), 0, api1, 1, 4);
    api1[5] = (byte)29;
    
    commandList.add(api1);
    
    return commandList;
  }
  
  /**
   * <p>
   * sets the ReportInterval as minutes on the sensor object<br/>
   * the next time this sensor checks in it will update
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorHeartBeat(Sensor sens,double heartBeat,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	    if(sens.ReportInterval < sens.getActiveStateInterval())
			  sens.setActiveStateInterval(sens.ReportInterval);
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
   * @param activeStateInterval a double value this is the aware state heart beat represented in minutes so aware state heart beat equals 1 that is one minute
   * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateInterval,int transmitIntervalLink,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval((activeStateInterval));
	  	
	    if(sens.ReportInterval < sens.getActiveStateInterval())
			  sens.setActiveStateInterval(sens.ReportInterval);
	  	
	  	if(transmitIntervalLink < 0)
	  		transmitIntervalLink = 0;
	  	else if (transmitIntervalLink > 250)
	  		transmitIntervalLink=250;
	  				
	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
  { 	
	  if(recovery < 0)
		  recovery = 0;
		
		if (recovery > 10)
			recovery = 10;
	  
	  	sens.Recovery = recovery;
	    sens.GeneralConfig2Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the ProfileConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param measurementsPerTransmission the amount of assessments the sensor where take in between heartbeats if during these assesments the value is above or below respective triggering thresholds it will send an aware state message. 
   * @param hysteresis A buffer to prevent the sensor from bouncing between standard operation and aware state when the asseessments are very close to a threashold. for example, if a maximum threashold is set to 90 degrees and a hysteresis of 1 degree, then once the sensor takes an assessment of 90.0 degrees and enters the aware state it will remain in the aware state until the temperature readings drop to 89.0 degrees
   * @param minimumThreshold the minimum temperature the sensor can report before it goes aware
   * @param maximumThreshold the maximum temperature the sensor can report before it goes aware
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorProfileConfig1(Sensor sens,int measurementsPerTransmission,long hysteresis,long minimumThreshold,long maximumThreshold,Boolean sendSensorUpdate )
  { 
		  sens.setMeasurementsPerTransmission(measurementsPerTransmission); 
		  
		  
		  sens.setHysteresis(hysteresis);
		  sens.setMinimumThreshold(minimumThreshold);
		  sens.setMaximumThreshold(maximumThreshold);//
	      sens.setProfileConfig1Dirty(sendSensorUpdate);
  }
  
  /**
   * <p>
   * Calibrates the sensor to correct values set by the user
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param actualReading actual reading of light in lux
   * @param tempTarget what the actual temperature of the room the sensor is located in is
   * @param isFahrenheit data is always stored as Celsius so if you are inputting values as Fahrenheit you will want to set this boolean value to true
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorCalibration(Sensor sens,int actualReading,int tempTarget,Boolean isFahrenheit,Boolean sendSensorUpdate) throws Exception
  { 
	  sens.setCalibration1(actualReading);
	  sens.setCalibration2(1);
      int temp = tempTarget;
      int calval3 = (int)((isFahrenheit ? ((5.0 / 9.0) * (temp - 32)) : temp) * 10);
      sens.setCalibration3(calval3);
     
      //LuxBase.CalibrateFrame(sensorID, actual, filter, temperatureCelcius)
      sens.setActionControlCommands(CalibrateFrame(sens.SensorID,(int)sens.getCalibration1(),(int)sens.getCalibration2(),sens.getCalibration3() / 10.0));
	  sens.setProfileConfig1Dirty(false);
	  sens.setProfileConfig2Dirty(false);
	  
	  sens.setPendingActionControlCommand(sendSensorUpdate);
  }
}