package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.Arrays;
import java.util.List;

public class PIRBase
  extends _ApplicationBase
{
  public static long MonnitApplicationID = 23L;
  public static eApplicationProfileType ProfileType = eApplicationProfileType.Trigger;
  public Boolean Motion;
  public int State;
  public List<Datum> Data;
  
  public Boolean getMotion()
  {
    return this.Motion;
  }
  
  public void setMotion(Boolean motion)
  {
    this.Motion = motion;
  }
  
  public int getState()
  {
    return this.State;
  }
  
  public void setState(int state)
  {
    this.State = state;
  }
  
  public List<Datum> getData()
  {
    return Arrays.asList(new Datum[] { new Datum("Motion", this.Motion), new Datum("State", Integer.valueOf(this.State)) });
  }
  
  public void setData(List<Datum> data)
  {
    this.Data = data;
  }
  
  public static PIRBase Create(int state, byte[] rawData)
    throws Exception
  {
    PIRBase obj = new PIRBase();
    try
    {
      obj.State = state;
      obj.Motion = BitConverter.ToBoolean(rawData[0]);
      return obj;
    }
    catch (Exception ex)
    {
      throw new Exception(ex.getMessage());
    }
  }
  
  public static double DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
  public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
  public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
  public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
  public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
  public static long DefaultHysteresis = _ApplicationBase.DefaultHysteresis;
  public static long DefaultMinimumThreshold = -_ApplicationBase.DefaultMinimumThreshold;
  public static long DefaultMaximumThreshold = _ApplicationBase.DefaultMaximumThreshold;
  public static long DefaultCalibration1 = _ApplicationBase.DefaultCalibration1;
  public static long DefaultCalibration2 = _ApplicationBase.DefaultCalibration2;
  public static long DefaultCalibration3 = _ApplicationBase.DefaultCalibration3;
  public static long DefaultCalibration4 = _ApplicationBase.DefaultCalibration4;
  public static int DefaultEventDetectionType = 0;
  public static int DefaultEventDetectionPeriod = 25;
  public static int DefaultEventDetectionCount = 9;
  public static int DefaultRearmTime = 30;
  public static int DefaultBiStable = 0;
  public static int DefaultEventDetectionCountPre2_3 = 3;
  
  /**
   * <p>
   * sets the ReportInterval as minutes on the sensor object<br/>
   * the next time this sensor checks in it will update
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorHeartBeat(Sensor sens,double heartbeat,Boolean sendSensorUpdate)
  {	
	  sens.ReportInterval = heartbeat;
	  if(sens.ReportInterval < sens.getActiveStateInterval())
		  sens.setActiveStateInterval(sens.ReportInterval);
	  sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
   * @param activeStateInterval a double value this is the aware state heart beat represented in minutes so aware state heart beat equals 1 that is one minute
   * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateInterval,int transmitIntervalLink,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval((activeStateInterval));
	  	
	  	if(sens.ReportInterval < sens.getActiveStateInterval())
			  sens.setActiveStateInterval(sens.ReportInterval);
	  	
	  	if(transmitIntervalLink < 0)
	  		transmitIntervalLink = 0;
	  	else if (transmitIntervalLink > 250)
	  		transmitIntervalLink=250;
	  				
	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig2(Sensor sens,int recovery,Boolean sendSensorUpdate)
  { 	
	  
	  if(recovery < 0)
		  recovery = 0;
		
		if (recovery > 10)
			recovery = 10;
		
	  	sens.Recovery = recovery;
	    sens.GeneralConfig2Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the ProfileConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param eventDectionType Allows aware state when motion = 0, no motion = 1, or state change = 2
   * @param biStable Aware reading = 1 if it is selected, then the time between the message detecting motion and the message saying no motion is (rearm time + awarestate heartbeat). if it is set to all state changes = 0, then the time between the two messages is (rearmtime + rearm time)
   * @param rearmTime The time in seconds after a triggering event that the sensor will wait before re-arming itself
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorProfileConfig1(Sensor sens,int eventDectionType,int biStable,int rearmTime,Boolean sendSensorUpdate)
  { 
	  //motion = 0
	  //no motion = 1
	  //state change = 2
		  sens.setEventDetectionType(eventDectionType);
	//aware reading = 1
	//all state changes = 0
		  sens.setBiStable(biStable);

	if(rearmTime < 1)
		rearmTime = 1;
	
	if (rearmTime > 600)
		rearmTime = 600;
	
	  sens.setRearmTime(rearmTime); 
	  
	  sens.setProfileConfig1Dirty(sendSensorUpdate);
	 
  }
  
 
}

