package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class Pressure300PSIBase extends _ApplicationBase {

	public static long MonnitApplicationID = 82;
    public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
    public double Pressure;
    
    public List<Datum> Data;
    
    public List<Datum> getData()
    {
    	  return Arrays.asList(new Datum[] {new Datum("Pressure", Pressure) });
    	  
    }
    
    public static Pressure50PSIBase Create(int state, byte[] rawData)throws Exception
    {
    	Pressure50PSIBase obj = new Pressure50PSIBase();
        obj.Pressure = (BitConverter.ToUInt16(rawData, 0)) / 10.0;
        return obj;
    }
    
    public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
    public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
    public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
    public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
    public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
    public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
    public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
    public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
    public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
    public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
    public static long DefaultHysteresis = 0;
    public static long DefaultMinimumThreshold = 0;
    public static long DefaultMaximumThreshold = 3000;
    public static long DefaultCalibration1 = 500;
    public static long DefaultCalibration2 = 4500;
    public static long DefaultCalibration3 = 750;
    public static long DefaultCalibration4 = 3000;
    public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
    public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
    public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
    public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
    public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
    
    public static List<byte[]> getCalibrationValue(long sensorID, double psi)
    {
    	 List<byte[]> commandList = new ArrayList();
    	 
        int tempPressure = (int)(psi);
        //Calibrate Temp
        int length = 8;
        byte[] api = new byte[length];
        api[0] = 0x74;// MSAPI.CMD_APPCMD_REQUEST;
        System.arraycopy(BitConverter.getBytes(sensorID), 0, api, 1, 4);
        api[5] = tempPressure == 0 ? (byte)3 : (byte)4; //Calibration (3 for Zero 4 for Span)
        if (tempPressure != 0)
        	 System.arraycopy(BitConverter.getBytes(tempPressure), 0, api, 6, 2);
        commandList.add(api);

        byte[] api1 = new byte[6];
        api1[0] = 0x70;// Read Sector
        System.arraycopy(BitConverter.getBytes(sensorID), 0, api1, 1, 4);
        api1[5] = (byte)29;
        
        commandList.add(api1);
        
        return commandList;
    }
    
    /**
     * <p>
     * sets the ReportInterval as minutes on the sensor object<br/>
     * the next time this sensor checks in it will update
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorHeartBeat(Sensor sens,double heartBeat,Boolean sendSensorUpdate)
    {
  	  	sens.ReportInterval = (heartBeat);
  	  	
  	  	if(sens.ReportInterval < sens.getActiveStateInterval())
  			  sens.setActiveStateInterval(sens.ReportInterval);
  	  	
  	    sens.GeneralConfig1Dirty = sendSensorUpdate;
    }
    
    /**
     * <p>
     * sets all data for the GeneralConfig1 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
     * @param activeStateInterval a double value this is the aware state heart beat represented in minutes so aware state heart beat equals 1 that is one minute
     * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateInterval,int transmitIntervalLink,Boolean sendSensorUpdate)
    {
  	  	sens.ReportInterval = (heartBeat);
  	  	sens.setActiveStateInterval((activeStateInterval));
  	  	
  	  	if(sens.ReportInterval < sens.getActiveStateInterval())
  			  sens.setActiveStateInterval(sens.ReportInterval);
  	  	
  	  	if(transmitIntervalLink < 0)
  	  		transmitIntervalLink = 0;
  	  	else if (transmitIntervalLink > 250)
  	  		transmitIntervalLink=250;
  	  				
  	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
  	    sens.GeneralConfig1Dirty = sendSensorUpdate;
    }
    
    /**
     * <p>
     * sets all data for the GeneralConfig2 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
    { 	
  	  if(recovery < 0)
  		  recovery = 0;
  		
  		if (recovery > 10)
  			recovery = 10;
  	  
  	  	sens.Recovery = recovery;
  	    sens.GeneralConfig2Dirty = sendSensorUpdate;
    }
   
    /**
     * <p>
     * sets all data for the ProfileConfig1 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param measurementsPerTransmission the amount of assessments the sensor where take inbetween heartbeats if during these assesments the value is above or below respective triggering thresholds it will send an aware state message. 
     * @param hysteresis A buffer to prevent the sensor from bouncing between standard operation and aware state when the asseessments are very close to a threashold. for example, if a maximum threashold is set to 90 degrees and a hysteresis of 1 degree, then once the sensor takes an assessment of 90.0 degrees and enters the aware state it will remain in the aware state until the temperature readings drop to 89.0 degrees
     * @param minimumThreshold the minimum temperature the sensor can report before it goes aware
     * @param maximumThreshold the maximum temperature the sensor can report before it goes aware
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorProfileConfig1(Sensor sens,int measurementsPerTransmission,double hysteresis,double minimumThreshold,double maximumThreshold,Boolean sendSensorUpdate )
    { 
  		  sens.setMeasurementsPerTransmission(measurementsPerTransmission); 
  		  sens.setHysteresis((long)(hysteresis * 10));
  		  sens.setMinimumThreshold((long)(minimumThreshold * 10));
  		  sens.setMaximumThreshold((long)(maximumThreshold * 10));
  	      sens.setProfileConfig1Dirty(sendSensorUpdate);
    }
    
    /**
     * <p>
     * Calibrates the sensor to correct values set by the user
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param actualPSI what the actual PSI of the sensor should be.
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorCalibration(Sensor sens,double actualPSI, Boolean sendSensorUpdate) throws Exception
    {  
    	double temp = actualPSI;
    	if (temp == 0)
         {
    		
           sens.setCalibration1(0x8000);
           sens.setActionControlCommands(getCalibrationValue(sens.SensorID,sens.getCalibration1() * 1.0));
         }
        else
         {
        
    	     temp = temp / 1.0;
             temp = temp * 10;
             temp = Math.round(temp);        
           
        
           sens.setCalibration2((int)(temp) | 0x8000);
           sens.setActionControlCommands(getCalibrationValue(sens.SensorID,sens.getCalibration2() * 1.0));
        }
       
  	  
  	  sens.setProfileConfig1Dirty(false);
  	  sens.setProfileConfig2Dirty(false);
  	  
  	  sens.setPendingActionControlCommand(sendSensorUpdate);
    }
	
}
