package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.Arrays;
import java.util.List;

public class SeatBase extends _ApplicationBase
{
    public static long MonnitApplicationID = 51;
    public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
    private double KOhms;
    private boolean Occupied;
    public List<Datum> Data;
    
    public double getKOhms()
    {
    		return this.KOhms;
    }
    
    public Boolean getOccupied()
    {
       return this.Occupied;
    }
    
    public void setKOhms(double kohms)
    {
    	this.KOhms = kohms;
    }
    
    public void setOccupied(Boolean occupied)
    {
      this.Occupied = occupied;
    }
    
    public List<Datum> getData()
    {
      return Arrays.asList(new Datum[] { new Datum("KOhms", Double.valueOf(this.KOhms)), new Datum("Occupied", Boolean.valueOf(this.Occupied)) });
    } 
    public void setData(List<Datum> data)
    {
    	this.Data = data;
    	
    }
       // get { return new List<Datum>() { new Datum("KOhms", KOhms), new Datum("Occupied", Occupied) }; }
    
    public static SeatBase Create(int state, byte[] rawData)
    throws Exception
    {
        SeatBase obj = new SeatBase();
        obj.setOccupied(rawData[0] != 0);
        
        
        try {
			obj.KOhms = (BitConverter.ToDouble(rawData, 1)) / 1000.0;
		} catch (Exception e) {
		
		}
        return obj;
    }
    
    public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
    public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
    public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
    public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
    public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
    public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
    public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
    public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
    public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
    public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
    public static long DefaultHysteresis = _ApplicationBase.DefaultHysteresis;
    public static long DefaultMinimumThreshold = 500;
    public static long DefaultMaximumThreshold = 2000;
    public static long DefaultCalibration1 = 5000;
    public static long DefaultCalibration2 = 1;
    public static long DefaultCalibration3 = 1;
    public static long DefaultCalibration4 = 1000;
    public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
    public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
    public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
    public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
    public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
    
    /**
     * <p>
     * sets the ReportInterval as minutes on the sensor object<br/>
     * the next time this sensor checks in it will update
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorHeartBeat(Sensor sens,double heartbeat,Boolean sendSensorUpdate)
    {	
  	  sens.ReportInterval = heartbeat;
  	 
  	  sens.GeneralConfig1Dirty = sendSensorUpdate;
    }
    
    /**
     * <p>
     * sets all data for the GeneralConfig1 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
     * @param activeStateInterval a double value this is the aware state heart beat represented in minutes so aware state heart beat equals 1 that is one minute
     * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateInterval,int transmitIntervalLink,Boolean sendSensorUpdate)
    {
  	  	sens.ReportInterval = (heartBeat);
  	  	sens.setActiveStateInterval((activeStateInterval));
  	  	
  	  	if(transmitIntervalLink < 0)
  	  		transmitIntervalLink = 0;
  	  	else if (transmitIntervalLink > 250)
  	  		transmitIntervalLink=250;
  	  				
  	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
  	    sens.GeneralConfig1Dirty = sendSensorUpdate;
    }
    
    /**
     * <p>
     * sets all data for the GeneralConfig2 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorGeneralConfig2(Sensor sens,int recovery,Boolean sendSensorUpdate)
    { 	
    	 if(recovery < 0)
   		  recovery = 0;
   		
   		if (recovery > 10)
   			recovery = 10;
    	
  	  	sens.Recovery = recovery;
  	    sens.GeneralConfig2Dirty = sendSensorUpdate;
    }
    
    /**
     * <p>
     * sets all data for the ProfileConfig1 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param measurementsPerTransmission the amount of assessments the sensor where take inbetween heartbeats if during these assesments the value is above or below respective triggering thresholds it will send an aware state message. 
     * @param maximumThreshold the maximum temperature the sensor can report before it goes aware
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorProfileConfig1(Sensor sens,int measurementsPerTransmission, double maximumThreshold,Boolean sendSensorUpdate)
    { 
  		  sens.setMeasurementsPerTransmission(measurementsPerTransmission); 
  		  sens.setMaximumThreshold((long) maximumThreshold * 1000);//
  		  if(sens.getMaximumThreshold() < DefaultMinimumThreshold)
  			 sens.setMaximumThreshold(DefaultMinimumThreshold);//
  		 if(sens.getMaximumThreshold() > 10000)
  			sens.setMaximumThreshold(10000);//
  	      sens.setProfileConfig1Dirty(sendSensorUpdate);
    }
    
    /**
     * <p>
     * sets all data for the ProfileConfig1 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param assessmentInterval How often the sensor checks (in seconds) to see if the seat is occupied. 
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorProfileConfig2(Sensor sens,double assessmentInterval,Boolean sendSensorUpdate) throws Exception
    { 
  	  sens.setCalibration1((long)assessmentInterval * 1000);
  	  sens.setProfileConfig2Dirty(sendSensorUpdate);  
    }
}

