package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


public class ShortRangeAsset extends _ApplicationBase
{
    
   public static long MonnitApplicationID = 85;
   public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
  boolean Present;
   public List<Datum> Data;
   
   public List<Datum> getData()
   {
   	return Arrays.asList(new Datum[] { new Datum("Present", Boolean.valueOf(Present)) });
   }
   
   public static ShortRangeAsset Create(int state, Byte[] rawData)
   {
	   ShortRangeAsset obj = new ShortRangeAsset();
	   obj.Present = true;
	   return obj;
   }
   
   public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
   public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
   public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
   public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
   public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
   public static int DefaultRetryCount = 0;
   public static int DefaultRecovery = 0;
   public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
   public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
   public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
   public static long DefaultHysteresis = 4294967295L;
   public static long DefaultMinimumThreshold = _ApplicationBase.DefaultMinimumThreshold;
   public static long DefaultMaximumThreshold = _ApplicationBase.DefaultMaximumThreshold;
   public static long DefaultCalibration1 = 0;
   public static long DefaultCalibration2 = 3;
   public static long DefaultCalibration3 = 0;
   public static long DefaultCalibration4 = _ApplicationBase.DefaultCalibration4;
   public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
   public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
   public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
   public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
   public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
   
   /**
    * <p>
    * sets the ReportInterval as minutes on the sensor object<br/>
    * the next time this sensor checks in it will update
    * </p>
    * 
    * @param sens the Sensor object to be updated 
    * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
    */
   public static void UpdateSensorHeartBeat(Sensor sens,double heartBeat,Boolean sendSensorUpdate)
   { 	  
	   sens.ReportInterval = (heartBeat);	  
	   sens.setActiveStateInterval(sens.ReportInterval); 
 	  	
 	   sens.GeneralConfig1Dirty = sendSensorUpdate;
   }
   
   /**
    * <p>
    * sets all data for the GeneralConfig1 configuration available on the sensor
    * </p>
    * 
    * @param sens the Sensor object to be updated 
    * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
    * @param transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
    */
   public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,int transmitIntervalLink,Boolean sendSensorUpdate)
   {
 	  sens.ReportInterval = (heartBeat);	  
 	  sens.setActiveStateInterval(sens.ReportInterval); 
 	  	 
 	  if(transmitIntervalLink < 0)
 	    transmitIntervalLink = 0;
 	  else if (transmitIntervalLink > 250)
 	    transmitIntervalLink=250;
 	  				
 	  sens.TransmitIntervalLink = transmitIntervalLink;  	 	
 	  sens.GeneralConfig1Dirty = sendSensorUpdate;
   }
  
   /**
    * <p>
    * sets all data for the GeneralConfig2 configuration available on the sensor
    * </p>
    * 
    * @param sens the Sensor object to be updated 
    * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
    */
   public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
   { 	
 	  if(recovery < 0)
 		  recovery = 0;
 		
 		if (recovery > 10)
 			recovery = 10;
 	  
 	  	sens.Recovery = recovery;
 	    sens.GeneralConfig2Dirty = sendSensorUpdate;
   }
   
   /**
    * <p>
    * sets all data for the ProfileConfig2 configuration available on the sensor
    * </p>
    * 
    * @param sens the Sensor object to be updated 
    * @param powerReduction Sets the power reduction allowing communication close to the gateway. (0 dB = 0, 6 dB = 1, 12 dB = 2, and 18 dB = 3)
    * @param doubleTransmit Determines if the sensor should first ping to obtain updated signal strength before transmitting.
    * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
    */
   public static void UpdateSensorProfileConfig2(Sensor sens,int powerReduction,Boolean doubleTransmit,Boolean sendSensorUpdate )
   { 
         
        if(powerReduction > 3)
           powerReduction = 3;
        if(powerReduction < 0)
           powerReduction = 0;
           
        sens.setCalibration2(powerReduction);
        if(doubleTransmit)
           sens.setCalibration3(1);
        else
        	sens.setCalibration3(0);
        
 	      sens.setProfileConfig2Dirty(sendSensorUpdate);
   }
   
}

