package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Command;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;
import com.monnit.mine.MonnitMineAPI.enums.eConfigSection;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SmartRangerBase
  extends _ApplicationBase
{
  public static long MonnitApplicationID = 45L;
  public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
  public int DevCount;
  public int RxMsgBySubnet;
  public int UplinkFails;
  public int MSGQueueCount;
  public int SubNetChannel;
  public List<Datum> Data;
  
  public int getDevCount()
  {
    return this.DevCount;
  }
  
  public void setDevCount(int devCount)
  {
    this.DevCount = devCount;
  }
  
  public int getRxMsgBySubnet()
  {
    return this.RxMsgBySubnet;
  }
  
  public void setRxMsgBySubnet(int rxMsgBySubnet)
  {
    this.RxMsgBySubnet = rxMsgBySubnet;
  }
  
  public int getUplinkFails()
  {
    return this.UplinkFails;
  }
  
  public void setUplinkFails(int uplinkFails)
  {
    this.UplinkFails = uplinkFails;
  }
  
  public int getMSGQueueCount()
  {
    return this.MSGQueueCount;
  }
  
  public void setMSGQueueCount(int mSGQueueCount)
  {
    this.MSGQueueCount = mSGQueueCount;
  }
  
  public int getSubNetChannel()
  {
    return this.SubNetChannel;
  }
  
  public void setSubNetChannel(int subNetChannel)
  {
    this.SubNetChannel = subNetChannel;
  }
  
  public List<Datum> getData()
  {
    return Arrays.asList(new Datum[] { new Datum("DevCount", Integer.valueOf(this.DevCount)), new Datum("RxMsgBySubnet", Integer.valueOf(this.RxMsgBySubnet)), new Datum("UplinkFails", Integer.valueOf(this.UplinkFails)), new Datum("MSGQueueCount", Integer.valueOf(this.MSGQueueCount)), new Datum("SubNetChannel", Integer.valueOf(this.SubNetChannel)) });
  }
  
  public void setData(List<Datum> data)
  {
    this.Data = data;
  }
  
  public static SmartRangerBase Create(int state, byte[] rawData)
    throws Exception
  {
    SmartRangerBase obj = new SmartRangerBase();
    obj.DevCount = BitConverter.ToUInt16(rawData, 0);
    obj.RxMsgBySubnet = BitConverter.ToUInt16(rawData, 2);
    obj.UplinkFails = BitConverter.ToUInt16(rawData, 4);
    obj.MSGQueueCount = BitConverter.ToUInt16(rawData, 6);
    obj.SubNetChannel = rawData[8];
    return obj;
  }
  
  public static double DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
  public static int DefaultStandardMessageDelay = 100;
  public static int DefaultTransmitIntervalLink = 101;
  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
  public static int DefaultRetryCount = 1;
  public static int DefaultRecovery = 10;
  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
  public static double ActiveStateInterval = 1.0D;
  public static double ReportInterval = 1.0D;
  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
  public static long DefaultHysteresis = 0L;
  public static long DefaultMinimumThreshold = 3L;
  public static long DefaultMaximumThreshold = 20L;
  public static long DefaultCalibration1 = _ApplicationBase.DefaultCalibration1;
  public static long DefaultCalibration2 = _ApplicationBase.DefaultCalibration2;
  public static long DefaultCalibration3 = 86400L;
  public static long DefaultCalibration4 = 6L;
  public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
  public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
  public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
  public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
  public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
  
  public static List<byte[]> CalibrateFrame(int sensorID, double temp)
  {
	  List<byte[]> commandList = new ArrayList();
	    Command comm = new Command();
      //Calibrate Temp
      int length = 8;

      byte[] api = new byte[length];
      api[0] = 0x74;// MSAPI.CMD_APPCMD_REQUEST;
      System.arraycopy(BitConverter.getBytes(sensorID), 0, api, 1, 4);
      api[5] = (byte)((int)temp);
     
      commandList.add(api);
      
      byte[] api1 = new byte[6];
      api1[0] = 0x70;// Read Sector
      System.arraycopy(BitConverter.getBytes(sensorID), 0, api1, 1, 4);
      api1[5] = (byte)29;
      
      commandList.add(api1);
      
      return commandList;
  }


  /**
   * <p>
   * sets the ReportInterval as minutes on the sensor object<br/>
   * the next time this sensor checks in it will update
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorHeartBeat(Sensor sens,double heartbeat,Boolean sendSensorUpdate)
  {	
	  sens.ReportInterval = heartbeat;// * 60;
	 
	  sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
  { 	
	  if(recovery < 0)
		  recovery = 0;
		
		if (recovery > 10)
			recovery = 10;
	  
	  	sens.Recovery = recovery;
	    sens.GeneralConfig2Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sends a new sensor list and channel to talk on
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void SendReform(Sensor sens,Boolean sendSensorUpdate)
  {	
	  sens.setMeasurementsPerTransmission(0x06);
	  sens.setProfileConfig1Dirty(false);
	  sens.setProfileConfig2Dirty(false);
	  sens.setActionControlCommands(CalibrateFrame(sens.SensorID,sens.getMeasurementsPerTransmission() * 1.0));
	  sens.setPendingActionControlCommand(sendSensorUpdate);
  }
  
  /**
   * <p>
   * sends just a sensor list
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void SendSensorList(Sensor sens,Boolean sendSensorUpdate)
  {	
	  sens.setMeasurementsPerTransmission(0x04);
	  sens.setProfileConfig1Dirty(false);
	  sens.setProfileConfig2Dirty(false);
	  sens.setActionControlCommands(CalibrateFrame(sens.SensorID,sens.getMeasurementsPerTransmission() * 1.0));
	  sens.setPendingActionControlCommand(sendSensorUpdate);
  }
  /**
   * <p>
   * Deletes the message que 
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void SendEraseQue(Sensor sens,Boolean sendSensorUpdate)
  {	
	  sens.setMeasurementsPerTransmission(0x03);
	  sens.setProfileConfig1Dirty(false);
	  sens.setProfileConfig2Dirty(false);
	  sens.setActionControlCommands(CalibrateFrame(sens.SensorID,sens.getMeasurementsPerTransmission() * 1.0));
	  sens.setPendingActionControlCommand(sendSensorUpdate);
  }
  
  public static Boolean IgnoreActiveStateInSimpleEditScreen = Boolean.valueOf(true);
}

