package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Command;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.List;

public class TemperatureRTDLowBase
  extends TemperatureBase
{
  public static long MonnitApplicationID = 46L;
  public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
  
  public static TemperatureRTDLowBase Create(int state, byte[] rawData)
    throws Exception
  {
    TemperatureRTDLowBase obj = new TemperatureRTDLowBase();
    obj.Temp = Double.valueOf(BitConverter.toInt16Backwards(rawData, 0) / 10.0D);
    return obj;
  }
  
  public static double DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
  public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
  public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
  public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
  public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
  public static long DefaultHysteresis = _ApplicationBase.DefaultHysteresis;
  public static long DefaultMinimumThreshold = -2500L;
  public static long DefaultMaximumThreshold = 2500L;
  public static long DefaultCalibration1 = 0L;
  public static long DefaultCalibration2 = 10000L;
  public static long DefaultCalibration3 = _ApplicationBase.DefaultCalibration3;
  public static long DefaultCalibration4 = _ApplicationBase.DefaultCalibration4;
  public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
  public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
  public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
  public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
  public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
  
  public static List<byte[]> CalibrateFrame(int sensorID, double temp)
  {
    return TemperatureBase.CalibrateFrame(sensorID, temp);
  }
  
  /**
   * <p>
   * sets the ReportInterval as minutes on the sensor object<br/>
   * the next time this sensor checks in it will update
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorHeartBeat(Sensor sens,double heartBeat,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	  	
	  	if(sens.ReportInterval < sens.getActiveStateInterval())
			  sens.setActiveStateInterval(sens.ReportInterval);
	  	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
   * @param activeStateInterval a double value this is the aware state heart beat represented in minutes so aware state heart beat equals 1 that is one minute
   * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateInterval,int transmitIntervalLink,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval((activeStateInterval));
	  	
	  	if(transmitIntervalLink < 0)
	  		transmitIntervalLink = 0;
	  	else if (transmitIntervalLink > 250)
	  		transmitIntervalLink=250;
	  				
	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
  { 	
	  if(recovery < 0)
		  recovery = 0;
		
		if (recovery > 10)
			recovery = 10;
	  
	  	sens.Recovery = recovery;
	    sens.GeneralConfig2Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the ProfileConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param measurementsPerTransmission the amount of assessments the sensor where take inbetween heartbeats if during these assesments the value is above or below respective triggering thresholds it will send an aware state message. 
   * @param hysteresis A buffer to prevent the sensor from bouncing between standard operation and aware state when the asseessments are very close to a threashold. for example, if a maximum threashold is set to 90 degrees and a hysteresis of 1 degree, then once the sensor takes an assessment of 90.0 degrees and enters the aware state it will remain in the aware state until the temperature readings drop to 89.0 degrees
   * @param minimumThreshold the minimum temperature the sensor can report before it goes aware
   * @param maximumThreshold the maximum temperature the sensor can report before it goes aware
   * @param isFahrenheit data is always stored as Celsius so if you are inputting values as Fahrenheit you will want to set this boolean value to true
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorProfileConfig1(Sensor sens,int measurementsPerTransmission,double hysteresis,double minimumThreshold,double maximumThreshold,boolean isFahrenheit,Boolean sendSensorUpdate )
  { 
	  sens.setMeasurementsPerTransmission(measurementsPerTransmission);

      if (hysteresis < 0) {
          hysteresis = 0;
      }
      
      if(hysteresis > 5){
         hysteresis = 5;
      }
      
      sens.setHysteresis((long) (hysteresis * (isFahrenheit ? (5.0 / 9.0) : 1) * 10));
      sens.setMinimumThreshold((long) (isFahrenheit ? Extensions.ToCelsius(minimumThreshold) * 10 : (minimumThreshold * 10)));
      if (sens.getMinimumThreshold() < DefaultMinimumThreshold) {
          sens.setMinimumThreshold(DefaultMinimumThreshold);
      }

      if (sens.getMinimumThreshold() > DefaultMaximumThreshold) {
          sens.setMinimumThreshold(DefaultMaximumThreshold);
      }
      sens.setMaximumThreshold((long) (isFahrenheit ? Extensions.ToCelsius(maximumThreshold) * 10 : (maximumThreshold * 10)));//
      if (sens.getMaximumThreshold() < DefaultMinimumThreshold) {
          sens.setMaximumThreshold(DefaultMinimumThreshold);
      }

      if (sens.getMaximumThreshold() > DefaultMaximumThreshold) {
          sens.setMaximumThreshold(DefaultMaximumThreshold);
      }

      if (sens.getMinimumThreshold() > sens.getMaximumThreshold()) {
          sens.setMaximumThreshold(sens.getMinimumThreshold());
      }
      sens.setProfileConfig1Dirty(sendSensorUpdate);
  }
  
  /**
   * <p>
   * Calibrates the sensor to correct values set by the user
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param actualTemp what the actual temperature of the room the sensor is located in is
   * @param isFahrenheit data is always stored as Celsius so if you are inputting values as Fahrenheit you will want to set this boolean value to true
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorCalibration(Sensor sens,double actualTemp, boolean isFahrenheit,Boolean sendSensorUpdate) throws Exception
  { long temp = Long.MIN_VALUE;
  
     if(isFahrenheit)
    	 temp = (long)((actualTemp - 32) * ( 5.0 / 9.0) * 10);
     else
    	 temp = (long)(actualTemp  * 10);
  
     if(temp < DefaultMinimumThreshold || temp > DefaultMaximumThreshold)
    	 throw new Exception("Calibration value out of range!");
     
	  sens.setCalibration1(temp);
	  sens.setProfileConfig1Dirty(false);
	  sens.setProfileConfig2Dirty(false);
	  sens.setActionControlCommands(CalibrateFrame(sens.SensorID,sens.getCalibration1() * 1.0));
	  sens.setPendingActionControlCommand(sendSensorUpdate);
  }
}
