package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class TiltBase extends _ApplicationBase
{
    public static long MonnitApplicationID = 75;
    public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
    public double Xval;
    public double Yval;
    public double Zval;
    
    public List<Datum> Data;
    
    public List<Datum> getData()
    {
    	  return Arrays.asList(new Datum[] { new Datum("Xval", Double.valueOf(Xval)), new Datum("Yval", Double.valueOf(Yval)), new Datum("Zval", Double.valueOf(Zval)) });
    	  
    }
    
  
    public static TiltBase Create(int state, byte[] rawData) throws Exception
    {
        TiltBase obj = new TiltBase();
        obj.Xval = BitConverter.toInt16(rawData, 0) / 1000.0;
        obj.Yval = BitConverter.toInt16(rawData, 0 + 2) / 1000.0;
        obj.Zval = BitConverter.toInt16(rawData, 0 + 4) / 1000.0;
        return obj;
    }
    
    public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
    public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
    public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
    public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
    public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
    public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
    public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
    public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
    public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
    public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
    public static long DefaultHysteresis = 0;
    public static long DefaultMinimumThreshold = 0xFF4C00B4;//pitch
    public static long DefaultMaximumThreshold = 0xFF4C00B4;  //Roll
    public static long DefaultCalibration1 = 0;
    public static long DefaultCalibration2 = 0;
    public static long DefaultCalibration3 = 0;
    public static long DefaultCalibration4 = 0;
    public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
    public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
    public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
    public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
    public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
    
    public static void SetPitchMin(Sensor sens, int value)
    {
        int pitchMin = (int)((int)(sens.getMinimumThreshold()) & 0x0000FFFF);
        sens.setMinimumThreshold( pitchMin | ((int)value << 16));
    }

    public static int GetPitchMin(Sensor sens)
    {

        int tmp = (int)(sens.getMinimumThreshold());
        tmp = (int)(tmp & 0xFFFF0000);
        short tmp2 = (short)(int)(tmp >> 16);
        int pitchMin = tmp2;

        return pitchMin;
    }

    public static void SetPitchMax(Sensor sens, int value)
    {
        int pitchMax = (int)((int)(sens.getMinimumThreshold()) & 0xFFFF0000);
        sens.setMinimumThreshold(pitchMax | ((int)value));
    }

    public static int GetPitchMax(Sensor sens)
    {
        int pitchMax = (short)(int)((int)(sens.getMinimumThreshold()) & 0x0000FFFF);
        return pitchMax;
    }

    public static void SetRollMin(Sensor sens, int value)
    {
        int rollMin = (int)((int)(sens.getMaximumThreshold()) & 0x0000FFFF);

        sens.setMaximumThreshold(rollMin | ((int)value << 16));
    }

    public static int GetRollMin(Sensor sens)
    {
        int tmp = (int)(sens.getMaximumThreshold());
        tmp = (int)(tmp & 0xFFFF0000);
        short tmp2 = (short)(int)(tmp >> 16);
        int rollMin = tmp2;


        return rollMin;
    }

    public static void SetRollMax(Sensor sens, int value)
    {
        int rollMax = (int)((int)(sens.getMaximumThreshold()) & 0xFFFF0000);
        sens.setMaximumThreshold(rollMax | (int)(value));
    }

    public static int GetRollMax(Sensor sens)
    {
        int rollMax = (short)(int)((int)(sens.getMaximumThreshold()) & 0x0000FFFF);
        return rollMax;
    }
    
    /**
     * <p>
     * sets the ReportInterval as minutes on the sensor object<br/>
     * the next time this sensor checks in it will update
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorHeartBeat(Sensor sens,double heartBeat,Boolean sendSensorUpdate)
    {
  	  	sens.ReportInterval = (heartBeat);
  	  	
  	  	if(sens.ReportInterval < sens.getActiveStateInterval())
  			  sens.setActiveStateInterval(sens.ReportInterval);
  	  	
  	    sens.GeneralConfig1Dirty = sendSensorUpdate;
    }
    
    /**
     * <p>
     * sets all data for the GeneralConfig1 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
     * @param activeStateInterval a double value this is the aware state heart beat represented in minutes so aware state heart beat equals 1 that is one minute
     * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateInterval,int transmitIntervalLink,Boolean sendSensorUpdate)
    {
  	  	sens.ReportInterval = (heartBeat);
  	  	sens.setActiveStateInterval((activeStateInterval));
  	  	
  	  	if(sens.ReportInterval < sens.getActiveStateInterval())
  			  sens.setActiveStateInterval(sens.ReportInterval);
  	  	
  	  	if(transmitIntervalLink < 0)
  	  		transmitIntervalLink = 0;
  	  	else if (transmitIntervalLink > 250)
  	  		transmitIntervalLink=250;
  	  				
  	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
  	    sens.GeneralConfig1Dirty = sendSensorUpdate;
    }
    
    /**
     * <p>
     * sets all data for the GeneralConfig2 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
    { 	
  	  if(recovery < 0)
  		  recovery = 0;
  		
  		if (recovery > 10)
  			recovery = 10;
  	  
  	  	sens.Recovery = recovery;
  	    sens.GeneralConfig2Dirty = sendSensorUpdate;
    }
   
    /**
     * <p>
     * sets all data for the ProfileConfig1 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param hysteresis A buffer to prevent the sensor from bouncing between standard operation and aware state when the assessments are very close to a threshold. for example, if a maximum threshold is set to 90 degrees and a hysteresis of 1 degree, then once the sensor takes an assessment of 90.0 degrees and enters the aware state it will remain in the aware state until the temperature readings drop to 89.0 degrees
     * @param pitchMinimum the minimum pitch the sensor can report before it goes aware. (value range: -180 - +180)
     * @param pitchMaximum the maximum pitch the sensor can report before it goes aware. (value range: -180 - +180)
     * @param rollMinimum the minimum roll the sensor can report before it goes aware. (value range: -180 - +180)
     * @param rollMaximum the maximum roll the sensor can report before it goes aware. (value range: -180 - +180)
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorProfileConfig1(Sensor sens,double hysteresis,int pitchMinimum,int pitchMaximum,int rollMinimum,int rollMaximum,Boolean sendSensorUpdate )
    { 
  		  //sens.setMeasurementsPerTransmission(measurementsPerTransmission);
    	if(hysteresis <0)
    		hysteresis = 0;
    	if(hysteresis > 5)
    		hysteresis = 5;
    	
  		sens.setHysteresis((long)(hysteresis));
  		 
  		if(pitchMinimum <-180)
  			pitchMinimum = -180;
    	if(pitchMinimum > 180)
    		pitchMinimum = 180;
  		  
  	    SetPitchMin(sens,pitchMinimum);
  		
  	    if(pitchMaximum <-180)
  			pitchMaximum = -180;
       	if(pitchMaximum > 180)
    		pitchMaximum = 180;
    	if(pitchMaximum < pitchMinimum)
    		pitchMaximum = pitchMinimum;
    	
  		SetPitchMax(sens,pitchMaximum);
  		  
  		if(rollMinimum <-180)
  			rollMinimum = -180;
    	if(rollMinimum > 180)
    		rollMinimum = 180;
  		
    	SetRollMin(sens,rollMinimum);
  		  
  		if(rollMaximum <-180)
  			 rollMaximum = -180;
     	if(rollMaximum > 180)
  	       	rollMaximum = 180;
  	    if(rollMaximum < rollMinimum)
  	    	rollMaximum = rollMinimum;
  	
  	    SetRollMax(sens,rollMaximum);
  		  
  	      sens.setProfileConfig1Dirty(sendSensorUpdate);
    }
    
}

