package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.Arrays;
import java.util.List;

public class VibrationBase
  extends _ApplicationBase
{
  public static long MonnitApplicationID = 5L;
  public static eApplicationProfileType ProfileType = eApplicationProfileType.Trigger;
  public Boolean Vibrating;
  public List<Datum> Data;
  
  public Boolean getVibrating()
  {
    return this.Vibrating;
  }
  
  public void setVibrating(Boolean vibrating)
  {
    this.Vibrating = vibrating;
  }
  
  public List<Datum> getData()
  {
    return Arrays.asList(new Datum[] { new Datum("Vibrating", this.Vibrating) });
  }
  
  public void setData(List<Datum> data)
  {
    this.Data = data;
  }
  
  public static VibrationBase Create(int state, byte[] rawData)
    throws Exception
  {
    VibrationBase obj = new VibrationBase();
    obj.Vibrating = BitConverter.toBool(rawData[0]);
    return obj;
  }
  
  public static double DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
  public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
  public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
  public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
  public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
  public static long DefaultHysteresis = _ApplicationBase.DefaultHysteresis;
  public static long DefaultMinimumThreshold = _ApplicationBase.DefaultMinimumThreshold;
  public static long DefaultMaximumThreshold = _ApplicationBase.DefaultMaximumThreshold;
  public static long DefaultCalibration1 = _ApplicationBase.DefaultCalibration1;
  public static long DefaultCalibration2 = _ApplicationBase.DefaultCalibration2;
  public static long DefaultCalibration3 = _ApplicationBase.DefaultCalibration3;
  public static long DefaultCalibration4 = _ApplicationBase.DefaultCalibration4;
  public static int DefaultEventDetectionType = 0;
  public static int DefaultEventDetectionPeriod = 100;
  public static int DefaultEventDetectionCount = 18;
  public static int DefaultRearmTime = 1;
  public static int DefaultBiStable = 1;
  
  /**
   * <p>
   * sets the ReportInterval as minutes on the sensor object<br/>
   * the next time this sensor checks in it will update
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorHeartBeat(Sensor sens,double heartbeat,Boolean sendSensorUpdate)
  {	
	  sens.ReportInterval = heartbeat;
	 
	  if(sens.ReportInterval < sens.getActiveStateInterval())
		  sens.setActiveStateInterval(sens.ReportInterval);
	  
	  sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
   * @param activeStateInterval a double value this is the aware state heart beat represented in minutes so aware state heart beat equals 1 that is one minute
   * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateInterval,int transmitIntervalLink,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval((activeStateInterval));
	  	
	  	
	  	if(sens.ReportInterval < sens.getActiveStateInterval())
			  sens.setActiveStateInterval(sens.ReportInterval);
	  	
	  	if(transmitIntervalLink < 0)
	  		transmitIntervalLink = 0;
	  	else if (transmitIntervalLink > 250)
	  		transmitIntervalLink=250;
	  				
	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig2(Sensor sens,int recovery,Boolean sendSensorUpdate)
  { 	
	  
	  if(recovery < 0)
		  recovery = 0;
		
		if (recovery > 10)
			recovery = 10;
		
	  	sens.Recovery = recovery;
	    sens.GeneralConfig2Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param eventDectionType Determines which reading of the sensor will trigger the aware state. (starts = 0, stops = 1, state change = 2)
   * @param sensitivity Set how much vibration is required before triggering the sensor, lower numbers are more sensitive. ( value range: 25 - 1000)
   * @param rearmTime Time in seconds after a triggering event that the sensor will wait before re-arming itself.(value range: 1 - 600)
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorProfileConfig1(Sensor sens,int eventDectionType,int sensitivity,int rearmTime,Boolean sendSensorUpdate)
  { 
	  //motion = 0
	  //no motion = 1
	  //state change = 2
	  if(eventDectionType < 0)
		  eventDectionType = 0;
		if(eventDectionType>2)
			eventDectionType = 2;
	sens.setEventDetectionType(eventDectionType);
	//aware reading = 1
	//all state changes = 0
		//  sens.setBiStable(biStable);
    if(sensitivity < 50)
    	sensitivity = 50;
	if(sensitivity>2000)
		sensitivity = 2000;
	
    sens.setEventDetectionPeriod(sensitivity);
	
    if(rearmTime < 1)
		rearmTime = 1;
	
	if (rearmTime > 600)
		rearmTime = 600;
	
	  sens.setRearmTime(rearmTime); 
	  
	  sens.setProfileConfig1Dirty(sendSensorUpdate);
	 
  }
}

