package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.Arrays;
import java.util.List;

public class VibrationMeterBase
  extends _ApplicationBase
{
  public static long MonnitApplicationID = 95L;
  public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
  public double xSpeed;
  public double ySpeed;
  public double zSpeed;
  public double xFrequency;
  public double yFrequency;
  public double zFrequency;
  public double DutyCycle;
  public List<Datum> Data;
  
  public List<Datum> getData()
  {
    return Arrays.asList(new Datum[] { new Datum("X-Axis Speed", Double.valueOf(this.xSpeed)), new Datum("X-Axis Frequency", Double.valueOf(this.xFrequency)), new Datum("Y-Axis Speed", Double.valueOf(this.ySpeed)), new Datum("Y-Axis Frequency", Double.valueOf(this.yFrequency)), new Datum("Z-Axis Speed", Double.valueOf(this.zSpeed)), new Datum("Z-Axis Frequency", Double.valueOf(this.zFrequency)), new Datum("Duty Cycle", Double.valueOf(this.DutyCycle)) });
  }
  
  public static VibrationMeterBase Create(int state, byte[] rawData)
    throws Exception
  {
    VibrationMeterBase obj = new VibrationMeterBase();
    Extensions.debug(Extensions.hexStr(rawData), 52);
    obj.xSpeed = (rawData[0] / 10.0D);
    obj.ySpeed = (rawData[1] / 10.0D);
    obj.zSpeed = (rawData[2] / 10.0D);
    obj.xFrequency = rawData[3];
    obj.yFrequency = rawData[4];
    obj.zFrequency = rawData[5];
    obj.DutyCycle = rawData[6];
    return obj;
  }
  
  public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
  public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
  public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
  public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
  public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
  public static long DefaultHysteresis = 0L;
  public static long DefaultMinimumThreshold = 0L;
  public static long DefaultMaximumThreshold = 0L;
  public static long DefaultCalibration1 = 0L;
  public static long DefaultCalibration2 = 60L;
  public static long DefaultCalibration3 = 2L;
  public static long DefaultCalibration4 = 62L;
  public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
  public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
  public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
  public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
  public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
  
  /**
   * <p>
   * sets the ReportInterval as minutes on the sensor object<br/>
   * the next time this sensor checks in it will update
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorHeartBeat(Sensor sens,double heartBeat,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);	
	  	sens.setActiveStateInterval((heartBeat));
	  	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
   * @param activeStateInterval a double value this is the aware state heart beat represented in minutes so aware state heart beat equals 1 that is one minute
   * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,int transmitIntervalLink,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval((heartBeat));
	  	
	  	if(transmitIntervalLink < 0)
	  		transmitIntervalLink = 0;
	  	else if (transmitIntervalLink > 250)
	  		transmitIntervalLink=250;
	  				
	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
  { 	
	  if(recovery < 0)
		  recovery = 0;
		
		if (recovery > 10)
			recovery = 10;
	  
	  	sens.Recovery = recovery;
	    sens.GeneralConfig2Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param measurementMethod Determines the method of measurement of the sensor. Presents axes data in RMS, Absolute Mean, or Peak only Data. (RMS = 0, Peak Only = 1, Absolute Mean = 2)
   * @param assessmentInterval Determines how often the sensor takes a measurement. For example, if the heart beat is 10 minutes and the assessment interval is 1 minute, the sensor will make a measurement every minute, but only report every 10 minutes.
   * @param vibrationIntensity this value determines the g-forces required to wake up the sensor to take a measurement. if the g-forces are not greater than this, the sensor will remain asleep. (Values range: 1 - 127, each unit is .063 g-forces)
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorProfileConfig2(Sensor sens,int measurementMethod,int assessmentInterval,int vibrationIntensity,Boolean sendSensorUpdate )
  { 
	     if (measurementMethod < 0)
	    	 measurementMethod = 0;
	     if(measurementMethod>2)
	    	 measurementMethod=2;
		  
	     sens.setCalibration1(measurementMethod); 
		  
		 if (assessmentInterval < 1)
		     assessmentInterval = 1 * 60;
		 else if(assessmentInterval > sens.ReportInterval)
		     assessmentInterval= (int)sens.ReportInterval * 60;
		 else
		     assessmentInterval = assessmentInterval * 60;
		  
		 sens.setCalibration2(assessmentInterval);
		 
		 if(vibrationIntensity < 0)
			 vibrationIntensity = 0;
		 else if (vibrationIntensity > 127)
			 vibrationIntensity = 127;
		 
		 sens.setCalibration3(vibrationIntensity);

	     sens.setProfileConfig2Dirty(sendSensorUpdate);
  }
  
  
}
