package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Command;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class WeightedActivityBase
  extends _ApplicationBase
{
  public static long MonnitApplicationID = 60L;
  public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
  public int Weight;
  public List<Datum> Data;
  
  public int getWeight()
  {
    return this.Weight;
  }
  
  public void setWeight(int weight)
  {
    this.Weight = weight;
  }
  
  public List<Datum> getData()
  {
    return Arrays.asList(new Datum[] { new Datum("Weight", Integer.valueOf(this.Weight)) });
  }
  
  public void setData(List<Datum> data)
  {
    this.Data = data;
  }
  
  public static WeightedActivityBase Create(int state, Byte[] rawData)
  {
    WeightedActivityBase obj = new WeightedActivityBase();
    obj.Weight = rawData[0].byteValue();
    return obj;
  }
  
  public static double DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
  public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
  public static int DefaultTransmitIntervalLink = 8;
  public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
  public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
  public static int DefaultRetryCount = 2;
  public static int DefaultRecovery = 2;
  public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
  public static double DefaultReportInterval = 10.0D;
  public static double DefaultActiveStateInterval = 10.0D;
  public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
  public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
  public static long DefaultHysteresis = 0L;
  public static long DefaultMinimumThreshold = 0L;
  public static long DefaultMaximumThreshold = 0L;
  public static long DefaultCalibration1 = 60L;
  public static long DefaultCalibration2 = 1L;
  public static long DefaultCalibration3 = 126L;
  public static long DefaultCalibration4 = 15L;
  public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
  public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
  public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
  public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
  public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
  
  public static List<byte[]> ClearStoredDataFrame(long sensorID)
  {
	  List<byte[]> commandList = new ArrayList();
    
    
    int length = 6;
    byte[] api = new byte[length];
    api[0] = 116;
    System.arraycopy(BitConverter.getBytes(sensorID), 0, api, 1, 4);
    api[5] = -55;
    
    
    
    commandList.add(api);
    
    return commandList;
  }
  
  public static List<byte[]> SetDataLogDeliveryPriority(long sensorID, Boolean highPriority)
  {
	  List<byte[]> commandList = new ArrayList();
    Command comm = new Command();
    
    int length = 6;
    byte[] api = new byte[length];
    api[0] = 116;
    System.arraycopy(BitConverter.getBytes(sensorID), 0, api, 1, 4);
    if (highPriority.booleanValue()) {
      api[5] = -54;
    } else {
      api[5] = -53;
    }
    
    
    commandList.add(api);
    
    return commandList;
  }
  
  public static List<byte[]> SetStudyMode(long sensorID, Boolean startStudy)
  {
	  List<byte[]> commandList = new ArrayList();
   
    
    int length = 6;
    byte[] api = new byte[length];
    api[0] = 116;
    System.arraycopy(BitConverter.getBytes(sensorID), 0, api, 1, 4);
    if (startStudy.booleanValue()) {
      api[5] = -52;
    } else {
      api[5] = -51;
    }
   
    
    commandList.add(api);
    
    return commandList;
  }
  
  /**
   * <p>
   * sets the ReportInterval as minutes on the sensor object<br/>
   * the next time this sensor checks in it will update
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
   public static void UpdateSensorHeartBeat(Sensor sens,double heartBeat,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval((heartBeat));	  	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
   
  /**
   * <p>
   * sets all data for the GeneralConfig1 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
   * @param transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,int transmitIntervalLink,Boolean sendSensorUpdate)
  {
	  	sens.ReportInterval = (heartBeat);
	  	sens.setActiveStateInterval((heartBeat));
	  	
	  	if(transmitIntervalLink < 0)
	  		transmitIntervalLink = 0;
	  	else if (transmitIntervalLink > 250)
	  		transmitIntervalLink=250;
	  				
	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
	    sens.GeneralConfig1Dirty = sendSensorUpdate;
  }
  
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
  { 	
	  if(recovery < 0)
		  recovery = 0;
		
		if (recovery > 10)
			recovery = 10;
	  
	  	sens.Recovery = recovery;
	    sens.GeneralConfig2Dirty = sendSensorUpdate;
  }
 
  /**
   * <p>
   * sets all data for the GeneralConfig2 configuration available on the sensor
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param minorInterval Time in seconds representing the amount of time is tallied for either no motion or motion value.(value range: 20 -600)
   * @param sensitivity Number of pulsed in every Minor Interval needed to qualify occupancy. (value range 1 - 500)
   * @param maximumReading Scalar multiplied with the percentage of the tallied motion time. (value range 10 - 250)
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorProfileConfig2(Sensor sens,int minorInterval,int sensitivity,int maximumReading,Boolean sendSensorUpdate )
  { 
	     if (minorInterval < 20)
	    	 minorInterval = 20;
	     if(minorInterval > 600)
	    	 minorInterval = 600;
		  
	     sens.setCalibration1(minorInterval); 
		  
		 if (sensitivity < 1)
			 sensitivity = 1 ;
		  if(sensitivity > 500)
			 sensitivity= 500;
  
		 sens.setCalibration2(sensitivity);
		 
		 if(maximumReading < 10)
			 maximumReading = 10;
		 else if (maximumReading > 250)
			 maximumReading = 250;
		 
		 sens.setCalibration3(maximumReading);

	     sens.setProfileConfig2Dirty(sendSensorUpdate);
  }
  
  /**
   * <p>
   * Calibrates the sensor to correct values set by the user
   * </p>
   * 
   * @param sens the Sensor object to be updated 
   * @param command Specific Command to send(Command Values: 201 - Clear DL Messages, 202 - set high priority DL Delivery, 203 - Set Low Priority DL Delivery, 204 - Study Mode Active, 205 - Study Mode Inactive)
   * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
   */
  public static void UpdateSensorCalibration(Sensor sens,int command,Boolean sendSensorUpdate) throws Exception
  { 
	  switch(command)
	  {
	  	case 201: //set flag 0x10 Clear DL Messages
	  		sens.setHysteresis(sens.getHysteresis() | 0x10);
	  		sens.setActionControlCommands(ClearStoredDataFrame(sens.SensorID));
	  		break;
	  	case 202:   //set flag 0x04 Set High Priority DL Delivery
	  		sens.setHysteresis(sens.getHysteresis() | 0x04);
	  		//clear flag 0x08 Set Low Priority DL Delivery
	  		sens.setHysteresis(sens.getHysteresis() & 0x17);
	  		
	  		sens.setActionControlCommands(SetDataLogDeliveryPriority(sens.SensorID,true));
	  		break;
	  	case 203:   //set flag 0x08 Set Low Priority DL Delivery
	  		sens.setHysteresis(sens.getHysteresis() | 0x08);
	  	    //clear flag 0x02 Study Mode Inactive
	  		sens.setHysteresis(sens.getHysteresis() & 0x1B);
	  		
	  		sens.setActionControlCommands(SetDataLogDeliveryPriority(sens.SensorID,false));
	  		break;
	  	case 204:     //set flag 0x01 Study Mode Active
	  		sens.setHysteresis(sens.getHysteresis() |  0x01);
	  		 //clear flag 0x02 Study Mode Inactive
	  		sens.setHysteresis(sens.getHysteresis() &  0x1D);
	  		
	  		sens.setActionControlCommands(SetStudyMode(sens.SensorID,false));
	  		break;
	  	case 205:     //set flag 0x02 Study Mode Inactive
	  		sens.setHysteresis(sens.getHysteresis() |  0x02);
	  		 //clear flag 0x01 Study Mode Active
	  		sens.setHysteresis(sens.getHysteresis() &  0x1E);
	  		
	  		sens.setActionControlCommands(SetStudyMode(sens.SensorID, true));
	  		break;
	  	default:
	  		break;
	  			
	  }
	
	  //This profile doesn't use the values in Hysteresis/ Min thresh/ Max thresh
      //We just use them to signal which commands to send for Pending Action Control
      //So alway mark this as not dirty so the config page isn't sent.
	  sens.setProfileConfig1Dirty(false);
	  sens.setProfileConfig2Dirty(false);
	  
	  sens.setPendingActionControlCommand(sendSensorUpdate);
  }
}

