package com.monnit.mine.BaseApplication.applicationclasses;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.Sensor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class ZeroToFiveVoltBase extends _ApplicationBase
{
    public static long MonnitApplicationID = 72;
    public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
    public double Volts;
    
 public List<Datum> Data;
    
    public List<Datum> getData()
    {
    	  return Arrays.asList(new Datum[] { new Datum("Volts", Double.valueOf(Volts))});
    	  
    }
   
    public static ZeroToFiveVoltBase Create(int state, byte[] rawData) throws Exception
    {
        ZeroToFiveVoltBase obj = new ZeroToFiveVoltBase();
        obj.Volts = BitConverter.ToUInt16(rawData, 0) / 1000.0;
        return obj;
    }
    
    public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
    public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
    public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
    public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
    public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
    public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
    public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
    public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
    public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
    public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
    public static long DefaultHysteresis = _ApplicationBase.DefaultHysteresis;
    public static long DefaultMinimumThreshold = 0;
    public static long DefaultMaximumThreshold = 5500;
    public static long DefaultCalibration1 = 53240;// WiFi Should be set to 3200
    public static long DefaultCalibration2 = 0;
    public static long DefaultCalibration3 = 0;//Calval3 PowerOptions Default 0 (0 none, 1 digital high SP2, 2 digital low SP2, 3 Amplifier SP9 Active Low)
    public static long DefaultCalibration4 = 0;//Calval4 Delay Time (Miniseconds) Default 0 (0-30,000)
    public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
    public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
    public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
    public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
    public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
    
    /// <summary>
    /// Firmware must be 2.0.1 or newer
    /// </summary>
    /// <param name="sensorID"></param>
    /// <param name="observed"></param>
    /// <param name="actuall"></param>
    /// <returns></returns>
    public static long Calibration1Value(long sensorID, long currentCalibration1, double observed, double actual, boolean isWiFi)
    {
        int Observed = (int)(observed * 1000);
        int Actual = (int)(actual * 1000);
        if (Observed > 0 && Actual > 0)
        {
            if (isWiFi)
            {
                int ADCCode = (int)(Observed * 1000000 / currentCalibration1);
                return Actual * 1000000 / ADCCode;
            }
            else
            {
                int ADCCode = (int)(Observed * 20470 / currentCalibration1);
                return Actual * 20470 / ADCCode;
            }
        }
        return currentCalibration1;
    }
    
    /**
     * <p>
     * sets the ReportInterval as minutes on the sensor object<br/>
     * the next time this sensor checks in it will update
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param heartbeat a double value this is represented in minutes so heartbeat equals 1 that is one minute
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorHeartBeat(Sensor sens,double heartBeat,Boolean sendSensorUpdate)
    {
  	  	sens.ReportInterval = (heartBeat);
  	  	
  	  	if(sens.ReportInterval < sens.getActiveStateInterval())
  			  sens.setActiveStateInterval(sens.ReportInterval);
  	  	
  	    sens.GeneralConfig1Dirty = sendSensorUpdate;
    }
    
    /**
     * <p>
     * sets all data for the GeneralConfig1 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param heartbeat a double value this is the ReportInterval and is represented in minutes so if heart beat equals 1 that is one minute
     * @param activeStateInterval a double value this is the aware state heart beat represented in minutes so aware state heart beat equals 1 that is one minute
     * @param  transmitIntervalLink How often the sensor attempts to relink with a gateway once it goes into link mode. Values below 12 are interpreted as hours. values above 12 are interpreted as minutes. the max value allowed in this field is 250. if a value of 0 is given the sensor will default to a relink interval of 2 hours
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorGeneralConfig1(Sensor sens,double heartBeat,double activeStateInterval,int transmitIntervalLink,Boolean sendSensorUpdate)
    {
  	  	sens.ReportInterval = (heartBeat);
  	  	sens.setActiveStateInterval((activeStateInterval));
  	  	
  	  	if(sens.ReportInterval < sens.getActiveStateInterval())
  			  sens.setActiveStateInterval(sens.ReportInterval);
  	  	
  	  	if(transmitIntervalLink < 0)
  	  		transmitIntervalLink = 0;
  	  	else if (transmitIntervalLink > 250)
  	  		transmitIntervalLink=250;
  	  				
  	    sens.TransmitIntervalLink = transmitIntervalLink;  	 	
  	    sens.GeneralConfig1Dirty = sendSensorUpdate;
    }
    
    /**
     * <p>
     * sets all data for the GeneralConfig2 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param recovery Once the recovery count is reached, the sensor will go to link mode and begin scanning channels and trying to find a gateway to join
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorGeneralConfig2(Sensor sens, int recovery,Boolean sendSensorUpdate)
    { 	
  	  if(recovery < 0)
  		  recovery = 0;
  		
  		if (recovery > 10)
  			recovery = 10;
  	  
  	  	sens.Recovery = recovery;
  	    sens.GeneralConfig2Dirty = sendSensorUpdate;
    }
   
    /**
     * <p>
     * sets all data for the ProfileConfig1 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param measurementsPerTransmission the amount of assessments the sensor where take inbetween heartbeats if during these assesments the value is above or below respective triggering thresholds it will send an aware state message. 
     * @param hysteresis A buffer to prevent the sensor from bouncing between standard operation and aware state when the asseessments are very close to a threashold. for example, if a maximum threashold is set to 90 degrees and a hysteresis of 1 degree, then once the sensor takes an assessment of 90.0 degrees and enters the aware state it will remain in the aware state until the temperature readings drop to 89.0 degrees
     * @param minimumThreshold the minimum temperature the sensor can report before it goes aware
     * @param maximumThreshold the maximum temperature the sensor can report before it goes aware
     * @param isFahrenheit data is always stored as Celsius so if you are inputting values as Fahrenheit you will want to set this boolean value to true
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorProfileConfig1(Sensor sens,int measurementsPerTransmission,double hysteresis,double minimumThreshold,double maximumThreshold,boolean isFahrenheit,Boolean sendSensorUpdate )
    { 
  		  sens.setMeasurementsPerTransmission(measurementsPerTransmission); 
  		  if(hysteresis <0)
    		hysteresis = 0;
    	  if(hysteresis > 2)
    		hysteresis = 2;
    	
  		  sens.setHysteresis((long)(hysteresis * 1000));
  		
  		  if(minimumThreshold <0)
  			minimumThreshold = 0;
    	  if(minimumThreshold > 5)
    		minimumThreshold = 5;
  		   
    	  sens.setMinimumThreshold((long)(minimumThreshold * 1000));
  		
  		  if(maximumThreshold <0)
  			maximumThreshold = 0;
    	  if(maximumThreshold > 5)
    		maximumThreshold = 5;
    	  if(maximumThreshold < minimumThreshold)
    		maximumThreshold = minimumThreshold;
  		  
    	  sens.setMaximumThreshold((long)(maximumThreshold * 1000));//
  	 
  		  sens.setProfileConfig1Dirty(sendSensorUpdate);
    }
    
    /**
     * <p>
     * sets all data for the ProfileConfig2 configuration available on the sensor
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param These options can be used to power/activate an external device for a set period of time prior to taking a measurement. The "None" setting disables any power options. "Digital High" forces the digital power pin to go high for the amount of time specified in "Delay in milliseconds" then goes low. "Digital Low" forces the digital power pin to go low for the time specified in "Delay in milliseconds" then goes high. "Amplifier SP9 Active Low" activates a transistor on the sensor for the time specified providing a path to ground/common which can be used to activate a higher current device. Max current output on the digital pin is 10 mA. Max current on the "Amplifier" pin is 40 mA. Any power drained from these ports will reduce battery life. power None = 0, Digital High = 1, Digital Low = 2, and Amplifier sp9 active low = 3
     * @param delay The amount of time in milliseconds the selected power option is active before the sensor takes a measurement. (default: 0, value range: 0 - 30000)
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorProfileConfig2(Sensor sens,int power,int delay,Boolean sendSensorUpdate )
    { 
    	if (delay < 0)//Less Than
        {
    		sens.setCalibration4(0);
        }
        else if (delay > 30000)
        {
        	sens.setCalibration4(30000);
        }
        else
        {
        	sens.setCalibration4(delay);
        }

        if (power == 0)//None
        {
        	sens.setCalibration3(0);
        }
        else if (power == 1)//Digital High
        {
        	sens.setCalibration3(1);
        }
        else if (power == 2)//Digital Low
        {
        	sens.setCalibration3(2);
        }
        else if (power == 3)//Amplifier sp9 active low
        {
        	sens.setCalibration3(3);
        }
  	      sens.setProfileConfig2Dirty(sendSensorUpdate);
    }
    
    /**
     * <p>
     * Calibrates the sensor to correct values set by the user
     * </p>
     * 
     * @param sens the Sensor object to be updated 
     * @param actualValue what the actual temperature of the room the sensor is located in is
     * @param lastSensorReading the last reading the sensor took.
     * @param sendSensorUpdate a Boolean flag that lets the sensor know if it should send up the new configuration
     */
    public static void UpdateSensorCalibration(Sensor sens,double actualValue, double lastSensorReading,Boolean sendSensorUpdate) throws Exception
    { 
       int observed = (int)lastSensorReading * 1000;
       int acutual = (int)actualValue * 1000;
       
       if(observed > 0 && acutual > 0)
       {
    	   int adccode = (int)(observed * 20470 / sens.getCalibration1());
    	   
    	   
    	   
    	   sens.setCalibration1(acutual * 20470 / adccode);
    	   sens.setProfileConfig1Dirty(false);
    	   sens.setProfileConfig2Dirty(sendSensorUpdate);
  	  
    	 
       }
    }
}

