package com.monnit.mine.EventHandling;

import java.util.Arrays;

public class Event<M>
{
  private Object[] listeners;
  
  public static class ListenerRegistration
  {
    private final Event<?> event;
    private final EventListener<?> listener;
    private boolean unregistered;
    
    private <M> ListenerRegistration(Event<M> event, EventListener<M> listener)
    {
      this.event = event;
      this.listener = listener;
    }
    
    public void remove()
    {
      if (!this.unregistered)
      {
        this.unregistered = true;
        this.event.remove(this.listener);
      }
    }
    
    public boolean isSubscribed()
    {
      return !this.unregistered;
    }
  }
  
  public ListenerRegistration add(EventListener<M> listener)
  {
    synchronized (this)
    {
      if (this.listeners == null)
      {
        this.listeners = new Object[] { listener };
      }
      else
      {
        this.listeners = Arrays.copyOf(this.listeners, this.listeners.length + 1);
        this.listeners[(this.listeners.length - 1)] = listener;
      }
    }
    return new ListenerRegistration(this, listener);
  }
  
  public void dispatch(Object source, M message)
  {
    Object[] local = this.listeners;
    if (local != null)
    {
      Object[] arrayOfObject1;
      int j = (arrayOfObject1 = local).length;
      for (int i = 0; i < j; i++)
      {
        Object listener = arrayOfObject1[i];
        ((EventListener)listener).onNotify(source, message);
      }
    }
  }
  
  private void remove(EventListener<?> listener)
  {
    synchronized (this)
    {
      if (this.listeners.length > 1)
      {
        Object[] newObservers = new Object[this.listeners.length - 1];
        int i = 0;
        for (int j = 0; i < this.listeners.length; i++) {
          if (!this.listeners[i].equals(listener)) {
            newObservers[(j++)] = this.listeners[i];
          }
        }
        this.listeners = newObservers;
      }
      else
      {
        this.listeners = null;
      }
    }
  }
}