package com.monnit.mine.MonnitAPN;

import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.MonnitAPN.MNTEventArgs.ActionCtrlEventArgs;
import com.monnit.mine.MonnitAPN.MNTEventArgs.BootloaderEventArgs;
import com.monnit.mine.MonnitAPN.MNTEventArgs.DataMessageEventArgs;
import com.monnit.mine.MonnitAPN.MNTEventArgs.NetworkStatusEventArgs;
import com.monnit.mine.MonnitAPN.MNTEventArgs.ParentMessageEventArgs;
import com.monnit.mine.MonnitAPN.MNTEventArgs.ReadConfigEventArgs;
import com.monnit.mine.MonnitAPN.MNTEventArgs.SensorStatusEventArgs;
import com.monnit.mine.MonnitAPN.MNTEventArgs.WriteConfigEventArgs;
import com.monnit.mine.MonnitMineAPI.MineServer;

public class MSAPI {
	// //#region Constants
	// API Command Frame Deliminators
	public final static byte START_FRAME_DELIMINATOR = (byte) 0xC5;
	public final static byte OPTIONS_NONE = (byte) 0x00;
	public final static byte OPTIONS_ENCRYPTION = (byte) 0x01;
	public final static byte OPTIONS_URGENT = (byte) 0x02;
	public final static byte OPTIONS_SENSORWAITING = (byte) 0x04;
	public final static byte OPTIONS_PROMISCUOUS = (byte) 0x08;

	// API Commands
	public final static byte CMD_FORM_NETWORK_REQUEST = (byte) 0x20;
	public final static byte CMD_UPDATE_NETWORK_STATE = (byte) 0x21;
	public final static byte CMD_REGISTER_SENSOR_REQUEST = (byte) 0x22;
	public final static byte CMD_NETWORK_STATUS_MESSAGE = (byte) 0x23;
	public final static byte CMD_MESSAGE_QUEUED_NOTIFY = (byte) 0x24;
	public final static byte CMD_SENSOR_STATUS_INDICATOR = (byte) 0x25;
	public final static byte CMD_REGISTERED_SENSOR_REQUEST = (byte) 0x26;
	public final static byte CMD_REGISTERED_SENSOR_RESPONSE = (byte) 0x27;

	public final static byte CMD_READ_DATASECTOR_REQUEST = (byte) 0x70;
	public final static byte CMD_READ_DATASECTOR_RESPONSE = (byte) 0x71;
	public final static byte CMD_WRITE_DATASECTOR_REQUEST = (byte) 0x72;
	public final static byte CMD_WRITE_DATASECTOR_RESPONSE = (byte) 0x73;
	public final static byte CMD_APPCMD_REQUEST = (byte) 0x74;
	public final static byte CMD_APPCMD_RESPONSE = (byte) 0x75;
	public final static byte CMD_PARENT_MSG = (byte) 0x52;
	public final static byte CMD_DATA_MESSAGE = (byte) 0x55;
	public final static byte CMD_DATA_MESSAGE_DL = (byte) 0x56;
	public final static byte CMD_BOOTLOADER_REQUEST = (byte) 0xD4;
	public final static byte CMD_BOOTLOADER_RESPONSE = (byte) 0xD5;

	public final static byte CMD_SPURIOUS_MESSAGE = (byte) 0x91;
	// //#endregion

	public byte[] frame;

	public MSAPI(byte[] formatted_frame) throws Exception {
		// GUIListenerFunctions.print("MSAPI Upper Constructor.");
		if (formatted_frame == null)
			throw new Exception();
		if (formatted_frame.length < 5)
			throw new Exception();
		frame = new byte[formatted_frame.length];
		System.arraycopy(formatted_frame, 0, frame, 0, frame.length);
		getCommandIndex();
		getCommand();
	}

	public MSAPI(byte[] cmdNdata, byte options) throws Exception {
		Extensions.debug("MSAPI Lower Constructor.", 12);
		if (cmdNdata == null)
			throw new Exception();
		if (cmdNdata.length == 0)
			throw new Exception();
		if ((options & OPTIONS_ENCRYPTION) == 0) {
			Extensions.debug("Entered if in MSAPI Lower constructor.", 12);
			frame = new byte[cmdNdata.length + 4];
			frame[0] = START_FRAME_DELIMINATOR;
			frame[1] = (byte) (cmdNdata.length + 1);
			frame[2] = options;
			System.arraycopy(cmdNdata, 0, frame, 3, cmdNdata.length);
			FrameCRC = CalcFrameCRC();
			frame[frame.length - 1] = FrameCRC;
			Extensions.debug("FrameCRC: " + FrameCRC, 14);
			getCommandIndex();
			getCommand();
		}
		// else if ((options & OPTIONS_ENCRYPTION) == 1)
		// {
		// frame[1] = (byte)(cmdNdata.length + 9);
		// }
		else
			throw new Exception();

	}

	// Propertys & Methods
	public Calendar _MessageDate = MineServer.CurrentUTC();

	public Calendar getMessageDate() {
		return _MessageDate;
	}

	public void setMessageDate(Calendar d) {
		_MessageDate = d;
	}

	private long _APNID = Long.MIN_VALUE;

	public long getAPNID() {
		return _APNID;
	}

	public void setAPNID(long value) {
		_APNID = value;
	}

	private int _ProtocolVersion = Integer.MIN_VALUE;

	public int getProtocolVersion() {
		if (_ProtocolVersion == Integer.MIN_VALUE)
			return GlobalAPN.getProtocolVersion();
		return _ProtocolVersion;
	}

	public int setProtocolVersion(int value) {
		return _ProtocolVersion = value;
	}

	private byte Length;

	public byte getLength() {
		return frame[1];
	}

	public void setLength(byte value) {
		frame[1] = value;
	}

	private byte DataLength;

	public byte getDataLength() {
		// remove Options / Command from calc. If Encrypted, also remove 8
		// helper bytes
		int val = getLength();
		val -= ((getOptions() & OPTIONS_ENCRYPTION) == 0) ? 2 : 10;
		DataLength = (byte) val;
		return (byte) val;
	}

	private byte Options;

	public byte getOptions() {
		return frame[2];
	}

	public Boolean IsSensorWaiting;

	public Boolean getIsSensorWaiting() {
		return ((Options & OPTIONS_SENSORWAITING) != 0);
	}

	private int CommandIndex;

	public int getCommandIndex() {
		CommandIndex = ((Options & OPTIONS_ENCRYPTION) == 0) ? 3 : 10;
		return ((Options & OPTIONS_ENCRYPTION) == 0) ? 3 : 10;
	}

	public byte Command;

	public byte getCommand() {
		if (Command != frame[CommandIndex])
			Command = frame[CommandIndex];
		return frame[CommandIndex];
	}

	public byte FrameCRC;

	public byte getFrameCRC() {
		return frame[Length + 2];
	}

	public void setFrameCRC(byte value) {
		frame[Length + 2] = value;
	}

	public byte CalcFrameCRC() {
		byte crc = 0;
		for (int i = 2; i < frame[1] + 2; i++)
			crc = CRC8.AddStep2CRC(crc, frame[i]);
		return crc;
	}

	public Boolean isAPNCommand() {
		switch (Command) {
		case CMD_UPDATE_NETWORK_STATE:
			return true;
		case CMD_NETWORK_STATUS_MESSAGE:
			return true;
		case CMD_FORM_NETWORK_REQUEST:
			return true;
		case CMD_REGISTER_SENSOR_REQUEST:
			return true;
		case CMD_MESSAGE_QUEUED_NOTIFY:
			return true;
		case CMD_REGISTERED_SENSOR_REQUEST:
			return true;
		default:
			return false;
		}
	}

	public Boolean isAck2Command(MSAPI cmdmsg) {
		switch (Command) {
		case CMD_READ_DATASECTOR_RESPONSE:
			if (cmdmsg.Command == CMD_READ_DATASECTOR_REQUEST)
				return true;
			else
				return false;
		case CMD_WRITE_DATASECTOR_RESPONSE:
			if (cmdmsg.Command == CMD_WRITE_DATASECTOR_REQUEST)
				return true;
			else
				return false;
		case CMD_APPCMD_RESPONSE:
			if (cmdmsg.Command == CMD_APPCMD_REQUEST)
				return true;
			else
				return false;
		default:
			return false;
		}
	}

	public Boolean isQueuedNotificationResponse() {
		if ((Command == CMD_SENSOR_STATUS_INDICATOR)
				&& (frame[CommandIndex + 7] == 2))
			return true;
		return false;
	}

	public Boolean isUndelivered() {
		if ((Command == CMD_SENSOR_STATUS_INDICATOR)
				&& (frame[CommandIndex + 7] == 4))
			return true;
		return false;
	}

	public int ExtractCMDAddress() throws Exception {
		if (isAPNCommand())
			return 0;
		else
			return BitConverter.toInt32(frame, CommandIndex + 1);
	}

	private String StartNaming(Boolean isTX) {
		String s = new Date().toString();
		if (isTX)
			s += "TX: ";
		else
			s += "RX: ";
		return s;
	}

	private String NetworkStateDecode(int State) {
		String s = "";
		if (State == 0)
			s += "\"RESET/IDLE\"";
		else if (State == 1)
			s += "\"ACTIVE/RESUME\"";
		else if (State == 2)
			s += "\"FACTORY/FULLFILLMENT\"";
		else
			s += "\"!!!UNKNOWN STATE!!!\"";
		return s;
	}

	private String HexPrint(int idx, int end) {
		return Integer.toHexString(idx);
	}

	public String TX_SHandler() throws Exception {
		String s = StartNaming(true);
		switch (Command) {
		case CMD_UPDATE_NETWORK_STATE:
			// [1] mode
			s += "Set Network State to "
					+ NetworkStateDecode(frame[CommandIndex + 1]);
			break;
		case CMD_FORM_NETWORK_REQUEST:
			// [1]Channel Mask - NETID[5]
			s += "Form Network Request: ChanMask: 0x"
					+ Integer.toHexString(BitConverter.toInt32(frame,
							CommandIndex + 1));
			s += " Requested Network ID: ";
			if (frame[CommandIndex + 5] == 0)
				s += "Random";
			else
				s += frame[CommandIndex + 5] + "";
			break;
		case CMD_REGISTER_SENSOR_REQUEST:
			// [1]Sensor Serial being ADDED
			s += "Add Device to Network: "
					+ BitConverter.toInt32(frame, CommandIndex + 1) + "";
			break;
		case CMD_MESSAGE_QUEUED_NOTIFY:
			// [1]TargetSID, [5] Queued/NotQueued
			if (frame[CommandIndex + 5] == 1)
				s += "Sending Waiting Message for "
						+ BitConverter.toInt32(frame, CommandIndex + 1) + "";
			else
				s += "Sending No Waiting Message for "
						+ BitConverter.toInt32(frame, CommandIndex + 1) + "";
			break;
		case CMD_REGISTERED_SENSOR_REQUEST:
			s += "Sending RSL Request to Index " + frame[CommandIndex + 1] + "";
			break;
		case CMD_READ_DATASECTOR_REQUEST:
			// [1]TargetSID, [5] Sector
			s += "Request Read (Device: "
					+ BitConverter.toInt32(frame, CommandIndex + 1);
			s += " Sector: " + frame[CommandIndex + 5] + ")";
			break;
		case CMD_WRITE_DATASECTOR_REQUEST:
			// [1]TargetSID, [5] Sector, [6]DATA32
			s += "Request Write (Device: "
					+ BitConverter.toInt32(frame, CommandIndex + 1);
			s += " Sector: " + frame[CommandIndex + 5] + ")";
			// Could show data32 here
			break;
		case CMD_APPCMD_REQUEST:
			// [1]TargetSID, [5] Type, [6] Data0-35
			s += "Application Command Request (Device: "
					+ BitConverter.toInt32(frame, CommandIndex + 1);
			s += " Type: " + frame[CommandIndex + 5] + ")";
			if (DataLength > 6)
				s += " Data: "
						+ HexPrint(CommandIndex + 6, CommandIndex + 6
								+ (DataLength - 6)); // errors here
			break;
		case CMD_BOOTLOADER_REQUEST:
			// [1]TargetSID, [5] Command, [6] Sector
			s += "Bootloader Command Request (Device: "
					+ BitConverter.toInt32(frame, CommandIndex + 1);
			s += " Command: " + frame[CommandIndex + 5];
			s += " Sector: " + BitConverter.toInt32(frame, CommandIndex + 6)
					+ ")";
			break;
		default:
			s = "";
			break;
		}
		return s;
	}

	public String RX_SHandler(MineServer _Server) throws Exception {
		String s = StartNaming(false);
		getDataLength();
		// System.out.println(Command);
		switch (Command) {
		case CMD_NETWORK_STATUS_MESSAGE:
			// [1]SID - [5]NetCNT - [7]Channel - [8]NETID - [9]MODE - [10]STATUS
			// #region CMD_NETWORK_STATUS_MESSAGE
			if ((frame[CommandIndex + 10] & 0x0F) == 1) {
				s = "Network Status: No network found, forming new network";
				// OnNoNetworkFound();
				break;
			} else {
				s += "Network Status: ";
				if ((frame[CommandIndex + 10] & 0x0F) == 0) {
					s += "APN: "
							+ BitConverter.ToUInt32(frame, CommandIndex + 1);
					s += ", NetCNT: "
							+ BitConverter.ToUInt16(frame, CommandIndex + 5);
					s += ", Channel: " + frame[CommandIndex + 7];
					s += ", NetID: " + frame[CommandIndex + 8];
					s += ", MODE: "
							+ NetworkStateDecode(frame[CommandIndex + 9]);
				} else {
					s += "ERROR [" + (frame[CommandIndex + 10] & 0x0F) + "]";
				}
			}

			try {
				_Server.OnNetworkStatus(new NetworkStatusEventArgs(
						(BitConverter.ToUInt32(frame, CommandIndex + 1)),
						(BitConverter.ToUInt16(frame, CommandIndex + 5)),
						BitConverter.ToInt32(frame[CommandIndex + 7]),
						BitConverter.ToInt32(frame[CommandIndex + 8]),
						BitConverter.ToInt32(frame[CommandIndex + 9]),
						BitConverter.ToInt32(frame[CommandIndex + 10])));
			} catch (Exception e) {
				s += String
						.format("\r\nError Reporting Network Status - SID:%s Status: %s",
								BitConverter.ToUInt32(frame, CommandIndex + 1),
								frame[CommandIndex + 10]);
				MonnitAPN.WriteExceptionToLog(this, e);
			}
			break;
		// #endregion
		case CMD_SENSOR_STATUS_INDICATOR:
			// [1]SID - [5]Status - [6] NodeDesc
			// #region CMD_SENSOR_STATUS_INTICATOR
			s += "Status Indication: ";
			s += "Device: " + BitConverter.ToUInt32(frame, CommandIndex + 1);
			if (frame[CommandIndex + 7] == 0)
				s += " has joined network";
			else if (frame[CommandIndex + 7] == 1)
				s += " has failed to join network";
			else if (frame[CommandIndex + 7] == 2)
				s += " has handled queued message notification";
			else if (frame[CommandIndex + 7] == 3)
				s += " address not found in network";
			else if (frame[CommandIndex + 7] == 4)
				s += " message not delivered";

			try {
				_Server.OnSensorStatus(new SensorStatusEventArgs(
						BitConverter.ToInt64(BitConverter.ToUInt32(frame,
								CommandIndex + 1)), BitConverter.ToUInt16(
								frame, CommandIndex + 5), BitConverter
								.ToInt32(frame[CommandIndex + 7]), s));
			} catch (Exception e) {
				s += String.format(
						"\r\nError Reporting Sensor Status - SID:%s Status:%s",
						BitConverter.ToUInt32(frame, CommandIndex + 1),
						frame[CommandIndex + 7]);
				MonnitAPN.WriteExceptionToLog(this, e);
			}
			break;
		// #endregion
		case CMD_REGISTERED_SENSOR_RESPONSE:
			// [1]Index - [4]ID0 - [4]ID1 - [4]ID2 - [4]ID3
			// #region CMD_REGISTERED_SENSOR_RESPONSE
			s += "RSL Response Index[" + frame[CommandIndex + 1] + "]: ";
			s += BitConverter.ToUInt32(frame, CommandIndex + 2) + ", ";
			s += BitConverter.ToUInt32(frame, CommandIndex + 6) + ", ";
			s += BitConverter.ToUInt32(frame, CommandIndex + 10) + ", ";
			s += BitConverter.ToUInt32(frame, CommandIndex + 14) + ", ";
			break;
		// #endregion
		case CMD_READ_DATASECTOR_RESPONSE:
			// [1]SID - [5]Sector - [6]Status - [7]Data32
			// #region CMD_READ_DATASECTOR_RESPONSE
			s += "Read: Device: "
					+ BitConverter.toInt32(frame, CommandIndex + 1);
			s += ", Sector: " + frame[CommandIndex + 5];
			s += ", Status: ";
			if (frame[CommandIndex + 6] != 0)
				s += "Failed!";
			else {
				if (getProtocolVersion() < 2) {
					s += "Success, Data: ";
					if ((frame[CommandIndex + 5] == 0)
							|| (frame[CommandIndex + 5] == 1)
							|| (frame[CommandIndex + 5] == 2)
							|| (frame[CommandIndex + 5] == 3))
						s += Extensions.byteToAsciiString(frame,
								CommandIndex + 7, CommandIndex + 7 + 32);
					else
						s += HexPrint(CommandIndex + 7, CommandIndex + 7 + 32);

					// If sector 10 read, then collect CID
					if (frame[CommandIndex + 5] == 10)
						GlobalAPN.setCID(BitConverter.ToUInt32(frame,
								CommandIndex + 7 + 4));
				} else {
					s += "Success, Data: ";
					if ((frame[CommandIndex + 5] == 0)
							|| (frame[CommandIndex + 5] == 1)
							|| (frame[CommandIndex + 5] == 2)
							|| (frame[CommandIndex + 5] == 3))
						s += Extensions.byteToAsciiString(frame,
								CommandIndex + 7, CommandIndex + 7 + 16);
					else
						s += HexPrint(CommandIndex + 7, CommandIndex + 7 + 16);

					// If sector 20 read, then collect CID
					if (frame[CommandIndex + 5] == 20)
						GlobalAPN.setCID(BitConverter.ToUInt32(frame,
								CommandIndex + 7 + 4));
				}
			}

			try {
				if (getProtocolVersion() < 2) {
					byte[] data = new byte[32];
					if (frame[CommandIndex + 6] == 0) {
						System.arraycopy(frame, CommandIndex + 7, data, 0, 32);
						_Server.OnReadDataAck(new ReadConfigEventArgs(
								BitConverter.ToInt64(BitConverter.ToUInt32(
										frame, CommandIndex + 1)), BitConverter
										.ToInt32(frame[CommandIndex + 5]),
								data, s));
					}
				} else {
					byte[] data = new byte[16];
					if (frame[CommandIndex + 6] == 0) {
						System.arraycopy(frame, CommandIndex + 7, data, 0, 16);
						_Server.OnReadDataAck(new ReadConfigEventArgs(
								BitConverter.ToInt64(BitConverter.ToUInt32(
										frame, CommandIndex + 1)), BitConverter
										.ToInt32(frame[CommandIndex + 5]),
								data, s));
					}
				}
			} catch (Exception e) {
				s += String.format(
						"\r\nError Reporting Sector Read - SID:%s Sector:%s",
						BitConverter.ToUInt32(frame, CommandIndex + 1),
						frame[CommandIndex + 5]);
				MonnitAPN.WriteExceptionToLog(this, e);
			}
			break;
		// #endregion
		case CMD_WRITE_DATASECTOR_RESPONSE:
			// [1]SID - [5]Sector - [6]Status
			// #region CMD_WRITE_DATASECTOR_RESPONSE
			Extensions.debug("Reached write datasector response.", 20);
			s += "Write: Device: "
					+ BitConverter.ToUInt32(frame, CommandIndex + 1);
			s += " Sector: " + frame[CommandIndex + 5];
			if (frame[CommandIndex + 6] == 0)
				s += " Status: Success";
			else
				s += " Status: Failed (" + frame[CommandIndex + 6] + ")";

			try {
				if (frame[CommandIndex + 6] == 0) {
					_Server.OnWriteDataAck(new WriteConfigEventArgs(
							BitConverter.ToInt64(BitConverter.ToUInt32(frame,
									CommandIndex + 1)), BitConverter
									.ToInt32(frame[CommandIndex + 5]), s));
					_Server.HandleWriteDataAck(
							null,
							new WriteConfigEventArgs(BitConverter
									.ToInt64(BitConverter.ToUInt32(frame,
											CommandIndex + 1)), BitConverter
									.ToInt32(frame[CommandIndex + 5]), s));
				}
			} catch (Exception e) {
				s += String
						.format("\r\nError Reporting Sector Written - SID:%s Sector:%s",
								BitConverter.ToUInt32(frame, CommandIndex + 1),
								frame[CommandIndex + 5]);
				MonnitAPN.WriteExceptionToLog(this, e);
			}
			Extensions.debug("Finished write datasector response.", 20);
			break;
		// #endregion
		case CMD_APPCMD_RESPONSE:
			// [1]SID - [5]AppCmdID - [6]Status - [7 and on] ResponseData
			// #region CMD_APPCMD_RESPONSE
			s += " AppCommand: Device: "
					+ BitConverter.ToUInt32(frame, CommandIndex + 1);
			s += " Command: " + frame[CommandIndex + 5];
			if (frame[CommandIndex + 6] == 0)
				s += " Status: Success ";
			else
				s += " Status: Failed (" + frame[CommandIndex + 6] + ") ";
			if (DataLength > 6)
				s += "Data: "
						+ HexPrint(CommandIndex + 7, CommandIndex + 7
								+ (DataLength - 6)); // errors here

			byte[] payload = new byte[(DataLength - 6)];
			System.arraycopy(frame, CommandIndex + 7, payload, 0,
					payload.length);

			try {
				_Server.OnACTLAck(new ActionCtrlEventArgs(
						BitConverter.ToInt64(BitConverter.ToUInt32(frame,
								CommandIndex + 1)), BitConverter
								.ToInt32(frame[CommandIndex + 5]),
						(frame[CommandIndex + 6] == 0) ? true : false, payload,
						s));
				// _Server.HandleActionControlAck(null, new
				// ActionCtrlEventArgs(BitConverter.ToInt64(BitConverter.ToUInt32(frame,
				// CommandIndex + 1)), BitConverter.ToInt32(frame[CommandIndex +
				// 5]), (frame[CommandIndex + 6] == 0) ? true : false,
				// payload));
			} catch (Exception e) {
				s += String.format(
						"\r\nError Reporting Action/Control Command - SID:%s",
						BitConverter.ToUInt32(frame, CommandIndex + 1));
				MonnitAPN.WriteExceptionToLog(this, e);
			}

			break;
		// #endregion
		case CMD_BOOTLOADER_RESPONSE:
			// [1]SID - [5]BLCmdID - [6]Sector - [8]Status
			// #region CMD_BLCMD_RESPONSE
			s += "BLCommand: Device: "
					+ BitConverter.ToUInt32(frame, CommandIndex + 1);
			s += " Command: " + frame[CommandIndex + 5];
			s += " Sector: " + BitConverter.ToUInt16(frame, CommandIndex + 6);
			if (frame[CommandIndex + 8] == 0)
				s += " Status: Success ";
			else
				s += " Status: Failed (" + frame[CommandIndex + 8] + ") ";

			try {
				_Server.OnBootloaderAck(new BootloaderEventArgs(
						BitConverter.ToInt64(BitConverter.ToUInt32(frame,
								CommandIndex + 1)), frame[CommandIndex + 5],
						BitConverter.ToUInt16(frame, CommandIndex + 6),
						frame[CommandIndex + 8], s));
			} catch (Exception e) {
				s += String.format(
						"\r\nError Reporting Bootloader Command - SID:%s",
						BitConverter.ToUInt32(frame, CommandIndex + 1));
				MonnitAPN.WriteExceptionToLog(this, e);
			}

			break;
		// #endregion
		case CMD_PARENT_MSG:
			// [1]SID - [5]PARENT
			// #region CMD_PARENT_MSG
			s += "Parent-Link: "
					+ BitConverter.ToUInt32(frame, CommandIndex + 1) + " --> "
					+ BitConverter.ToUInt32(frame, CommandIndex + 5);
			try {
				_Server.OnParentMsg(new ParentMessageEventArgs(
						BitConverter.ToInt64(BitConverter.ToUInt32(frame,
								CommandIndex + 1)), BitConverter
								.ToInt64(BitConverter.ToUInt32(frame,
										CommandIndex + 5)), s));
			} catch (Exception e) {
				s += String.format(
						"\r\nError Reporting Parent Message - SID:%s PID:%s",
						BitConverter.ToUInt32(frame, CommandIndex + 1),
						BitConverter.ToUInt32(frame, CommandIndex + 5));
				MonnitAPN.WriteExceptionToLog(this, e);
			}
			break;
		// #endregion
		case CMD_DATA_MESSAGE:
		case CMD_DATA_MESSAGE_DL:
		// [1]SID - [5]APN-RSSI - [6]Sensor-RSSI - [7]PowerCounter -
		// [8]SensorType - [10]State - [11]-Data
		// [1]SID - [5]Time - [9]APN-RSSI - [10]Sensor-RSSI - [11]PowerCounter -
		// [12]SensorType - [14]State - [15]-Data
		// #region CMD_DATA_MESSAGE/CMD_DATA_MESSAGE_DL
		{
			s += GetMessageString(s);
			try {
				_Server.OnProcessData(new DataMessageEventArgs(DM_SensorID,
						DM_Voltage, DM_RssiInbound, DM_RssiOutbound, DM_State,
						getAPNID(), DM_RawData, DM_MessageDate, s));
			} catch (Exception e) {
			}
			break;
		}
		// #endregion
		case CMD_SPURIOUS_MESSAGE:
			s += "Spurious Message Detected!: "
					+ ((getIsSensorWaiting()) ? "Valid" : "InValid");
			break;
		default:
			s = "";
			break;
		}
		return s;
	}

	public long DM_SensorID;

	public long getDM_SensorID() throws Exception {
		if (Command == MSAPI.CMD_DATA_MESSAGE
				|| Command == MSAPI.CMD_DATA_MESSAGE_DL)
			return BitConverter.toInt32(frame, CommandIndex + 1);
		return 0;

	}

	public int DM_RssiInbound;

	public int getDM_RssiInbound() throws Exception {
		int rssiInbound = 0;
		if (Command == MSAPI.CMD_DATA_MESSAGE)
			rssiInbound = frame[CommandIndex + 5]; // rssi
													// of
													// APN
		else if (Command == MSAPI.CMD_DATA_MESSAGE_DL)
			rssiInbound = frame[CommandIndex + 9]; // rssi
													// of
													// APN

		if (rssiInbound > 128)
			rssiInbound -= 256;
		return rssiInbound;
	}

	public int DM_RssiOutbound;

	public int getDM_RssiOutbound() throws Exception {
		int rssiOutbound = 0;
		if (Command == MSAPI.CMD_DATA_MESSAGE)
			rssiOutbound = frame[CommandIndex + 6]; // rssi
													// of
													// APN
													// talking
													// to
													// EDN
													// (Last
													// message
													// sent)
		else if (Command == MSAPI.CMD_DATA_MESSAGE_DL)
			rssiOutbound = frame[CommandIndex + 10]; // rssi
														// of
														// APN
														// talking
														// to
														// EDN
														// (Last
														// message
														// sent)
		if (rssiOutbound > 128)
			rssiOutbound -= 256;
		return rssiOutbound;
	}

	public double DM_Voltage;

	public double getDM_Voltage() {
		// Extensions.debug(Extensions.hexStr(frame), 23);
		int RawReading = 0;
		if (Command == MSAPI.CMD_DATA_MESSAGE)
			RawReading = frame[CommandIndex + 7] & 0xFF;
		else if (Command == MSAPI.CMD_DATA_MESSAGE_DL) {
			RawReading = frame[CommandIndex + 11] & 0xFF;
			Extensions.debug("Going through the ranger. ", 23);
		}

		if (RawReading == 0)
			return 0;

		// Extensions.debug(""+RawReading, 23);
		return (RawReading + 150) / 100.0; // Voltage + 1.5 volts
											// (x100)
	}

	public int DM_NodeDesc;

	public int getDM_NodeDesc() throws Exception {
		Extensions.debug(Extensions.hexStr(frame), 22);
		Extensions.debug("CommandIndex: " + CommandIndex, 22);
		if (CommandIndex == 3) {
			Extensions.debug("\tIf index were 10: ", 21);
			try {
				Extensions.debug("\t\tPlus 8 spots: " + frame[10 + 8], 21);
				Extensions.debug("\t\tPlus 12 spots: " + frame[10 + 12], 21);
			} catch (Exception e) {
				Extensions.debug("\t\tIt would be out of range.", 21);
			}
		} else if (CommandIndex == 10) {
			Extensions.debug("\tIf index were 3: ", 21);
			Extensions.debug("\t\tPlus 8 spots: " + frame[3 + 8], 21);
			Extensions.debug("\t\tPlus 12 spots: " + frame[3 + 12], 21);
		} else {
			Extensions.debug("Command index bad " + CommandIndex, 21);
		}
		Extensions.debug("\tPlus 8 spots: " + frame[CommandIndex + 8], 21);
		Extensions.debug("\tPlus 12 spots: " + frame[CommandIndex + 12], 21);
		if (Command == MSAPI.CMD_DATA_MESSAGE)
			// return (int) frame[CommandIndex+8];
			return (BitConverter.twoBytesToInt(frame, CommandIndex + 8));
		else if (Command == MSAPI.CMD_DATA_MESSAGE_DL)
			// return (int) frame[CommandIndex+12];
			return (BitConverter.twoBytesToInt(frame, CommandIndex + 12));
		return 0;
	}

	public int DM_State;

	public int getDM_State() throws Exception {
		Extensions.debug("Getting state from " + Extensions.hexStr(frame), 23);
		Extensions.debug("\tCommand index: " + CommandIndex, 23);
		int dmstate = 0;
		if (Command == MSAPI.CMD_DATA_MESSAGE)
			dmstate = frame[CommandIndex + 10];
		// return BitConverter.toInt32(frame, CommandIndex + 10);
		else if (Command == MSAPI.CMD_DATA_MESSAGE_DL)
			dmstate = frame[CommandIndex + 14];
		// return BitConverter.toInt32(frame, CommandIndex + 14);
		Extensions.debug("\tState: " + dmstate, 23);
		return dmstate;
	}

	public Calendar DM_MessageDate;

	public Calendar getDM_MessageDate() throws Exception {
		long time = 0;
		if (Command == MSAPI.CMD_DATA_MESSAGE)
			return getMessageDate();
		else if (Command == MSAPI.CMD_DATA_MESSAGE_DL) 
		{ 
			time =  BitConverter.toLongFromUint32(frame, CommandIndex + 5); //BitConverter.toLongFromFourBytes(frame, CommandIndex + 5);
			
		   Calendar c = MineServer.GetCalendarFromSeconds(time);
		   return c;
		}
		
		return MineServer.CurrentUTC();
	}

	public byte[] DM_RawData;

	public byte[] getDM_RawData() throws InterruptedException {
		byte[] buffer = null;
		if (Command == MSAPI.CMD_DATA_MESSAGE) {
			Extensions.debug("" + getDataLength(), 2);
			buffer = new byte[getDataLength() - 10];
			System.arraycopy(frame, CommandIndex + 11, buffer, 0, buffer.length);
		} else if (Command == MSAPI.CMD_DATA_MESSAGE_DL) {
			buffer = new byte[getDataLength() - 14];
			System.arraycopy(frame, CommandIndex + 15, buffer, 0, buffer.length);
		}
		return buffer;
	}

	public String GetMessageString(String s) throws Exception {
		s += "Sensor Data: Device: " + DM_SensorID + "";
		s += ", SensorType: " + DM_NodeDesc;
		s += ", RSSI: " + DM_RssiInbound + ""; // String.format(" RSSI: %s / %s",
												// rssiInbound, rssiOutbound);
		s += "dBm, Volts: " + DM_Voltage + "";
		s += "V, STS: 0x" + Integer.toHexString(DM_State);
		// KELLY ADDED!!!!!! FOR ECHO ONE APPLICATION!!!!!! FOR KWJ TESTING
		// 4/2014
		if (DM_NodeDesc == 34)
			s += String.format(", Data: {0://#0.//# C}, %d ppm, %d ppm-TWA",
					(BitConverter.toInt16(DM_RawData, 0)) / 10.0,
					BitConverter.toInt16(DM_RawData, 2),
					BitConverter.toInt16(DM_RawData, 4));
		else
			s += ", Data: "
					+ BitConverter.toString(DM_RawData).replace("-", "");
		if (Command == MSAPI.CMD_DATA_MESSAGE_DL)
			s += ", DLTIME: " + _MessageDate.toString();
		return s;
	}
}
