package com.monnit.mine.MonnitMineAPI;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.MonnitMineAPI.enums.eGatewayBattery;
import com.monnit.mine.MonnitMineAPI.enums.eGatewayType;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import javax.crypto.SecretKey;

public class Gateway
{
  SecretKey PreSharedKey;
  SecretKey EncryptionKey;
  Calendar DecryptFailureDate;
  byte[] EncryptionIV;
  public String GatewaySubnet;
  boolean ObserveAware;
  private double _PollInterval = 0.0D;
  int _AutoConfigTime;
  double GPSReportInterval = 720.0D;
  public String Username;
  public String Password;
  public String CellAPNName;
  public int AutoConfigCommandTime = 5;
  private List<Long> _SensorIDs;
  public int GatewayID;
  public eGatewayType GatewayType;
  public String RadioFirmwareVersion;
  public String GatewayFirmwareVersion;
  public String ExternalIdentifier;
  public double ReportInterval;
  public int Port;
  public String ServerHostAddress;
  public String GatewayIP;
  public String DefaultRouterIP;
  public Boolean IsDirty;
  public String GatewayDNS;
  public String SecondaryDNS;
  double NetworkListInterval;
  public double PollInterval;
  public boolean HasUrgentTraffic;
  public boolean SendResetNetworkRequest;
  int LastSequence;
  public Boolean PendingActionControlCommand;
  List<byte[]> ActionControlCommands;
  public List<Integer> AssignedSensors;
  
  public Gateway(int gatewayID, eGatewayType gatewayType, String gatewayFirmwareVersion, String radioFirmwareVersion, String hostAddress, int port)
  {
    this.GatewayID = gatewayID;
    this.GatewayType = gatewayType;
    this.GatewayFirmwareVersion = gatewayFirmwareVersion;
    this.RadioFirmwareVersion = radioFirmwareVersion;
    
    this.ServerHostAddress = hostAddress;
    this.Port = port;
    
    this.ReportInterval = 5.0D;
    this.ObserveAware = true;
    this.HasUrgentTraffic = false;
    this.GatewayIP = "0.0.0.0";
    this.GatewaySubnet = "0.0.0.0";
    this.DefaultRouterIP = "0.0.0.0";
    this.GatewayDNS = "0.0.0.0";
    this.SecondaryDNS = "0.0.0.0";
    
    this.LastSequence = 0;
    this.SendResetNetworkRequest = false;
    this.ExternalIdentifier = "";
    this.NetworkListInterval = 720.0D;
    
    getAssignedSensors();
  }
  
  public Gateway(int gatewayID, eGatewayType gatewayType, String gatewayFirmwareVersion, String radioFirmwareVersion, String hostAddress, int port, SecretKey key)
  {
    this.PreSharedKey = key;
    
    this.GatewayID = gatewayID;
    this.GatewayType = gatewayType;
    this.GatewayFirmwareVersion = gatewayFirmwareVersion;
    this.RadioFirmwareVersion = radioFirmwareVersion;
    
    this.ServerHostAddress = hostAddress;
    this.Port = port;
    
    this.ReportInterval = 5.0D;
    this.ObserveAware = true;
    this.HasUrgentTraffic = false;
    this.GatewayIP = "0.0.0.0";
    this.GatewaySubnet = "0.0.0.0";
    this.DefaultRouterIP = "0.0.0.0";
    this.GatewayDNS = "0.0.0.0";
    this.SecondaryDNS = "0.0.0.0";
    
    this.LastSequence = 0;
    this.SendResetNetworkRequest = false;
    this.ExternalIdentifier = "";
    this.NetworkListInterval = 720.0D;
    
    getAssignedSensors();
  }
  
  public Gateway(int gatewayID, eGatewayType gatewayType, String gatewayFirmwareVersion, String radioFirmwareVersion, String hostAddress, int port, SecretKey presharedkey, double reportInterval, Boolean observeAware, double pollInterval, Boolean hasUrgentTraffic, String gatewayIP, String gatewaySubnet, String defaultRouterIP, String gatewayDNS, String secondaryDNS)
  {
    this(gatewayID, gatewayType, gatewayFirmwareVersion, radioFirmwareVersion, hostAddress, port);
    if (reportInterval < 0.0D) {
      reportInterval = 5.0D;
    }
    if (observeAware == null) {
      observeAware = Boolean.valueOf(true);
    }
    if (pollInterval < 0.0D) {
      pollInterval = 0.0D;
    }
    if (hasUrgentTraffic == null) {
      hasUrgentTraffic = Boolean.valueOf(false);
    }
    if (gatewayIP == null) {
      gatewayIP = "0.0.0.0";
    }
    if (gatewaySubnet == null) {
      gatewaySubnet = "0.0.0.0";
    }
    if (defaultRouterIP == null) {
      defaultRouterIP = "0.0.0.0";
    }
    if (gatewayDNS == null) {
      gatewayDNS = "0.0.0.0";
    }
    if (secondaryDNS == null) {
      secondaryDNS = "0.0.0.0";
    }
    this.ReportInterval = reportInterval;
    this.ObserveAware = observeAware.booleanValue();
    
    this.PollInterval = pollInterval;
    this.HasUrgentTraffic = hasUrgentTraffic.booleanValue();
    
    this.GatewayIP = gatewayIP;
    this.GatewaySubnet = gatewaySubnet;
    this.DefaultRouterIP = defaultRouterIP;
    this.GatewayDNS = gatewayDNS;
    this.SecondaryDNS = secondaryDNS;
    
    this.PreSharedKey = presharedkey;
    
    getAssignedSensors();
  }
  
  public Gateway(int gatewayID, eGatewayType gatewayType, String gatewayFirmwareVersion, String radioFirmwareVersion, String hostAddress, int port, double reportInterval, Boolean observeAware, double pollInterval, Boolean hasUrgentTraffic, String gatewayIP, String gatewaySubnet, String defaultRouterIP, String gatewayDNS, String secondaryDNS)
  {
    this(gatewayID, gatewayType, gatewayFirmwareVersion, radioFirmwareVersion, hostAddress, port);
    if (reportInterval < 0.0D) {
      reportInterval = 5.0D;
    }
    if (observeAware == null) {
      observeAware = Boolean.valueOf(true);
    }
    if (pollInterval < 0.0D) {
      pollInterval = 0.0D;
    }
    if (hasUrgentTraffic == null) {
      hasUrgentTraffic = Boolean.valueOf(false);
    }
    if (gatewayIP == null) {
      gatewayIP = "0.0.0.0";
    }
    if (gatewaySubnet == null) {
      gatewaySubnet = "0.0.0.0";
    }
    if (defaultRouterIP == null) {
      defaultRouterIP = "0.0.0.0";
    }
    if (gatewayDNS == null) {
      gatewayDNS = "0.0.0.0";
    }
    if (secondaryDNS == null) {
      secondaryDNS = "0.0.0.0";
    }
    this.ReportInterval = reportInterval;
    this.ObserveAware = observeAware.booleanValue();
    
    this.PollInterval = pollInterval;
    this.HasUrgentTraffic = hasUrgentTraffic.booleanValue();
    
    this.GatewayIP = gatewayIP;
    this.GatewaySubnet = gatewaySubnet;
    this.DefaultRouterIP = defaultRouterIP;
    this.GatewayDNS = gatewayDNS;
    this.SecondaryDNS = secondaryDNS;
    
    getAssignedSensors();
  }
  
  public Gateway(int gatewayID, eGatewayType gatewayType, SecretKey presharedkey)
  {
    this(gatewayID, gatewayType, "3.0.0.0", "2.3.0.0", "0.0.0.0", 3000, presharedkey, 2.0D, null, 2.0D, null, null, null, null, null, null);
  }
  
  public Gateway(int gatewayID, eGatewayType gatewayType)
  {
    this(gatewayID, gatewayType, "3.0.0.0", "2.3.0.0", "0.0.0.0", 3000, 2.0D, null, 2.0D, null, null, null, null, null, null);
  }
  
  public int getGatewayID()
  {
    return this.GatewayID;
  }
  
  private void setGatewayID(int gatewayID)
  {
    this.GatewayID = gatewayID;
  }
  
  public eGatewayType getGatewayType()
  {
    return this.GatewayType;
  }
  
  private void setGatewayType(eGatewayType gatewayType)
  {
    this.GatewayType = gatewayType;
  }
  
  public String getRadioFirmwareVersion()
  {
    return this.RadioFirmwareVersion;
  }
  
  private void setRadioFirmwareVersion(String radioFirmwareVersion)
  {
    this.RadioFirmwareVersion = radioFirmwareVersion;
  }
  
  public String getGatewayFirmwareVersion()
  {
    return this.GatewayFirmwareVersion;
  }
  
  void setGatewayFirmwareVersion(String gatewayFirmwareVersion)
  {
    this.GatewayFirmwareVersion = gatewayFirmwareVersion;
  }
  
  public String getExternalIdentifier()
  {
    return this.ExternalIdentifier;
  }
  
  void setExternalIdentifier(String externalIdentifier)
  {
    this.ExternalIdentifier = externalIdentifier;
  }
  
  public double getReportInterval()
  {
    return this.ReportInterval;
  }
  
  void setReportInterval(double reportInterval)
  {
    this.ReportInterval = reportInterval;
  }
  
  public String getServerHostAddress()
  {
    return this.ServerHostAddress;
  }
  
  void setServerHostAddress(String serverHostAddress)
  {
    this.ServerHostAddress = serverHostAddress;
  }
  
  public int getPort()
  {
    return this.Port;
  }
  
  public void setPort(int port)
  {
    this.Port = port;
  }
  
  public String getGatewayIP()
  {
    return this.GatewayIP;
  }
  
  void setGatewayIP(String gatewayIP)
  {
    this.GatewayIP = gatewayIP;
  }
  
  public SecretKey getPreSharedKey()
  {
    return this.PreSharedKey;
  }
  
  void setPreSharedKey(SecretKey preSharedKey)
  {
    this.PreSharedKey = preSharedKey;
  }
  
  public SecretKey getEncryptionKey()
  {
    return this.EncryptionKey;
  }
  
  void setEncryptionKey(SecretKey encryptionKey)
  {
    this.EncryptionKey = encryptionKey;
  }
  
  public byte[] getEncryptionIV()
  {
    return this.EncryptionIV;
  }
  
  void setEncryptionIV(byte[] encryptionIV)
  {
    this.EncryptionIV = encryptionIV;
  }
  
  public Calendar getDecryptFailureDate()
  {
    return this.DecryptFailureDate;
  }
  
  public void setDecryptFailureDate(Calendar decryptFailureDate)
  {
    this.DecryptFailureDate = decryptFailureDate;
  }
  
  public String getGatewaySubnet()
  {
    return this.GatewaySubnet;
  }
  
  void setGatewaySubnet(String gatewaysubnet)
  {
    this.GatewaySubnet = gatewaysubnet;
  }
  
  public String getDefaultRouterIP()
  {
    return this.DefaultRouterIP;
  }
  
  void setDefaultRouterIP(String defaultrouterip)
  {
    this.DefaultRouterIP = defaultrouterip;
  }
  
  public String getGatewayDNS()
  {
    return this.GatewayDNS;
  }
  
  void setGatewayDNS(String gatewaydns)
  {
    this.GatewayDNS = gatewaydns;
  }
  
  public String getSecondaryDNS()
  {
    return this.SecondaryDNS;
  }
  
  void setSecondaryDNS(String secondarydns)
  {
    this.SecondaryDNS = secondarydns;
  }
  
  public double getNetworkListInterval()
  {
    return this.NetworkListInterval;
  }
  
  public void setNetworkListInterval(double networkListInterval)
  {
    this.NetworkListInterval = networkListInterval;
  }
  
  public boolean getObserveAware()
  {
    return this.ObserveAware;
  }
  
  public void setObserveAware(boolean observeAware)
  {
    this.ObserveAware = observeAware;
  }
  
  public Boolean getIsDirty()
  {
    if (this.IsDirty == null) {
      this.IsDirty = Boolean.valueOf(false);
    }
    return this.IsDirty;
  }
  
  void setIsDirty(Boolean isdirty)
  {
    this.IsDirty = isdirty;
  }
  
  public double getPollInterval()
  {
    return (this._PollInterval >= 0.0D) && (this._PollInterval <= 720.0D) ? this._PollInterval : 0.0D;
  }
  
  void setPollInterval(double pollInterval)
  {
    this.PollInterval = pollInterval;
  }
  
  public Boolean getHasUrgentTraffic()
  {
    return Boolean.valueOf(this.HasUrgentTraffic);
  }
  
  void setHasUrgentTraffic(Boolean hasurgenttraffic)
  {
    this.HasUrgentTraffic = hasurgenttraffic.booleanValue();
  }
  
  public Boolean getSendResetNetworkRequest()
  {
    return Boolean.valueOf(this.SendResetNetworkRequest);
  }
  
  void setSendResetNetworkRequest(Boolean sendresetnetworkrequest)
  {
    this.SendResetNetworkRequest = sendresetnetworkrequest.booleanValue();
  }
  
  public int getLastSequence()
  {
    return this.LastSequence;
  }
  
  public void setLastSequence(int lastSequence)
  {
    this.LastSequence = lastSequence;
  }
  
  public int getAutoConfigTime()
  {
    return this._AutoConfigTime;
  }
  
  public void setAutoConfigTime(int _AutoConfigTime)
  {
    this._AutoConfigTime = _AutoConfigTime;
  }
  
  public void setGPSReportInterval(double interval)
  {
    this.GPSReportInterval = interval;
    this.IsDirty = Boolean.valueOf(true);
  }
  
  public double getGPSReportInterval()
  {
    return this.GPSReportInterval;
  }
  
  public Boolean getPendingActionControlCommand()
  {
    if (this.PendingActionControlCommand == null) {
      this.PendingActionControlCommand = Boolean.valueOf(false);
    }
    return this.PendingActionControlCommand;
  }
  
  public List<byte[]> getActionControlCommands()
  {
    if (this.ActionControlCommands == null) {
      this.ActionControlCommands = new ArrayList();
    }
    return this.ActionControlCommands;
  }
  
  public static byte[] getAutoConfigCommand(long gatewayID, int time)
  {
    int length = 8;
    
    byte[] api = new byte[length];
    api[0] = 116;
    System.arraycopy(BitConverter.getFourBytes((int)gatewayID), 0, api, 1, 
      4);
    api[5] = 4;
    System.arraycopy(BitConverter.getTwoBytes(time), 0, api, 6, 2);
    
    return api;
  }
  
  public void setActionControlCommands(List<byte[]> actionControlCommands)
  {
    this.ActionControlCommands = actionControlCommands;
  }
  
  public void addActionControlCommand(byte[] comm)
  {
    if (this.ActionControlCommands == null) {
      this.ActionControlCommands = new ArrayList();
    }
    this.ActionControlCommands.add(comm);
  }
  
  public List<Long> getSensorIDs()
  {
    if (this._SensorIDs == null) {
      this._SensorIDs = new ArrayList();
    }
    return this._SensorIDs;
  }
  
  public List<Integer> getAssignedSensors()
  {
    if (this.AssignedSensors == null) {
      this.AssignedSensors = new ArrayList();
    }
    return this.AssignedSensors;
  }
  
  public void AssignSensor(int sensorID)
  {
    if (!this.AssignedSensors.contains(sensorID)) {
      this.AssignedSensors.add((sensorID));
    }
  }
  
  public void RemoveSensor(int sensorID)
  {
    this.AssignedSensors.remove(sensorID);
  }
  
  public void ForceConfiguration()
  {
    this.IsDirty = Boolean.valueOf(true);
  }
  
  public void UpdateServerInformation(String host, int port)
  {
    if (this.ServerHostAddress != host) {
      this.IsDirty = Boolean.valueOf(true);
    }
    if (this.Port != port) {
      this.IsDirty = Boolean.valueOf(true);
    }
    this.ServerHostAddress = host;
    this.Port = port;
  }
  
  public void UpdateReportInterval(double reportInterval)
  {
    if (reportInterval < 0.0D) {
      reportInterval = 0.0D;
    }
    if (reportInterval > 720.0D) {
      reportInterval = 720.0D;
    }
    if (this.ReportInterval != reportInterval) {
      this.IsDirty = Boolean.valueOf(true);
    }
    this.ReportInterval = reportInterval;
  }
  
  public void UpdatePollInterval(double pollInterval)
  {
    if (pollInterval < 0.0D) {
      pollInterval = 0.0D;
    }
    if (pollInterval > 720.0D) {
      pollInterval = 720.0D;
    }
    if (this.PollInterval != pollInterval) {
      this.IsDirty = Boolean.valueOf(true);
    }
    this.PollInterval = pollInterval;
  }
  
  public void UpdateGatewayIP(String staticIP, String subnet, String defaultRouter, String primaryDNS, String secondaryDNS)
  {
    if (this.GatewayIP != staticIP) {
      this.IsDirty = Boolean.valueOf(true);
    }
    if (this.GatewaySubnet != subnet) {
      this.IsDirty = Boolean.valueOf(true);
    }
    if (this.DefaultRouterIP != defaultRouter) {
      this.IsDirty = Boolean.valueOf(true);
    }
    if (this.GatewayDNS != primaryDNS) {
      this.IsDirty = Boolean.valueOf(true);
    }
    if (this.SecondaryDNS != secondaryDNS) {
      this.IsDirty = Boolean.valueOf(true);
    }
    this.GatewayIP = staticIP;
    this.GatewaySubnet = subnet;
    this.DefaultRouterIP = defaultRouter;
    this.GatewayDNS = primaryDNS;
    this.SecondaryDNS = secondaryDNS;
  }
  
  public void ReformNetwork()
  {
    this.SendResetNetworkRequest = true;
  }
  
  public static double BatteryPercentageHelper(eGatewayBattery batteryType, double reportedVoltage)
  {
    if (reportedVoltage < 0.1D) {
      return 100.0D;
    }
    switch (batteryType)
    {
    case Line: 
      return BatteryPercentageHelper(3.45D, 3.65D, 3.7D, 3.85D, 4.0D, 4.2D, 
        reportedVoltage);
    }
    return 100.0D;
  }
  
  public static double BatteryPercentageHelper(double zero, double ten, double twentyFive, double fifty, double seventyFive, double oneHundred, double reportedVoltage)
  {
    try
    {
      reportedVoltage = ((int)(reportedVoltage * 1000.0D) & 0x7FFF) / 1000.0D;
      if (reportedVoltage <= zero) {
        return 0.0D;
      }
      if ((reportedVoltage > zero) && (reportedVoltage <= ten)) {
        return LinearInterpolation(reportedVoltage, zero, 0.0D, ten, 10.0D);
      }
      if ((reportedVoltage > ten) && (reportedVoltage <= twentyFive)) {
        return LinearInterpolation(reportedVoltage, ten, 10.0D, 
          twentyFive, 25.0D);
      }
      if ((reportedVoltage > twentyFive) && (reportedVoltage <= fifty)) {
        return LinearInterpolation(reportedVoltage, twentyFive, 25.0D, 
          fifty, 50.0D);
      }
      if ((reportedVoltage > fifty) && (reportedVoltage <= seventyFive)) {
        return LinearInterpolation(reportedVoltage, fifty, 50.0D, 
          seventyFive, 75.0D);
      }
      if ((reportedVoltage > seventyFive) && (reportedVoltage <= oneHundred)) {
        return LinearInterpolation(reportedVoltage, seventyFive, 75.0D, 
          oneHundred, 100.0D);
      }
      return 100.0D;
    }
    catch (Exception e) {}
    return 0.0D;
  }
  
  private static double LinearInterpolation(double x, double lowX, double lowY, double highX, double highY)
  {
    double Slope = (highY - lowY) / (highX - lowX);
    double Inetercept = Slope * lowX - lowY;
    
    return Slope * x - Inetercept;
  }
}
