package com.monnit.mine.MonnitMineAPI;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.*;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.WebPackages.TcpListener;

/**
 * <p>
 * Responsible for TCPSocket objects
 * </p>
 * 
 * @author Monnit Corp
 * @version 0.8.0.1, 22 Oct 2015
 */
public class MineTCPSocket extends Thread {
	public InetAddress IPAddress;
	public int Port;
	private TcpListener TCPListener;

	public TcpListener getTCPListener() {
		return TCPListener;
	}

	public void setTCPListener(TcpListener tCPListener) {
		TCPListener = tCPListener;
	}

	private int _RunningClients = 0;
	private MineServer ServerInstance;

	public MineServer getServerInstance() {
		return ServerInstance;
	}

	public void setServerInstance(MineServer serverInstance) {
		ServerInstance = serverInstance;
	}

	MineTCPSocket(MineServer instance, InetAddress ipAddress, int port)
			throws IOException {
		// System.out.println(ipAddress.toString());
		IPAddress = ipAddress;
		Port = port;
		ServerInstance = instance;
	}

	void StartTCPListeners() throws Exception {
		Extensions.debug("Reached StartTCPListeners.");
		try {
			TCPListener = new TcpListener(IPAddress, Port, this);
			Extensions.debug("Initialized TCPListener.");
			TCPListener.start();
			Extensions.debug("Started TCPListener.");
			// TCPListener.BeginAcceptTcpClient(HandleTCPRequestAsync,
			// TCPListener);
		} catch (Exception ex) {
			Extensions.debug("Caught exception at StartTCPListeners"
					+ ex.toString());
			ServerInstance.OnLogException(ex, String.format(
					"MineTCPSocket.StartTCPListeners() IP:%s  Port:%d",
					IPAddress, Port));
			throw ex;// Throw the error so the service doesn't start without TCP
						// Support
		}
		Extensions.debug("Finished StartTCPListeners.");
	}

	void StopTCPListeners() throws InterruptedException, IOException {
		TCPListener.Stop();

		int Timeout = 30;// 30 sec timeout
		while (_RunningClients > 0 && Timeout > 0)// Try to let the thread
													// quietly finish
		{
			Thread.sleep(1000); // Check every second to see if all clients are
								// done
			Timeout--;
		}
	}

	

	public void HandleTCPClientComm(Socket clientSocket) throws IOException,
			InterruptedException {
		Extensions.debug("Reached HandleTCPClientComm", 30);
		try {
			_RunningClients++;
			int ReadTimeout = 30000;
			clientSocket.setSoTimeout(ReadTimeout);// 30 seconds

			InputStream inputStream = clientSocket.getInputStream();
			byte[] inboundBuffer = new byte[4096];
			while (true) {
				int bytesRead = inputStream.read(inboundBuffer, 0, 4096);
				if (bytesRead <= 0)
					break; // eof
				byte[] Message = new byte[bytesRead];
				System.arraycopy(inboundBuffer, 0, Message, 0, bytesRead);
				Extensions.debug(
						"Initial Message: " + Extensions.hexStr(Message), 51);
				byte[] response = ServerInstance.ProcessBytes(Message);
				Extensions.debug("Response (Custom, using ProcessBytes): "
						+ Extensions.hexStr(response), 51);
				OutputStream outputStream = clientSocket.getOutputStream();
				outputStream.write(response, 0, response.length);
				outputStream.flush();
				Extensions.debug("Sent response through default netcode.", 51);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			ServerInstance.OnLogException(ex,
					"MineTCPSocket.HandleTCPClientComm(object client)");
		} finally {
			_RunningClients--;
			clientSocket.close();
			Extensions.debug("Closed tcpClient.", 50);
		}
	}
}
