package com.monnit.mine.MonnitMineAPI;

import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.EventHandling.IAsyncResult;
import com.monnit.mine.WebPackages.IPEndPoint;
import com.monnit.mine.WebPackages.UdpClient;

/**
 * <p>
 * Responsible for UDPSocket objects
 * </p>
 * 
 * @author Monnit Corp
 * @version 0.8.0.1, 22 Oct 2015
 */
public class MineUDPSocket {

	private int Port;

	public int getPort() {
		return Port;
	}

	public void setPort(int port) throws SocketException {
		Port = port;
	}

	private UdpClient UDPListener;

	public UdpClient getUDPListener() {
		return UDPListener;
	}

	public void setUDPListener(UdpClient uDPListener) {
		UDPListener = uDPListener;
	}

	volatile private int _RunningClients = 0;
	private MineServer ServerInstance;

	public MineServer getServerInstance() {
		return ServerInstance;
	}

	public void setServerInstance(MineServer serverInstance) {
		ServerInstance = serverInstance;
	}

	private Object UdpConLock = new Object();
	private static List<InboundPacket> ActiveUDPConnections = new ArrayList<InboundPacket>();

	MineUDPSocket(MineServer instance, int port) throws SocketException {
		Port = port;
		ServerInstance = instance;
	}

	void StartUDPListeners() throws Exception {
		Extensions.debug("Reached StartUDPListeners.");
		try {
			UDPListener = new UdpClient(Port, this);
			Extensions.debug("Initialized UDPClient.");
			UDPListener.start();
			// UDPListener.BeginReceive();//HandleUDPRequestAsync, UDPListener);
			Extensions.debug("Finished UDPListener.BeginReceive.");
		} catch (Exception ex) {
			Extensions
					.debug("Caught (and rethrew) exception at StartUDPListeners.");
			// ServerInstance.OnLogException(ex,
			// String.format("MineUDPSocket.StartUDPSockets() IP:%d  Port:%d  - (Failed on Startup) ",
			// IPAddress, Port));
			throw ex;// Throw the error so the service doesn't start without UDP
						// Support
		}
		Extensions.debug("Finished StartUDPListeners.");
	}

	void StopUDPListeners() throws InterruptedException {
		UDPListener.close();

		int Timeout = 30;// 30 sec timeout
		while (_RunningClients > 0 && Timeout > 0)// Try to let the thread
													// quietly finish
		{
			Thread.sleep(1000);// Check every second to see if all clients are
								// done
			Timeout--;
		}
	}

	public void HandleUDPRequestAsync(IAsyncResult ar) throws Exception {
		Extensions.debug("Entered HandleUDPRequestAsync", 35);
		try {
			UdpClient UdpListener = (UdpClient) ar.AsyncState;
			InetAddress LocalEndPoint = UdpListener.socket.getLocalAddress();
			int port = UdpListener.socket.getLocalPort();
			DatagramPacket localMsg;
			IPEndPoint clientEP = new IPEndPoint(
					new InetSocketAddress(0).getAddress(), 0);
			try {
				localMsg = UdpListener.EndReceive(ar, clientEP);
				clientEP = UdpListener.clientEP;
				Extensions.debug("Got localMsg.", 35);
			} catch (Exception ex) {
				Extensions.debug(
						"Caught Exception in upper inner try of HandleUDPRequestAsync: "
								+ ex.toString(), 35);
				ServerInstance
						.OnLogException(
								ex,
								"MineUDPSocket.HandleUDPRequestAsync(IAsyncResult ar) -> UdpSocket.EndReceiveFrom(ar, ref clientEP)");
				UdpListener.close();

				UdpClient NewListener = new UdpClient(port, this);
				NewListener.BeginReceive();// HandleUDPRequestAsync,
											// NewListener);

				return;
			}

			try {
				Extensions.debug(
						"Starting new listener (waiting until port opens)", 35);
				// The listener restarts itself
				// UdpListener.close();
				// UdpListener.interrupt();
				// UdpListener.start();
				// UdpListener.BeginReceive();//HandleUDPRequestAsync,
				// UdpListener);
				UdpClient NewListener = new UdpClient(port, this);
				NewListener.start();
			} catch (Exception ex) {
				Extensions.debug(
						"Caught Exception in lower inner try of HandleUDPRequestAsync: "
								+ ex.toString(), 35);
				// ServerInstance.OnLogException(ex,
				// "MineUDPSocket.HandleUDPRequestAsync(IAsyncResult ar) -> UdpSocket.BeginReceive(HandleUDPRequestAsync, UdpListener)");

				// start a new listener...
				UdpClient NewListener = new UdpClient(port, this);
				NewListener.start();
				// NewListener.BeginReceive();//HandleUDPRequestAsync,
				// NewListener);
			}

			Extensions.debug("Going to HandleUDPSocketComm", 35);
			HandleUDPSocketComm(localMsg, clientEP, UdpListener);
		} catch (Exception ex) {
			Extensions.debug(
					"Caught Exception in outer try of HandleUDPRequestAsync: "
							+ ex.toString(), 35);
			ServerInstance.OnLogException(ex,
					"MineUDPSocket.HandleUDPRequestAsync(IAsyncResult ar)");

			// Fail all
			StopUDPListeners();
			StartUDPListeners();
		}
		Extensions.debug("Exiting HandleUDPRequestAsync.", 35);
	}

	private void HandleUDPSocketComm(DatagramPacket packet,
			IPEndPoint clientEndPoint, UdpClient udpListener)
			throws InterruptedException {
		Extensions.debug("Entered HandleUDPSocketComm", 35);
		try {
			_RunningClients++;
			byte[] Message = packet.getData();
			int bytesRead = packet.getLength();
			Message = Arrays.copyOf(Message, bytesRead);
			try {
				byte[] ResponseBuffer = ServerInstance.ProcessBytes(Message);
				udpListener.SendTo(ResponseBuffer, clientEndPoint);
			} catch (Exception e) {
				ServerInstance
						.OnLogException(e,
								"MineUDPSocket.HandleUDPSocketComm(), at ProcessBytes call.");
			}
		} catch (Exception ex) {
			Extensions
					.debug("Caught Exception in lowest catch of HandleUDPSocketComm: "
							+ ex.toString());
			ServerInstance
					.OnLogException(
							ex,
							"MineUDPSocket.HandleUDPSocketComm(byte[] Message, IPEndPoint clientEndPoint, UdpClient udpListener)");
		} finally {
			_RunningClients--;
		}

		Extensions.debug("Exiting HandleUDPSocketComm.", 35);
	}
}
