package com.monnit.mine.MonnitMineAPI;

import com.monnit.mine.BaseApplication.BitConverter;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication._ApplicationBase;
import com.monnit.mine.BaseApplication.enums.eApplicationProfileType;
import com.monnit.mine.MonnitMineAPI.enums.eConfigSection;
import com.monnit.mine.MonnitMineAPI.enums.eSensorApplication;
import com.monnit.mine.MonnitMineAPI.enums.eSensorBattery;
import java.util.ArrayList;
import java.util.List;

public class Sensor
{
  public int SensorID;
  public eSensorApplication MonnitApplication;
  public eApplicationProfileType ApplicationProfileType;
  public double ReportInterval;
  double ActiveStateInterval;
  public Boolean GeneralConfig1Dirty;
  public int Recovery;
  public int RetryCount;
  public Boolean GeneralConfig2Dirty;
  int MeasurementsPerTransmission;
  long MaximumThreshold;
  long MinimumThreshold;
  long Hysteresis;
  int EventDetectionCount;
  int EventDetectionPeriod;
  int EventDetectionType;
  int BiStable;
  int RearmTime;
  Boolean ProfileConfig1Dirty;
  long Calibration1;
  long Calibration2;
  long Calibration3;
  long Calibration4;
  Boolean ProfileConfig2Dirty;
  Boolean PendingActionControlCommand;// was pendingActionControlCommmand
  List<byte[]> ActionControlCommands;// was actioncontrolcommand
  Boolean ActionControlCommandIsUrgent;
  Boolean DataLog;
  public int TransmitOffset = Integer.MIN_VALUE;
  public String FirmwareVersion;
  public int TransmitIntervalLink;
  
  public Sensor(int sensorID, eSensorApplication monnitApplication, String firmwareVersion)
    throws Exception
  {
    this.SensorID = sensorID;
    this.MonnitApplication = monnitApplication;
    this.FirmwareVersion = firmwareVersion;
    Class<?> T = _ApplicationBase.GetType(this.MonnitApplication.Value());
    
    this.ReportInterval = 120.0D;
    this.ActiveStateInterval = 120.0D;
    
    this.TransmitIntervalLink = ((Integer)Extensions.getStaticPropertyValue(T, "DefaultTransmitIntervalLink")).intValue();
    
    this.RetryCount = ((Integer)Extensions.getStaticPropertyValue(T, 
      "DefaultRetryCount")).intValue();
    this.Recovery = ((Integer)
      Extensions.getStaticPropertyValue(T, "DefaultRecovery")).intValue();
    
    this.MeasurementsPerTransmission = ((Integer)Extensions.getStaticPropertyValue(
      T, "DefaultMeasurementsPerTransmission")).intValue();
    this.MeasurementsPerTransmission = ((Integer)Extensions.getStaticPropertyValue(
      T, "DefaultMeasurementsPerTransmission")).intValue();
    
    this.Hysteresis = ((Long)Extensions.getStaticPropertyValue(T, 
      "DefaultHysteresis")).longValue();
    this.MinimumThreshold = ((Long)Extensions.getStaticPropertyValue(T, 
      "DefaultMinimumThreshold")).longValue();
    this.MaximumThreshold = ((Long)Extensions.getStaticPropertyValue(T, 
      "DefaultMaximumThreshold")).longValue();
    this.Calibration1 = ((Long)Extensions.getStaticPropertyValue(T, 
      "DefaultCalibration1")).longValue();
    this.Calibration2 = ((Long)Extensions.getStaticPropertyValue(T, 
      "DefaultCalibration2")).longValue();
    this.Calibration3 = ((Long)Extensions.getStaticPropertyValue(T, 
      "DefaultCalibration3")).longValue();
    this.Calibration4 = ((Long)Extensions.getStaticPropertyValue(T, 
      "DefaultCalibration4")).longValue();
    this.EventDetectionType = ((Integer)Extensions.getStaticPropertyValue(T, 
      "DefaultEventDetectionType")).intValue();
    this.EventDetectionPeriod = ((Integer)Extensions.getStaticPropertyValue(T, 
      "DefaultEventDetectionPeriod")).intValue();
    this.EventDetectionCount = ((Integer)Extensions.getStaticPropertyValue(T, 
      "DefaultEventDetectionCount")).intValue();
    this.RearmTime = ((Integer)Extensions.getStaticPropertyValue(T, 
      "DefaultRearmTime")).intValue();
    this.BiStable = ((Integer)
      Extensions.getStaticPropertyValue(T, "DefaultBiStable")).intValue();
  }
  
  public Sensor(int sensorID, eSensorApplication monnitApplication, String firmwareVersion, double reportInterval, int linkInterval, int retryCount, int recoveryCount)
    throws Exception
  {
    this(sensorID, monnitApplication, firmwareVersion);
    if (reportInterval < 0.0D) {
      reportInterval = 120.0D;
    }
    if (linkInterval < 0) {
      linkInterval = 120;
    }
    if (retryCount < 0) {
      retryCount = 2;
    }
    if (recoveryCount < 0) {
      recoveryCount = 2;
    }
    UpdateReportInterval(reportInterval);
    UpdateLinkInterval(linkInterval);
    this.GeneralConfig1Dirty = Boolean.valueOf(false);
    
    UpdateRetryCount(retryCount);
    UpdateRecoveryCount(recoveryCount);
    this.GeneralConfig2Dirty = Boolean.valueOf(false);
  }
  
  public Sensor(int sensorID, eSensorApplication monnitApplication, String firmwareVersion, double reportInterval, int linkInterval, int retryCount)
    throws Exception
  {
    this(sensorID, monnitApplication, firmwareVersion, reportInterval, linkInterval, retryCount, -1);
  }
  
  public Sensor(int sensorID, eSensorApplication monnitApplication, String firmwareVersion, double reportInterval, int linkInterval)
    throws Exception
  {
    this(sensorID, monnitApplication, firmwareVersion, reportInterval, linkInterval, -1, -1);
  }
  
  public Sensor(int sensorID, eSensorApplication monnitApplication, String firmwareVersion, double reportInterval)
    throws Exception
  {
    this(sensorID, monnitApplication, firmwareVersion, reportInterval, -1, -1, -1);
  }
  
  public int getSensorID()
  {
    return this.SensorID;
  }
  
  private void setSensorID(int sensorID)
  {
    this.SensorID = sensorID;
  }
  
  public eSensorApplication getMonnitApplication()
  {
    return this.MonnitApplication;
  }
  
  private void setMonnitApplication(eSensorApplication monnitApplication)
  {
    this.MonnitApplication = monnitApplication;
  }
  
  public String getFirmwareVersion()
  {
    return this.FirmwareVersion;
  }
  
  private void setFirmwareVersion(String firmwareVersion)
  {
    this.FirmwareVersion = firmwareVersion;
  }
  
  public double getReportInterval()
  {
    return this.ReportInterval;
  }
  
  private void setReportInterval(double reportInterval)
  {
    this.ReportInterval = reportInterval;
  }
  
  public double getActiveStateInterval()
  {
    return this.ActiveStateInterval;
  }
  
  public void setActiveStateInterval(double activeStateInterval)
  {
    this.ActiveStateInterval = activeStateInterval;
  }
  
  public int getTransmitIntervalLink()
  {
    return this.TransmitIntervalLink;
  }
  
  private void setTransmitIntervalLink(int transmitIntervalLink)
  {
    this.TransmitIntervalLink = transmitIntervalLink;
  }
  
  public Boolean getGeneralConfig1Dirty()
  {
    if (this.GeneralConfig1Dirty == null) {
      this.GeneralConfig1Dirty = Boolean.valueOf(false);
    }
    return this.GeneralConfig1Dirty;
  }
  
  public void setGeneralConfig1Dirty(Boolean generalConfig1Dirty)
  {
    this.GeneralConfig1Dirty = generalConfig1Dirty;
  }
  
  public int getRecovery()
  {
    return this.Recovery;
  }
  
  public void setRecovery(int recovery)
  {
    this.Recovery = recovery;
  }
  
  public int getRetryCount()
  {
    return this.RetryCount;
  }
  
  public void setRetryCount(int retryCount)
  {
    this.RetryCount = retryCount;
  }
  
  public Boolean getGeneralConfig2Dirty()
  {
    if (this.GeneralConfig2Dirty == null) {
      this.GeneralConfig2Dirty = Boolean.valueOf(false);
    }
    return this.GeneralConfig2Dirty;
  }
  
  public void setGeneralConfig2Dirty(Boolean generalConfig2Dirty)
  {
    this.GeneralConfig2Dirty = generalConfig2Dirty;
  }
  
  public int getMeasurementsPerTransmission()
  {
    return this.MeasurementsPerTransmission;
  }
  
  public void setMeasurementsPerTransmission(int measurementsPerTransmission)
  {
    this.MeasurementsPerTransmission = measurementsPerTransmission;
  }
  
  public long getMaximumThreshold()
  {
    return this.MaximumThreshold;
  }
  
  public void setMaximumThreshold(long maximumThreshold)
  {
    this.MaximumThreshold = maximumThreshold;
  }
  
  public long getMinimumThreshold()
  {
    return this.MinimumThreshold;
  }
  
  public void setMinimumThreshold(long minimumThreshold)
  {
    this.MinimumThreshold = minimumThreshold;
  }
  
  public long getHysteresis()
  {
    return this.Hysteresis;
  }
  
  public void setHysteresis(long hysteresis)
  {
    this.Hysteresis = hysteresis;
  }
  
  public int getEventDetectionCount()
  {
    return this.EventDetectionCount;
  }
  
  public void setEventDetectionCount(int eventDetectionCount)
  {
    this.EventDetectionCount = eventDetectionCount;
  }
  
  public int getEventDetectionPeriod()
  {
    return this.EventDetectionPeriod;
  }
  
  public void setEventDetectionPeriod(int eventDetectionPeriod)
  {
    this.EventDetectionPeriod = eventDetectionPeriod;
  }
  
  public int getEventDetectionType()
  {
    return this.EventDetectionType;
  }
  
  public void setEventDetectionType(int eventDetectionType)
  {
    this.EventDetectionType = eventDetectionType;
  }
  
  public int getBiStable()
  {
    return this.BiStable;
  }
  
  public void setBiStable(int biStable)
  {
    this.BiStable = biStable;
  }
  
  public int getRearmTime()
  {
    return this.RearmTime;
  }
  
  public void setRearmTime(int rearmTime)
  {
    this.RearmTime = rearmTime;
  }
  
  public Boolean getProfileConfig1Dirty()
  {
    if (this.ProfileConfig1Dirty == null) {
      this.ProfileConfig1Dirty = Boolean.valueOf(false);
    }
    return this.ProfileConfig1Dirty;
  }
  
  public void setProfileConfig1Dirty(Boolean profileConfig1Dirty)
  {
    this.ProfileConfig1Dirty = profileConfig1Dirty;
  }
  
  public long getCalibration1()
  {
    return this.Calibration1;
  }
  
  public void setCalibration1(long calibration1)
  {
    this.Calibration1 = calibration1;
  }
  
  public long getCalibration2()
  {
    return this.Calibration2;
  }
  
  public void setCalibration2(long calibration2)
  {
    this.Calibration2 = calibration2;
  }
  
  public long getCalibration3()
  {
    return this.Calibration3;
  }
  
  public void setCalibration3(long calibration3)
  {
    this.Calibration3 = calibration3;
  }
  
  public long getCalibration4()
  {
    return this.Calibration4;
  }
  
  public void setCalibration4(long calibration4)
  {
    this.Calibration4 = calibration4;
  }
  
  public Boolean getProfileConfig2Dirty()
  {
    if (this.ProfileConfig2Dirty == null) {
      this.ProfileConfig2Dirty = Boolean.valueOf(false);
    }
    return this.ProfileConfig2Dirty;
  }
  
  public void setProfileConfig2Dirty(Boolean profileConfig2Dirty)
  {
    this.ProfileConfig2Dirty = profileConfig2Dirty;
  }
  
  void FillConfig(eConfigSection section, byte[] configData)
    throws Exception
  {
    switch (section)
    {
    case General_1: 
      FillGeneralConfig1(configData);
      break;
    case General_2: 
      FillGeneralConfig2(configData);
      break;
    case Profile_1: 
      FillProfileConfig1(configData);
      break;
    case Profile_2: 
      FillProfileConfig2(configData);
    }
  }
  
  private void FillGeneralConfig1(byte[] configData)
    throws Exception
  {
    this.ActiveStateInterval = (BitConverter.ToUInt16(configData, 10) / 60);
    this.ReportInterval = (BitConverter.ToUInt16(configData, 12) / 60);
    this.TransmitIntervalLink = configData[14];
  }
  
  private void FillGeneralConfig2(byte[] configData)
  {
    this.RetryCount = configData[2];
    this.Recovery = configData[3];
  }
  
  private void FillProfileConfig1(byte[] configData)
    throws Exception
  {
    if (configData[0] == eApplicationProfileType.Interval.Value())
    {
      this.MeasurementsPerTransmission = configData[1];
      this.Hysteresis = BitConverter.toInt32(configData, 4);
      this.MinimumThreshold = BitConverter.toInt32(configData, 8);
      this.MaximumThreshold = BitConverter.toInt32(configData, 12);
      
    }
    else
    {
      this.EventDetectionType = BitConverter.toInt32(configData, 1);
      this.EventDetectionPeriod = BitConverter.ToUInt16(configData, 2);
      this.EventDetectionCount = BitConverter.ToUInt16(configData, 4);
      this.RearmTime = BitConverter.ToUInt16(configData, 6);
      this.BiStable = BitConverter.ToUInt16(configData, 8);
    
    }
  }
  
  private void FillProfileConfig2(byte[] configData)
    throws Exception
  {
    this.Calibration1 = BitConverter.toInt32(configData, 0);
    this.Calibration2 = BitConverter.toInt32(configData, 4);
    this.Calibration3 = BitConverter.toInt32(configData, 8);
    this.Calibration4 = BitConverter.toInt32(configData, 12);
  }
  
  public void setPendingActionControlCommand(Boolean isPending)
  {
    this.PendingActionControlCommand = isPending;
  }
  
  public Boolean getPendingActionControlCommand()
  {
    if (this.PendingActionControlCommand == null) {
      this.PendingActionControlCommand = Boolean.valueOf(false);
    }
    return this.PendingActionControlCommand;
  }
  
  public List<byte[]> getActionControlCommands()
  {
    if (this.ActionControlCommands == null) {
      this.ActionControlCommands = new ArrayList();
    }
    return this.ActionControlCommands;
  }
  
  public void removeActionControlCommands(int index)
  {
    this.ActionControlCommands.remove(index);
  }
  
  public void setActionControlCommands(List<byte[]> actionControlCommands)
  {
    this.ActionControlCommands = actionControlCommands;
  }
  
  public void addActionControlCommand(byte[] comm)
  {
    if (this.ActionControlCommands == null) {
      this.ActionControlCommands = new ArrayList();
    }
    this.ActionControlCommands.add(comm);
  }
  
  public Boolean getActionControlCommandIsUrgent()
  {
    if (this.ActionControlCommandIsUrgent == null) {
      this.ActionControlCommandIsUrgent = Boolean.valueOf(false);
    }
    return this.ActionControlCommandIsUrgent;
  }
  
  public void setActionControlCommandIsUrgent(Boolean actionControlCommandIsUrgent)
  {
    this.ActionControlCommandIsUrgent = actionControlCommandIsUrgent;
  }
  
  public static byte[] ReadConfigCommand(eConfigSection section, int sensorid)
  {
    byte[] api = new byte[6];
    api[0] = 112;
    System.arraycopy(BitConverter.getBytes(sensorid), 0, api, 1, 4);
    api[5] = ((byte)section.Value());
    return api;
  }
  
  public void MarkClean()
  {
    this.GeneralConfig1Dirty = Boolean.valueOf(false);
    this.GeneralConfig2Dirty = Boolean.valueOf(false);
    
    this.ProfileConfig1Dirty = Boolean.valueOf(false);
    this.ProfileConfig2Dirty = Boolean.valueOf(false);
    this.PendingActionControlCommand = Boolean.valueOf(false);
  }
  
  byte[] ConfigSection(eConfigSection section)
    throws Exception
  {
    switch (section)
    {
    case General_1: 
      return GeneralConfig1();
    case General_2: 
      return GeneralConfig2();
    case Profile_1: 
      return ProfileConfig1();
    case Profile_2: 
      return ProfileConfig2();
    }
    return null;
  }
  
  private byte[] GeneralConfig1()
  {
    Extensions.debug("    Reached GeneralConfig1", 14);
    
    Extensions.debug(
      "Values as they are called here:\n        this.ActiveStateInterval: " + 
      this.ActiveStateInterval + "\n        " + 
      this.ReportInterval, 14);
    
    byte[] api = new byte[22];
    
    api[0] = 114;
    System.arraycopy(BitConverter.getBytes(this.SensorID), 0, api, 1, 4);
    
    api[5] = 24;
    
    System.arraycopy(BitConverter.getBytes(-1), 0, api, 6, 4);
    
    System.arraycopy(
      BitConverter.getTwoBytes(this.MonnitApplication.Value()), 0, 
      api, 10, 2);
    System.arraycopy(BitConverter.getTwoBytes(50), 0, api, 12, 2);
    
    System.arraycopy(
      BitConverter.getTwoBytes((int)(this.ActiveStateInterval * 60.0D)), 
      0, api, 14, 2);
    System.arraycopy(
      BitConverter.getTwoBytes((int)(this.ReportInterval * 60.0D)), 0, 
      api, 16, 2);
    api[18] = ((byte)this.TransmitIntervalLink);
    
    api[19] = 126;
    
    api[20] = 1;
    
    api[21] = 1;
    
    Extensions.debug("    Exiting GeneralConfig1", 14);
    return api;
  }
  
  private byte[] GeneralConfig2()
  {
    byte[] api = new byte[22];
    
    api[0] = 114;
    System.arraycopy(BitConverter.getBytes(this.SensorID), 0, api, 1, 4);
    
    api[5] = 25;
    api[6] = ((byte)this.RetryCount);
    
    api[7] = ((byte)this.Recovery);
    System.arraycopy(BitConverter.getSixBytes(-1), 0, api, 8, 6);
    
    System.arraycopy(BitConverter.getSixBytes(-1), 0, api, 14, 6);
    
    System.arraycopy(BitConverter.getBytes(65535), 0, api, 20, 2);
    
    return api;
  }
  
  private byte[] ProfileConfig1()
    throws Exception
  {
    byte[] api = new byte[22];
    
    api[0] = 114;
    System.arraycopy(BitConverter.getBytes(this.SensorID), 0, api, 1, 4);
    
    api[5] = 28;
    if (_ApplicationBase.ProfileType(this.MonnitApplication.Value()) == eApplicationProfileType.Interval)
    {
      api[6] = ((byte)eApplicationProfileType.Interval.Value());
      
      api[7] = ((byte)this.MeasurementsPerTransmission);
      System.arraycopy(BitConverter.getBytes(0), 0, api, 8, 2);
      System.arraycopy(BitConverter.getBytes(this.Hysteresis), 0, api, 
        10, 4);
      System.arraycopy(BitConverter.getBytes(this.MinimumThreshold), 0, 
        api, 14, 4);
      System.arraycopy(BitConverter.getBytes(this.MaximumThreshold), 0, 
        api, 18, 4);
    }
    else if (_ApplicationBase.ProfileType(this.MonnitApplication.Value()) == eApplicationProfileType.Trigger)
    {
      api[6] = ((byte)eApplicationProfileType.Trigger.Value());
      
      api[7] = ((byte)this.EventDetectionType);
      System.arraycopy(BitConverter.getBytes(this.EventDetectionPeriod), 
        0, api, 8, 2);
      System.arraycopy(BitConverter.getBytes(this.EventDetectionCount), 
        0, api, 10, 2);
      System.arraycopy(BitConverter.getBytes(this.RearmTime), 0, api, 12, 
        2);
      api[14] = ((byte)this.BiStable);
      System.arraycopy(BitConverter.getBytes(-1), 0, api, 15, 7);
    }
    return api;
  }
  
  private byte[] ProfileConfig2()
    throws Exception
  {
    byte[] api = new byte[22];
    
    api[0] = 114;
    System.arraycopy(BitConverter.getBytes(this.SensorID), 0, api, 1, 4);
    
    api[5] = 29;
    if (_ApplicationBase.ProfileType(this.MonnitApplication.Value()) == eApplicationProfileType.Interval)
    {
      System.arraycopy(BitConverter.getBytes(this.Calibration1), 0, api, 
        6, 4);
      System.arraycopy(BitConverter.getBytes(this.Calibration2), 0, api, 
        10, 4);
      System.arraycopy(BitConverter.getBytes(this.Calibration3), 0, api, 
        14, 4);
      System.arraycopy(BitConverter.getBytes(this.Calibration4), 0, api, 
        18, 4);
    }
    else if (_ApplicationBase.ProfileType(this.MonnitApplication.Value()) == eApplicationProfileType.Trigger)
    {
      System.arraycopy(BitConverter.getBytes(-1), 0, api, 6, 8);
      System.arraycopy(BitConverter.getBytes(-1), 0, api, 14, 8);
    }
    return api;
  }
  
  public Boolean getDataLog()
  {
    return this.DataLog;
  }
  
  public void setDataLog(Boolean dataLog)
  {
    this.DataLog = dataLog;
  }
  
  public void ForceConfiguration(eConfigSection section)
  {
    switch (section)
    {
    case General_1: 
      this.GeneralConfig1Dirty = Boolean.valueOf(true);
      break;
    case General_2: 
      this.GeneralConfig2Dirty = Boolean.valueOf(true);
      break;
    case Profile_1: 
      this.ProfileConfig1Dirty = Boolean.valueOf(true);
      break;
    case Profile_2: 
      this.ProfileConfig2Dirty = Boolean.valueOf(true);
    }
  }
  
  public void UpdateReportInterval(double reportInterval)
  {
    if (reportInterval < 0.017D) {
      reportInterval = 0.017D;
    }
    if (reportInterval > 720.0D) {
      reportInterval = 720.0D;
    }
    if (this.ReportInterval != reportInterval) {
      this.GeneralConfig1Dirty = Boolean.valueOf(true);
    }
    this.ReportInterval = reportInterval;
    this.ActiveStateInterval = reportInterval;
  }
  
  public void UpdateLinkInterval(int linkInterval)
  {
    if (linkInterval < 1) {
      linkInterval = 1;
    }
    if (linkInterval > 720) {
      linkInterval = 720;
    }
    if (linkInterval < 60) {
      linkInterval += 100;
    } else {
      linkInterval = (int)Math.floor(linkInterval / 60.0D + 0.5D);
    }
    if (this.TransmitIntervalLink != linkInterval) {
      this.GeneralConfig1Dirty = Boolean.valueOf(true);
    }
    this.TransmitIntervalLink = linkInterval;
  }
  
  public void UpdateRetryCount(int retryCount)
  {
    if (retryCount < 0) {
      retryCount = 0;
    }
    if (retryCount > 10) {
      retryCount = 10;
    }
    if (this.RetryCount != retryCount) {
      this.GeneralConfig2Dirty = Boolean.valueOf(true);
    }
    this.RetryCount = retryCount;
  }
  
  public void UpdateRecoveryCount(int recovery)
  {
    if (recovery < 0) {
      recovery = 0;
    }
    if (recovery > 10) {
      recovery = 10;
    }
    if (this.Recovery != recovery) {
      this.GeneralConfig2Dirty = Boolean.valueOf(true);
    }
    this.Recovery = recovery;
  }
  
  public static double BatteryPercentageHelper(eSensorBattery batteryType, double reportedVoltage)
  {
    switch (batteryType)
    {
    case AAx1r: 
      return BatteryPercentageHelper(2.7D, 2.75D, 2.8D, 2.85D, 2.9D, 3.0D, 
        reportedVoltage);
    case AAx2: 
      return BatteryPercentageHelper(2.4D, 2.55D, 2.6D, 2.8D, 2.9D, 3.0D, 
        reportedVoltage);
    case CR123: 
      return BatteryPercentageHelper(2.3D, 2.7D, 2.8D, 3.0D, 3.1D, 3.25D, 
        reportedVoltage);
    case CR2032: 
      return BatteryPercentageHelper(2.1D, 2.2D, 2.4D, 2.7D, 3.0D, 3.3D, 
        reportedVoltage);
    case Line: 
      return BatteryPercentageHelper(2.4D, 2.55D, 2.6D, 2.8D, 2.9D, 3.0D, 
        reportedVoltage);
    }
    return 100.0D;
  }
  
  public static double BatteryPercentageHelper(double zero, double ten, double twentyFive, double fifty, double seventyFive, double oneHundred, double reportedVoltage)
  {
    try
    {
      reportedVoltage = ((int)(reportedVoltage * 1000.0D) & 0x7FFF) / 1000.0D;
      if (reportedVoltage <= zero) {
        return 0.0D;
      }
      if ((reportedVoltage > zero) && (reportedVoltage <= ten)) {
        return LinearInterpolation(reportedVoltage, zero, 0.0D, ten, 10.0D);
      }
      if ((reportedVoltage > ten) && (reportedVoltage <= twentyFive)) {
        return LinearInterpolation(reportedVoltage, ten, 10.0D, 
          twentyFive, 25.0D);
      }
      if ((reportedVoltage > twentyFive) && (reportedVoltage <= fifty)) {
        return LinearInterpolation(reportedVoltage, twentyFive, 25.0D, 
          fifty, 50.0D);
      }
      if ((reportedVoltage > fifty) && (reportedVoltage <= seventyFive)) {
        return LinearInterpolation(reportedVoltage, fifty, 50.0D, 
          seventyFive, 75.0D);
      }
      if ((reportedVoltage > seventyFive) && (reportedVoltage <= oneHundred)) {
        return LinearInterpolation(reportedVoltage, seventyFive, 75.0D, 
          oneHundred, 100.0D);
      }
      return 100.0D;
    }
    catch (Exception e) {}
    return 0.0D;
  }
  
  public static double LinearInterpolation(double x, double lowX, double lowY, double highX, double highY)
  {
    double Slope = (highY - lowY) / (highX - lowX);
    double Inetercept = Slope * lowX - lowY;
    
    return Slope * x - Inetercept;
  }
}