package com.monnit.mine.MonnitMineAPI;

import com.monnit.mine.BaseApplication.Datum;
import com.monnit.mine.BaseApplication.ExtensionMethods.Extensions;
import com.monnit.mine.BaseApplication._ApplicationBase;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;

public class SensorMessage
{
  public int SensorID;
  public int ProfileID;
  public Calendar MessageDate;
  public int State;
  public int SignalStrength;
  public double Voltage;
  private byte[] _RawData;
  public List<Datum> Data;
  
  public SensorMessage()
    throws InterruptedException
  {}
  
  public SensorMessage(int sensorID, int profileID, Calendar messageDate, int state, int signalStrength, double voltage, byte[] data)
    throws InterruptedException
  {
    Extensions.debug(
      "Constructed a sensor message. ProfileID: " + profileID, 21);
    this.SensorID = sensorID;
    this.ProfileID = profileID;
    this.MessageDate = messageDate;
    this.State = state;
    this.SignalStrength = signalStrength;
    this.Voltage = voltage;
    this._RawData = data;
  }
  
  public int getSensorID()
  {
    return this.SensorID;
  }
  
  public void setSensorID(int sensorID)
  {
    this.SensorID = sensorID;
  }
  
  public int getProfileID()
  {
    return this.ProfileID;
  }
  
  public void setProfileID(int profileID)
  {
    this.ProfileID = profileID;
  }
  
  public Calendar getMessageDate()
  {
    return this.MessageDate;
  }
  
  public void setMessageDate(Calendar messageDate)
  {
    this.MessageDate = messageDate;
  }
  
  public int getState()
  {
    return this.State;
  }
  
  public void setState(int state)
  {
    this.State = state;
  }
  
  public int getSignalStrength()
  {
    return this.SignalStrength;
  }
  
  public void setSignalStrength(int signalStrength)
  {
    this.SignalStrength = signalStrength;
  }
  
  public double getVoltage()
  {
    return this.Voltage;
  }
  
  public void setVoltage(double voltage)
  {
    this.Voltage = voltage;
  }
  
  public byte[] getRawData()
  {
    return this._RawData;
  }
  
  public void setRawData(byte[] rawData)
  {
    this._RawData = rawData;
  }
  
  public List<Datum> getData()
    throws RuntimeException
  {
    _ApplicationBase Base = null;
    try
    {
      Base = _ApplicationBase.CreateApplicationBase(getState(), 
        this._RawData, getProfileID());
    }
    catch (RuntimeException e)
    {
      Extensions.debug(Extensions.hexStr(this._RawData), 24);
      throw e;
    }
    if (Base != null) {
      return Base.getData();
    }
    Extensions.debug("returning unknown datum", 2);
    return Arrays.asList(new Datum[] { new Datum("Unknown", this._RawData) });
  }
  
  public int SignalStrengthPercent()
  {
    int rssiPercent = 0;
    if (this.SignalStrength != 0) {
      if (this.SignalStrength > -80) {
        rssiPercent = (this.SignalStrength + 80) * 4 / 3 + 60;
      } else {
        rssiPercent = (this.SignalStrength + 100) * 3;
      }
    }
    return rssiPercent < 0 ? 0 : rssiPercent > 100 ? 100 : rssiPercent;
  }
  
  public int WiFiSignalStrengthPercent()
  {
    int rssiPercent = 0;
    if (this.SignalStrength != 0) {
      if (this.SignalStrength > -70) {
        rssiPercent = (this.SignalStrength + 70) * 2 + 60;
      } else {
        rssiPercent = (int)((this.SignalStrength + 84) * 4.2857D);
      }
    }
    return rssiPercent < 0 ? 0 : rssiPercent > 100 ? 100 : rssiPercent;
  }
  
  public String toString()
  {
    try
    {
      String data;
      try
      {
         data = "";
        List<Datum> datums = getData();
        for (Datum d : datums) {
          data = data + d.Description + ": " + d.Data + " ";
        }
      }
      catch (Exception e)
      {
        data = "";
        e.printStackTrace();
      }
      
      
      return MineServer.DisplayDateAsUTC(getMessageDate()) + " SensorID: " + getSensorID() + " State: " + getState() + " Voltage: " + getVoltage() + " Signal: " + SignalStrengthPercent() + " Data: " + data;
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
    return "SensorMessage toString failed.";
  }
}