
public class HumidityBase extends _ApplicationBase
{
    public static long MonnitApplicationID = 18;
    public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
    public double Hum { get; set; }
    public double Temperature { get; set; }
    public override List<Datum> Data
    {
        get { return new List<Datum> { new Datum("Humidity", Hum), new Datum("Temperature", Temperature) }; }
    }
    public static HumidityBase Create(int state, Byte[] rawData)
    {
        HumidityBase obj = new HumidityBase();
        double TemperatureTicks = Convert.ToDouble(BitConverter.ToInt16(rawData, 0));
        double HumidityTicks = Convert.ToDouble(BitConverter.ToInt16(rawData, 0 + 2));
        obj.Temperature = CalcTemperature(TemperatureTicks);
        obj.Hum = CalcHumidity(HumidityTicks, obj.Temperature);
        return obj;
    }
    #region Reference Formulas from Manufacturer
    ////----------------------------------------------------------------------------------------
    //// calculates temperature [°C] and humidity [%RH] 
    //// input extends  humi [Ticks] (12 bit) (ALREADY CONVERTED TO DECIMAL)
    ////          temp [Ticks] (14 bit) (ALREADY CONVERTED TO DECIMAL)
    //// outputextends  humi [%RH]
    ////          temp [°C]
    //void calc_sth11(float* p_humidity, float* p_temperature)
    //{
    //    const float C1 = -4.0;              // for 12 Bit
    //    const float C2 = +0.0405;           // for 12 Bit
    //    const float C3 = -0.0000028;        // for 12 Bit
    //    const float T1 = +0.01;             // for 14 Bit @ 5V
    //    const float T2 = +0.00008;           // for 14 Bit @ 5V	
    //    float rh = *p_humidity;             // rhextends      Humidity [Ticks] 12 Bit 
    //    float t = *p_temperature;           // textends       Temperature [Ticks] 14 Bit
    //    float rh_lin;                     // rh_linextends  Humidity linear
    //    float rh_true;                    // rh_trueextends Temperature compensated humidity
    //    float t_C;                        // t_C   extends  Temperature [°C]
    //    t_C = t * 0.01 - 40;                  //calc. temperature from ticks to [°C]
    //    rh_lin = C3 * rh * rh + C2 * rh + C1;     //calc. humidity from ticks to [%RH]
    //    rh_true = (t_C - 25) * (T1 + T2 * rh) + rh_lin;   //calc. temperature compensated humidity [%RH]
    //    if (rh_true > 100) rh_true = 100;       //cut if the value is outside of
    //    if (rh_true < 0.1) rh_true = 0.1;       //the physical possible range
    //    *p_temperature = t_C;               //return temperature [°C]
    //    *p_humidity = rh_true;              //return humidity[%RH]
    //}
    ////--------------------------------------------------------------------
    //float calc_dewpoint(float h, float t)
    ////--------------------------------------------------------------------
    //// calculates dew point
    //// inputextends   humidity [%RH], temperature [°C]
    //// outputextends  dew point [°C]
    //{
    //    float logEx, dew_point;
    //    logEx = 0.66077 + 7.5 * t / (237.3 + t) + (log10(h) - 2);
    //    dew_point = (logEx - 0.66077) * 237.3 / (0.66077 + 7.5 - logEx);
    //    return dew_point;
    //}
    
    public static double CalcHumidity(double humidityTicks, double temperatureC)
    {
        const double C1 = -4.0;              // for 12 Bit
        const double C2 = +0.0405;           // for 12 Bit
        const double C3 = -0.0000028;        // for 12 Bit
        const double T1 = +0.01;             // for 14 Bit @ 5V
        const double T2 = +0.00008;           // for 14 Bit @ 5V	
        double rh = humidityTicks;             // rhextends      Humidity [Ticks] 12 Bit 
        double rh_lin;                     // rh_linextends  Humidity linear
        double rh_true;                    // rh_trueextends Temperature compensated humidity
        rh_lin = C3 * rh * rh + C2 * rh + C1;     //calc. humidity from ticks to [%RH]
        rh_true = (temperatureC - 25.0) * (T1 + T2 * rh) + rh_lin;   //calc. temperature compensated humidity [%RH]
        if (rh_true > 100.0) rh_true = 100.0;       //cut if the value is outside of
        if (rh_true < 0.1) rh_true = 0.1;       //the physical possible range
        return rh_true;              //return humidity[%RH]
    }
    public static double CalcTemperature(double temperatureTicks)
    {
        return (temperatureTicks * 0.01) - 40.0; //calc. temperature from ticks to [°C]
    }
    
    public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
    public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
    public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;} }
    public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
    public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
    public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
    public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
    public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
    public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
    public static int DefaultTransmitOffset = _ApplicationBase.DefaultTransmitOffset;
    public static long DefaultHysteresis = _ApplicationBase.DefaultHysteresis;
    public static long DefaultMinimumThreshold = 6488064;
    public static long DefaultMaximumThreshold = 218775168;
    public static long DefaultCalibration1 = _ApplicationBase.DefaultCalibration1;
    public static long DefaultCalibration2 = _ApplicationBase.DefaultCalibration2;
    public static long DefaultCalibration3 = _ApplicationBase.DefaultCalibration3;
    public static long DefaultCalibration4 = _ApplicationBase.DefaultCalibration4;
    public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
    public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
    public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
    public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
    public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
    
}

