
public class Gas_COBase extends _ApplicationBase
{
    public static long MonnitApplicationID = 34;
    public static eApplicationProfileType ProfileType = eApplicationProfileType.Interval;
    public double Temp { get; set; }
    public int PPM { get; set; }
    public int PPM_TWA { get; set; }
    public override List<Datum> Data
    {
        get { return new List<Datum>() { new Datum("Temperature", Temp), new Datum("PPM", PPM), new Datum("PPM_TWA", PPM_TWA) }; }
    }
    public static Gas_COBase Create(int state, Byte[] rawData)
    {
        Gas_COBase obj = new Gas_COBase();
        obj.Temp = Convert.ToDouble((BitConverter.ToInt16(rawData, 0)) / 10.0);
        obj.PPM = BitConverter.ToUInt16(rawData, 0 + 2);
        obj.PPM_TWA = BitConverter.ToUInt16(rawData, 0 + 4);
        return obj;
    }
    
    public static long DefaultChannelMask = _ApplicationBase.DefaultChannelMask;
    public static int DefaultStandardMessageDelay = _ApplicationBase.DefaultStandardMessageDelay;
    public static int DefaultTransmitIntervalLink = _ApplicationBase.DefaultTransmitIntervalLink;
    public static int DefaultTransmitIntervalTest = _ApplicationBase.DefaultTransmitIntervalTest;
    public static int DefaultTestTransmitCount = _ApplicationBase.DefaultTestTransmitCount;
    public static int DefaultRetryCount = _ApplicationBase.DefaultRetryCount;
    public static int DefaultRecovery = _ApplicationBase.DefaultRecovery;
    public static byte[] DefaultTimeOfDayActive = _ApplicationBase.DefaultTimeOfDayActive;
    public static int DefaultMeasurementsPerTransmission = _ApplicationBase.DefaultMeasurementsPerTransmission;
    public static int DefaultTransmitOffset = 7;
    public static long DefaultHysteresis = 0x00000000;
    public static long DefaultMinimumThreshold = 0xF9C00000;
    public static long DefaultMaximumThreshold = 0x1F401932;
    public static long DefaultCalibration1 = 0x13882710;
    public static long DefaultCalibration2 = 0x3A980118;
    public static long DefaultCalibration3 = 0x00000014;
    public static long DefaultCalibration4 = 0x2B212214;
    public static int DefaultEventDetectionType = _ApplicationBase.DefaultEventDetectionType;
    public static int DefaultEventDetectionPeriod = _ApplicationBase.DefaultEventDetectionPeriod;
    public static int DefaultEventDetectionCount = _ApplicationBase.DefaultEventDetectionCount;
    public static int DefaultRearmTime = _ApplicationBase.DefaultRearmTime;
    public static int DefaultBiStable = _ApplicationBase.DefaultBiStable;
    
    public static byte[] CalibrationFrameTemperature(long sensorID, double temp)
    {
        //Calibrate Temp
        int length = 8;
        byte[] api = new Byte[length];
        api[0] = 0x74;// MSAPI.CMD_APPCMD_REQUEST;
        Array.Copy(BitConverter.GetBytes(sensorID), 0, api, 1, 4);
        api[5] = 3;//Calibration TEMP
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(temp * 10)), 0, api, 6, 2);
        return api;
    }
    public static byte[] CalibrationFrameZero(long sensorID)
    {
        //Calibrate 0 PPM
        int length = 18;
        byte[] api = new Byte[length];
        api[0] = 0x74;// MSAPI.CMD_APPCMD_REQUEST;
        Array.Copy(BitConverter.GetBytes(sensorID), 0, api, 1, 4);
        api[5] = 4;//Calibration 0ppm
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(0)), 0, api, 6, 2);
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(5)), 0, api, 8, 2);
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(5)), 0, api, 10, 2);
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(5)), 0, api, 12, 2);
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(280)), 0, api, 14, 2);
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(350)), 0, api, 16, 2);
        return api;
    }
    public static byte[] CalibrationFrameSpan(long sensorID, double span)
    {
        //Calibrate Span
        int length = 18;
        byte[] api = new Byte[length];
        api[0] = 0x74;// MSAPI.CMD_APPCMD_REQUEST;
        Array.Copy(BitConverter.GetBytes(sensorID), 0, api, 1, 4);
        api[5] = 4;//Calibration Span
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(span)), 0, api, 6, 2);
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(5)), 0, api, 8, 2);
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(5)), 0, api, 10, 2);
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(5)), 0, api, 12, 2);
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(12000)), 0, api, 14, 2);
        Array.Copy(BitConverter.GetBytes(Convert.ToUInt16(32000)), 0, api, 16, 2);
        return api;
    }
}

