package MonnitAPN;

import java.util.Date;

import BaseApplication.BitConverter;
import BaseApplication.ExtensionMethods.Extensions;
import EventHandling.Event;
import MonnitAPN.MNPEventArgs.*;

public class MSP
{
    //This was moved here to put this under control of MNP code, not MonnetAPN
    public static byte[] CryptSTKey = { (byte) 0x02, (byte) 0x04, (byte) 0x08, (byte) 0x16, (byte) 0x32, (byte) 0x64, (byte) 0x28, (byte) 0x56, (byte) 0x12, (byte) 0x24, (byte) 0x48, (byte) 0x96, (byte) 0x92, (byte) 0x84, (byte) 0x68, (byte) 0x36 };
    public static byte[] CryptGLKey = { (byte) 0x7C, (byte) 0x5D, (byte) 0x64, (byte) 0x2E, (byte) 0x60, (byte) 0x69, (byte) 0x4D, (byte) 0xD8, (byte) 0xFA, (byte) 0x97, (byte) 0x1F, (byte) 0x78, (byte) 0x13, (byte) 0x03, (byte) 0x72, (byte) 0x3D };
    public static byte[] CryptCLKey = { (byte) 0x01, (byte) 0x12, (byte) 0x23, (byte) 0x34, (byte) 0x45, (byte) 0x56, (byte) 0x67, (byte) 0x78, (byte) 0x89, (byte) 0x9A, (byte) 0xAB, (byte) 0xBC, (byte) 0xCD, (byte) 0xDE, (byte) 0xEF, (byte) 0xF0 };
   

    //APN/REN/EDN Supported Message Types
    public final byte MTYPE_ACK     = (byte) 0x00;
    public final byte MTYPE_ERROR   = (byte) 0xFF;
    public final byte MTYPE_RDATA   = (byte) 0x01;
    public final byte MTYPE_WDATA   = (byte) 0x02;
    public final byte MTYPE_SCFG    = (byte) 0x08;
        
    //APN Supported
    public final byte MTYPE_STIME   = (byte) 0x10;
    public final byte MTYPE_SMODE   = (byte) 0x11;
    public final byte MTYPE_WMSG    = (byte) 0x12;
    public final byte MTYPE_MDF     = (byte) 0x27;
    public final byte MTYPE_STN     = (byte) 0x29;

    //REN/EDN Supported
    public final byte MTYPE_ACTL    = (byte) 0x33;
    public final byte MTYPE_WCODE   = (byte) 0x45;
    public final byte MTYPE_SOM     = (byte) 0x50;
    public final byte MTYPE_SDM     = (byte) 0x51;

    public final byte MSPTYPE_MNP   = (byte) 0xA1;
    public final byte MSPTYPE_RFMNP = (byte) 0xA5;

     
    public byte[] msp = new byte[64];
    private int MNPIdx;
        
    public MSP() { }
    public MSP(byte[] indata) throws Exception 
    {
        System.arraycopy(indata, 0, msp, 0, msp.length);
        SetMNPIdx();
    }
    public MSP(byte Type, long SID, int NDesc, byte[] mnppart) throws Exception
    {
        //START OF MSP----------------------------------------------------------------------
        msp[0] = Type;                                  //MSP Payload Type
        msp[1] = (byte)(mnppart.length + 8);            //MSP Payload Length

        SetMNPIdx();

        //START OF MNP------------------------------------------------------------------
        msp[MNPIdx+1] = (byte)(mnppart.length - 1);                     //MNP Payload Length (mnp comes in with mType attached)
        System.arraycopy(BitConverter.getBytes(SID), 0, msp, MNPIdx+2, 4);    //MNP SID
        System.arraycopy(BitConverter.getBytes(NDesc), 0, msp, MNPIdx+6, 2);  //MNP NodeDesc
        System.arraycopy(mnppart, 0, msp, MNPIdx+8, mnppart.length);          //MNP MType + Payload 
        msp[MNPIdx] = MSP_ReturnCRC();                                  //MNP CRC-8
    }
    private void SetMNPIdx() throws Exception
    {
        if (msp[0] == (byte) 0xA1) MNPIdx = 2;
        else if (msp[0] == (byte) 0xA5) MNPIdx = 4;
        else throw new Exception("MSP Type Not Supported");
    }
    public Boolean isSpecialSetTimeResponse()
    {
        if (msp[MNPIdx + 8] == MTYPE_ACK)
        {
            if (msp[MNPIdx + 9] == MTYPE_STIME)
            {
                return true;
            }
        }
        return false;
    }
    public Boolean isSpuriousMessage()
    {
        if ((msp[MNPIdx + 8] == MTYPE_SDM) || (msp[MNPIdx + 8] == MTYPE_SOM))
            return true;
        return false;
    }
    public Boolean isMDFMessage() throws Exception
    {

        if ((msgID() == GlobalAPN.getSID()) && (msp[MNPIdx + 8] == MTYPE_MDF))
        {
            if (MDFmsgID() == 17) return false;
            else return true;
        }
        return false;
    }
    public Boolean isAck2Command(MSP cmd)
    {
        if(msp[MNPIdx + 8] == MTYPE_ERROR) return true; //Allow Dequeue
        if (msp[MNPIdx + 10] == 255) return true;//TODO: make sure this is right Allow Dequeue error writing 
        if(msp[MNPIdx + 8] != MTYPE_ACK) return false;  //Do not allow Dequeue
        if(msp[MNPIdx + 9] != cmd.msp[2 + 8]) return false; //Not the same major command
        switch (msp[MNPIdx+9]) //ACK CMD TYPE
        {
            case MTYPE_RDATA:
            case MTYPE_WDATA:
            case MTYPE_SCFG:
            case MTYPE_WCODE:
            case MTYPE_ACTL:
                if (msp[MNPIdx + 10] == cmd.msp[2 + 9]) return true; //Same Action
                else return false;
            case MTYPE_STIME:
            case MTYPE_SMODE:
            case MTYPE_WMSG:
            default:
                return true; //On failure to decode, just let it slide
        }
    }
    public String MSP_TX_SHandler() throws Exception
    {
        String s = StartNaming(true);

        //Based on Message type
        switch (msp[MNPIdx + 8])
        {
            case MTYPE_RDATA:
                s += "Request Read (Sector-" + msp[MNPIdx + 9] + ")";
                break;
            case MTYPE_WDATA:
                s += "Request Write (Sector-" + msp[MNPIdx + 9] + ")";
                break;
            case MTYPE_SCFG:
                s += "Secure Transfer ";
                if (msp[MNPIdx + 9] == 0) s += "(Link Provisioning)";
                else if (msp[MNPIdx + 9] == 1) s += "(Device Provisioning)";
                else s += "(Gloabal Provisioning)";
                break;
            case MTYPE_STIME:
                s += "Set APN time to - " + (new Date().getTime()) + " (code " + BitConverter.toInt16(msp, MNPIdx + 10);
                s += (msp[MNPIdx + 9] == 1) ? "PM)" : "AM)";
                break;
            case MTYPE_SMODE:
                if (msp[MNPIdx + 9] == 0) s += "APN Mode --> IDLE";
                else if (msp[MNPIdx + 9] == 1) s += "APN Mode --> ACTIVE";
                else s += "APN Mode --> TEST";
                break;
            case MTYPE_WMSG:
                if (msp[MNPIdx + 9] == 1)
                    s += "Sending Waiting Message for " + BitConverter.toInt32(msp, MNPIdx + 10);
                else
                    s += "Sending No Waiting Message for " + BitConverter.toInt32(msp, MNPIdx + 10);
                break;
            case MTYPE_ACTL:
                if (msp[MNPIdx + 9] == 0)
                {
                    if (msp[MNPIdx + 10] == 0) s += "Activate";
                    else s += "Deactivate";
                    s += " Sensor";
                }
                else if (msp[MNPIdx + 9] == 1)
                {
                    if (msp[MNPIdx + 10] == 0) s += "Deactivate";
                    else s += "Activate";
                    s += " Test Mode";
                }
                else if (msp[MNPIdx + 9] == 2)
                {
                    if (msp[MNPIdx + 10] == 0) s += "Deactivate";
                    else s += "Activate";
                    s += " Aware Mode";
                }
                else s += "Unknown ACTL Command";
                break;
            default:
                s = "";
                break;
        }

        return s;
    }
    public String MSP_RX_SHandler() throws Exception
    {
        //Analyse & CRC for correctness
        //if(msp[2] != MSP_ReturnCRC()) return "";
        if (msp[MNPIdx] != MSP_ReturnCRC()) return "";

        //String s = "RX[0x" + BitConverter.toUInt32(msp, MNPIdx+2).ToString("X8") + "]: ";
        String s = StartNaming(false);

        int rssiInbound = (msp[2]);
        if (rssiInbound > 128) rssiInbound -= 256;//rssi of EDN talking to APN
                
        //Based on Message type
        switch (msp[MNPIdx+8])
        {
            case MTYPE_ACK: //  Acknowledge
                switch (msp[MNPIdx+9])
                {
                    case MTYPE_RDATA:
                        s += "Read Sector- " + msp[MNPIdx + 10] + ": ";
                        if ((msp[MNPIdx + 10] == 0) || (msp[MNPIdx + 10] == 1) || (msp[MNPIdx + 10] == 2) || (msp[MNPIdx + 10] == 3))
                            s += Extensions.byteToAsciiString(msp, MNPIdx + 11, MNPIdx + 11 + 32);
                        else s += MSP_HexPrint(MNPIdx + 11, MNPIdx + 11 + 32);

                        if (msp[MNPIdx + 10] == 10) GlobalAPN.setCID(BitConverter.toInt32(msp, MNPIdx + 15));

                        try
                        {
                            byte[] data = new byte[32];
                            System.arraycopy(msp, MNPIdx + 11, data, 0, 32);
                            OnReadDataAck((int)(msgID()), (int)(msp[MNPIdx + 10]), data);
                        }
                        catch (Exception e)
                        {
                            s += String.format("\r\nError Reporting Sector Read - SID:{0} Sector:{1}", msgID(), msp[MNPIdx + 10]);
                            MonnitAPN.WriteExceptionToLog(this, e);
                        }
                        break;
                    case MTYPE_WDATA:
                        s += "Written Sector- " + msp[MNPIdx + 10] + ": ";
                        try
                        {
                            OnWriteDataAck((int)(msgID()), (int)(msp[MNPIdx + 10]));
                        }
                        catch (Exception e)
                        {
                            s += String.format("\r\nError Reporting Sector Written - SID:{0} Sector:{1}", msgID(), msp[MNPIdx + 10]);
                            MonnitAPN.WriteExceptionToLog(this, e);
                        }
                        break;
                    case MTYPE_SCFG:
                        s += "Secure Transfer Ack ";
                        if (msp[MNPIdx + 10] == 0) s += "(Link Provisioning)";
                        else if (msp[MNPIdx + 10] == 1) s += "(Device Provisioning)";
                        else s += "(Gloabal Provisioning)";
                        break;
                    case MTYPE_STIME:
                        GlobalAPN.setSID(BitConverter.toInt32(msp, MNPIdx + 2));
                        s += "APN Time Set";
                        try
                        {
                            OnTimeSetAck(GlobalAPN.getSID());
                        }
                        catch (Exception e)
                        {
                            s += String.format("\r\nError Reporting Time Set - SID:{0}", msgID());
                            MonnitAPN.WriteExceptionToLog(this, e);
                        }
                        break;
                    case MTYPE_SMODE:
                        if (msp[MNPIdx + 10] >= 253) s += "APN Mode Request Failed! (" + msp[MNPIdx + 10] + ")";
                        else s += "APN Mode Changed";
                        try
                        {
                            OnModeSetAck((int)(msp[MNPIdx + 10]));
                        }
                        catch (Exception e)
                        {
                            s += String.format("\r\nError Reporting Mode Set - Mode:{0}", msp[MNPIdx + 10]);
                            MonnitAPN.WriteExceptionToLog(this, e);
                        }
                        break;
                    case MTYPE_WMSG:
                        s += String.format("Waiting Message {0}SUCCESSFULLY enqueued.", msp[MNPIdx + 10] == 0 ? "FAILED to be " : "has been ");
                        break;
                    case MTYPE_ACTL:
                        s += String.format("ACTL change {0}", msp[MNPIdx + 11] == 0 ? "successful" : "failed!");
                        try
                        {
                            if (msp[MNPIdx + 11] == 0)
                            {
                                OnACTLAck((msgID()), 0);
                            }
                        }
                        catch (Exception e)
                        {
                            s += String.format("\r\nError Reporting Sector Read - SID:{0} Sector:{1}", msgID(), msp[MNPIdx + 10]);
                            MonnitAPN.WriteExceptionToLog(this, e);
                        } 
                        break;
                    case MTYPE_WCODE:
                        s += "Write Code Sector: " + msp[MNPIdx + 10];
                        break;
                    default:
                        s = "";
                        return s;
                }
                break;
            
            
            case MTYPE_STN: //  Spurious Traffic Notification
                s += "Spurious Traffic Notification Was Received" + Integer.toHexString(msp[MNPIdx + 8]);
                break;
                           
            case MTYPE_SOM:
                s += "Orphan Message- CNTR: " + BitConverter.toInt16(msp, MNPIdx + 9);
                s += ", CCID: " + BitConverter.toInt32(msp, MNPIdx + 11);
                s += ", PWR: " + msp[MNPIdx + 15];
                s += ", STATE: " + msp[MNPIdx + 16];
                if ((msp[MNPIdx + 16] & (byte) 0x01) == (byte) 0x01) s += " Test Mode Active";
                if ((msp[MNPIdx + 16] & (byte) 0x02) == (byte) 0x02) s += " Aware Mode Active";
                break;
            case MTYPE_SDM:
                
                long SensorID = BitConverter.toInt32(msp, MNPIdx + 2);
                int Voltage = (int)msp[MNPIdx + 11];
                int rssiOutbound = (int)(msp[MNPIdx + 12]);//rssi of APN talking to EDN (Last message sent)
                if (rssiOutbound > 128) rssiOutbound -= 256;
                //int LinkQuality = BitConverter.toInt32(msp[MNPIdx + 13]);
                int State = (int)(msp[MNPIdx + 14]);
                int NodeDesc = (int)(BitConverter.toInt16(msp, MNPIdx + 6));
                Date MessageDate = new Date(System.currentTimeMillis());

                //Length may not be perfect
                byte[] Data = new byte[((byte) msp[1]) - 17];
                System.arraycopy(msp, MNPIdx + 15, Data, 0, Data.length);
                
                
                s += "SDM- CNTR: " + BitConverter.toInt16(msp, MNPIdx + 9);
                s += ", PWR: " + Voltage;
                s += ", RSSI: " + rssiInbound;
                s += ", STS: " + State;
                s += ", Data: " + BitConverter.toString(Data).replace("-", "");
                
                try
                {
                    OnProcessData(SensorID, Voltage, rssiInbound, rssiOutbound, State, GlobalAPN.getSID(), Data, MessageDate);
                }
                catch (Exception e)
                {
                    s = "ERROR processing SDM: " + s;
                    MonnitAPN.WriteInfoToLog(this, "ERROR SDM: " + MSP_HexPrint(0, msp.length));
                    MonnitAPN.WriteExceptionToLog(this, e);
                }
                break;
            case MTYPE_MDF:
                s += "Message Delivery Failure for " + MDFmsgID();
                break;
            default:
                s += "Unknown Message Type" + Integer.toHexString(msp[MNPIdx + 8]);
                break;
        }

        if (msp[0] == MSPTYPE_RFMNP)
        {
            s += " (" + rssiInbound + ")";
        }

        return s;
    }


    public Event<DataMessageEventArgs> ProcessData;
    public Event<WriteConfigEventArgs> WriteDataAck;
    public Event<ReadConfigEventArgs> ReadDataAck;
    public Event<SensorIDEventArgs> TimeSetAck;
    public Event<ModeSetEventArgs> ModeSetAck;
    public Event<ActionCtrlEventArgs> ACTLAck;
    protected void OnProcessData(long sensorID, double voltage, int rssiInbound, int rssiOutbound, int state, long gatewayID, byte[] data, Date messageDate)
    {
        if (ProcessData != null)
            ProcessData(this, new DataMessageEventArgs(sensorID, voltage, rssiInbound, rssiOutbound, state, gatewayID, data, messageDate));
    }
    protected void OnWriteDataAck(long sensorID, int configSection)
    {
        if (WriteDataAck != null)
            WriteDataAck(this, new WriteConfigEventArgs(sensorID, configSection));
    }
    protected void OnReadDataAck(long sensorID, int configSection, byte[] data)
    {
        if (ReadDataAck != null)
            ReadDataAck(this, new ReadConfigEventArgs(sensorID, configSection, data));
    }
    protected void OnTimeSetAck(long sensorID)
    {
        if (TimeSetAck != null)
            TimeSetAck(this, new SensorIDEventArgs(sensorID));
    }
    protected void OnModeSetAck(int mode)
    {
        if (ModeSetAck != null)
            ModeSetAck(this, new ModeSetEventArgs(mode));
    }
    protected void OnACTLAck(long sensorID, int actionCommand)
    {
        if(ACTLAck != null)
            ACTLAck(this, new ActionCtrlEventArgs(sensorID, actionCommand, true, new byte[0]));
    }
    
    public int msgID() throws Exception
    {
        return BitConverter.toInt32(msp, MNPIdx + 2);
    }
    public int MDFmsgID() throws Exception
    {
        return BitConverter.toInt32(msp, MNPIdx + 9);
    }
    public int msgND() throws Exception
    {
        return BitConverter.toInt16(msp, MNPIdx + 6);
    }
    private String StartNaming(Boolean isTX) throws Exception
    {
        String s = new Date().toString();
        if (isTX) s += "TX[";
        else s += "RX[";
        s += msgID() + "] ";

        //Name device based on ID
        //s += MonnitApplicationBase.ApplicationName(BitConverter.toUInt16(msp, MNPIdx + 6));
        s += "[" + msgND() + "]: ";
        return s;
    }
    public byte MSP_ReturnCRC()
    {
        byte crc = 0;
        for (int i = MNPIdx + 1; i <= msp[1] + MNPIdx - 1; i++) crc = CRC8.AddStep2CRC(crc, msp[i]);
        return crc;
    }
    private String MSP_HexPrint(int idx, int end)
    {
        String s = "0x";
        while (idx < end)
        {
            s += Integer.toHexString(msp[idx] / 16) + Integer.toHexString(msp[idx] % 16) + " ";
            idx++;
        }
        return s;
    }
    public static byte[] GenerateAESIV(byte help1, byte help2)
    {
        byte[] iv = new byte[16];
        iv[0] = CRC8.AddStep2CRC(help1, help2);
        iv[1] = CRC8.AddStep2CRC(help2, iv[0]);
        for (int i = 2; i < 16; i++) iv[i] = CRC8.AddStep2CRC(iv[i - 2], iv[i - 1]);
        return iv;
    }
}